/*global define*/

define('logging/CoreLogging',[], function () {
    return function(message) {
        if(typeof message == 'string' || message instanceof String) {
            console.log('Application: '+message);
        } else {
            console.log('Application: ',message);
        }
    };
});

/**
 * Backbone localStorage Adapter
 * Version 1.1.16
 *
 * https://github.com/jeromegn/Backbone.localStorage
 */
(function (root, factory) {
  if (typeof exports === 'object' && typeof require === 'function') {
    module.exports = factory(require("backbone"));
  } else if (typeof define === "function" && define.amd) {
    // AMD. Register as an anonymous module.
    define('backbonels',["backbone"], function(Backbone) {
      // Use global variables if the locals are undefined.
      return factory(Backbone || root.Backbone);
    });
  } else {
    factory(Backbone);
  }
}(this, function(Backbone) {
// A simple module to replace `Backbone.sync` with *localStorage*-based
// persistence. Models are given GUIDS, and saved into a JSON object. Simple
// as that.

// Generate four random hex digits.
function S4() {
   return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
};

// Generate a pseudo-GUID by concatenating random hexadecimal.
function guid() {
   return (S4()+S4()+"-"+S4()+"-"+S4()+"-"+S4()+"-"+S4()+S4()+S4());
};

function isObject(item) {
  return item === Object(item);
}

function contains(array, item) {
  var i = array.length;
  while (i--) if (array[i] === item) return true;
  return false;
}

function extend(obj, props) {
  for (var key in props) obj[key] = props[key]
  return obj;
}

function result(object, property) {
    if (object == null) return void 0;
    var value = object[property];
    return (typeof value === 'function') ? object[property]() : value;
}

// Our Store is represented by a single JS object in *localStorage*. Create it
// with a meaningful name, like the name you'd give a table.
// window.Store is deprectated, use Backbone.LocalStorage instead
Backbone.LocalStorage = window.Store = function(name, serializer) {
  if( !this.localStorage ) {
    throw "Backbone.localStorage: Environment does not support localStorage."
  }
  this.name = name;
  this.serializer = serializer || {
    serialize: function(item) {
      return isObject(item) ? JSON.stringify(item) : item;
    },
    // fix for "illegal access" error on Android when JSON.parse is passed null
    deserialize: function (data) {
      return data && JSON.parse(data);
    }
  };
  var store = this.localStorage().getItem(this.name);
  this.records = (store && store.split(",")) || [];
};

extend(Backbone.LocalStorage.prototype, {

  // Save the current state of the **Store** to *localStorage*.
  save: function() {
    this.localStorage().setItem(this.name, this.records.join(","));
  },

  // Add a model, giving it a (hopefully)-unique GUID, if it doesn't already
  // have an id of it's own.
  create: function(model) {
    if (!model.id && model.id !== 0) {
      model.id = guid();
      model.set(model.idAttribute, model.id);
    }
    this.localStorage().setItem(this._itemName(model.id), this.serializer.serialize(model));
    this.records.push(model.id.toString());
    this.save();
    return this.find(model);
  },

  // Update a model by replacing its copy in `this.data`.
  update: function(model) {
    this.localStorage().setItem(this._itemName(model.id), this.serializer.serialize(model));
    var modelId = model.id.toString();
    if (!contains(this.records, modelId)) {
      this.records.push(modelId);
      this.save();
    }
    return this.find(model);
  },

  // Retrieve a model from `this.data` by id.
  find: function(model) {
    return this.serializer.deserialize(this.localStorage().getItem(this._itemName(model.id)));
  },

  // Return the array of all models currently in storage.
  findAll: function() {
    var result = [];
    for (var i = 0, id, data; i < this.records.length; i++) {
      id = this.records[i];
      data = this.serializer.deserialize(this.localStorage().getItem(this._itemName(id)));
      if (data != null) result.push(data);
    }
    return result;
  },

  // Delete a model from `this.data`, returning it.
  destroy: function(model) {
    this.localStorage().removeItem(this._itemName(model.id));
    var modelId = model.id.toString();
    for (var i = 0, id; i < this.records.length; i++) {
      if (this.records[i] === modelId) {
        this.records.splice(i, 1);
      }
    }
    this.save();
    return model;
  },

  localStorage: function() {
    return localStorage;
  },

  // Clear localStorage for specific collection.
  _clear: function() {
    var local = this.localStorage(),
      itemRe = new RegExp("^" + this.name + "-");

    // Remove id-tracking item (e.g., "foo").
    local.removeItem(this.name);

    // Match all data items (e.g., "foo-ID") and remove.
    for (var k in local) {
      if (itemRe.test(k)) {
        local.removeItem(k);
      }
    }

    this.records.length = 0;
  },

  // Size of localStorage.
  _storageSize: function() {
    return this.localStorage().length;
  },

  _itemName: function(id) {
    return this.name+"-"+id;
  }

});

// localSync delegate to the model or collection's
// *localStorage* property, which should be an instance of `Store`.
// window.Store.sync and Backbone.localSync is deprecated, use Backbone.LocalStorage.sync instead
Backbone.LocalStorage.sync = window.Store.sync = Backbone.localSync = function(method, model, options) {
  var store = result(model, 'localStorage') || result(model.collection, 'localStorage');

  var resp, errorMessage;
  //If $ is having Deferred - use it.
  var syncDfd = Backbone.$ ?
    (Backbone.$.Deferred && Backbone.$.Deferred()) :
    (Backbone.Deferred && Backbone.Deferred());

  try {

    switch (method) {
      case "read":
        resp = model.id != undefined ? store.find(model) : store.findAll();
        break;
      case "create":
        resp = store.create(model);
        break;
      case "update":
        resp = store.update(model);
        break;
      case "delete":
        resp = store.destroy(model);
        break;
    }

  } catch(error) {
    if (error.code === 22 && store._storageSize() === 0)
      errorMessage = "Private browsing is unsupported";
    else
      errorMessage = error.message;
  }

  if (resp) {
    if (options && options.success) {
      if (Backbone.VERSION === "0.9.10") {
        options.success(model, resp, options);
      } else {
        options.success(resp);
      }
    }
    if (syncDfd) {
      syncDfd.resolve(resp);
    }

  } else {
    errorMessage = errorMessage ? errorMessage
                                : "Record Not Found";

    if (options && options.error)
      if (Backbone.VERSION === "0.9.10") {
        options.error(model, errorMessage, options);
      } else {
        options.error(errorMessage);
      }

    if (syncDfd)
      syncDfd.reject(errorMessage);
  }

  // add compatibility with $.ajax
  // always execute callback for success and error
  if (options && options.complete) options.complete(resp);

  return syncDfd && syncDfd.promise();
};

Backbone.ajaxSync = Backbone.sync;

Backbone.getSyncMethod = function(model, options) {
  var forceAjaxSync = options && options.ajaxSync;

  if(!forceAjaxSync && (result(model, 'localStorage') || result(model.collection, 'localStorage'))) {
    return Backbone.localSync;
  }

  return Backbone.ajaxSync;
};

// Override 'Backbone.sync' to default to localSync,
// the original 'Backbone.sync' is still available in 'Backbone.ajaxSync'
Backbone.sync = function(method, model, options) {
  return Backbone.getSyncMethod(model, options).apply(this, [method, model, options]);
};

return Backbone.LocalStorage;
}));

/*global define*/

define('storage/StorageHandler',[
    'jquery',
    'underscore',
    'backbone',
    'backbonels',
    'logging/CoreLogging'
], function ($, _, Backbone, clog) {
    'use strict';
    return Backbone.Collection.extend({

        model: Backbone.Model,
        localStorage: new Backbone.LocalStorage("default-storage-data"),

        persistToStorage: function(key,value) {
            this.add({
                id:key,
                value:value
            },{merge: true});
            this.save({id:key});
        },

        loadFromStorage: function(key) {
            return this.where({id:key})[0].attributes.value;
        },

        addFromFile: function(filePath) {
            var self = this;
            $.getJSON(filePath, function (data) {
                self.add(data);
                self.trigger('storagehandler:filedata:added');
            });
        },

        save: function(queryParams) {
            if(!queryParams) {
                this.models.forEach(function(model){
                    model.save();
                });
            } else {
                this.where(queryParams)[0].save();
            }
        }

    });
});

define('events/EventChannel',['backbone', 'underscore'], function (Backbone, _) {
    var channel = _.extend({}, Backbone.Events); 
    return channel;
});
/*global define*/

define('factory/TemplateFactory',[
    'jquery',
    'underscore',
    'backbone',
    'handlebars',
    'logging/CoreLogging'
], function ($, _, Backbone, JST, clog) {
    'use strict';
    return Backbone.Model.extend({

        templateCache : {},

        _toCache: function(templateKey, template) {
            this.templateCache[templateKey] = template;
        },

        _fromCache: function(templateKey) {
            return this.templateCache[templateKey];
        },

        retrieveTemplate : function(templateKey, baseUrl) {

            var self = this;
            var currentTemplate = null;

            /*
             * try to load from cache
             */
            currentTemplate = self._fromCache(templateKey);
            if(currentTemplate!=null&&currentTemplate!=undefined) {
                self._toCache(templateKey,currentTemplate);
                return currentTemplate;
            }

            /*
             * if not cached, try to load by JST
             */
            try {
                currentTemplate = JST[templateKey]();
                if(currentTemplate!=null&&currentTemplate!=undefined) {
                    self._toCache(templateKey,currentTemplate);
                    return currentTemplate;
                }
            } catch (e) {
                currentTemplate = null;
            }
            
            if(!baseUrl) {
                baseUrl = './scripts/templates/';
            }

            /*
             * if failed, try to load from file
             */
            var currentTemplate;
            $.ajax({
                url: baseUrl+templateKey+'.hbs',
                method: 'GET',
                async: false,
                success: function(data) {
                    currentTemplate = data;
                },
                error: function() {
                    currentTemplate = null;
                }
            });
            if(currentTemplate!=null&&currentTemplate!=undefined) {
                self._toCache(templateKey,currentTemplate);
                return currentTemplate;
            }

        }

    });
});

/*global define*/

define('Application',[
    'jquery',
    'underscore',
    'backbone',
    'i18next',
    'logging/CoreLogging',
    'storage/StorageHandler',
    'events/EventChannel',
    'factory/TemplateFactory'
], function ($, _, Backbone, i18n, clog, StorageHandler, eventChannel, TemplateFactory) {
    'use strict';
    return Backbone.Model.extend({
        
        clog: clog,

        /*
         * Router Implementation
         */
        applicationRouter: null,

        /*
         * Storage-Handler Implementation
         */
        storageHandler: null,

        /*
         * Cached Template-Factory
         */
        templateFactory: null,

        /*
         * Cached Request-Factory
         */
        requestFactory: null,

        /*
         * Application View
         */
        applicationView: null,

        /*
         * Application Configuration
         */
        configuration: null,
        configurationId: null,
        
        buildCallback: null,
        
        postBuildSetup: function(options) {
            if(options.applicationRouter) {
                this.applicationRouter = options.applicationRouter;
                this.applicationRouter.appInstance = this;
            }
            if(options.templateFactory) {
                this.templateFactory = options.templateFactory;
            } else {
                this.templateFactory = new TemplateFactory();
            }
            if(options.applicationView) {
                this.applicationView = options.applicationView;
                this.applicationView.templateFactory = this.templateFactory;
            }
            if(options.requestFactory) {
                this.requestFactory = options.requestFactory;
            }
        },

        build: function(options,buildCallback) {
            var self = this;
            this.buildCallback = buildCallback;
            if(this.buildCallback) {
                Backbone.listenTo(this,'application:initialize:finished',this.buildCallback);
            }
            if(options.configurationIdentifier) {
                self.configurationId = options.configurationIdentifier;
            }
            self.storageHandler = new StorageHandler();
            self.listenToOnce(self.storageHandler,'sync',function(){
                self._storageLoaded(options);
            });
            self.storageHandler.fetch();
            eventChannel.on('app:busy:false',function(){
                if(self.applicationView) {
                    self.applicationView.hideLoadingIndicator();
                }
            });
            eventChannel.on('app:busy:true',function(){
                if(self.applicationView) {
                    self.applicationView.showLoadingIndicator();
                }
            });
            $(document).keypress(function (e) {
              if (e.keyCode == '13') {
                e.preventDefault();
                e.stopPropagation();
                return false;
              }
            });
        },

        saveConfiguration: function() {
            this.storageHandler.save(self.configurationId);
        },

        loadNameSpace:function(namespaces,callBack){
            i18n.loadNamespaces(namespaces, function() {
                if (callBack!=null && callBack != undefined){
                    callBack();
                }
            });
        },

        setLanguage: function (lang) {
            var self = this;
            this.data.langParameters.selectedLang = lang;
            i18n.setLng(this.data.langParameters.selectedLang, function () {
                clog("done loading new language");
                self.applicationView.render();
            });
        },

        /*
         * Internal
         */

        _storageLoaded: function(options) {
            if(this.configurationId) {
                var configurations = this.storageHandler.where(options.configurationIdentifier);
                if(configurations&&configurations.length>0) {
                    this._loadExistingConfiguration(configurations);
                } else {
                    this._loadInitialConfiguration(options.configurationPath,options.configurationIdentifier);
                }
            } else {
                this.trigger('application:initialize:finished');
            }
        },

        _loadInitialConfiguration: function(configPath,configId) {
            var self = this;
            self.listenTo(self.storageHandler,'storagehandler:filedata:added',function(){
                clog('Loading Initial Configuration');
                self.storageHandler.save(configId);
                var configurations = self.storageHandler.where(configId);
                self.configuration = configurations[0].attributes;
                self.trigger('application:initialize:finished');
            });
            self.storageHandler.addFromFile(configPath);
        },

        _loadExistingConfiguration: function(configurations) {
            var self = this;
            clog('Loading Existing Configuration');
            self.configuration = configurations[0].attributes;
            self.trigger('application:initialize:finished');
        }

    });
});

/*global define*/

define('view/ApplicationView',[
    'jquery',
    'underscore',
    'backbone',
    'logging/CoreLogging'
], function ($, _, Backbone, clog) {
    'use strict';
    return Backbone.View.extend({

        el: '#appContainer',

        templateFactory: null,
        headerView: null,
        contentView: null,
        bottomView: null,
        
        bottomViewHidden: true,
        showInfoButton: true,
 
        loadingIndicatorInitialized: false,
        loadingIndicatorImage: null,
        
        getContentHeight: function() {
            return (window.innerHeight)-$('#appHeader').height()-$('#appBottom').height();
        },
 
        initialize:function(options){
            clog("ApplicationView initialized");
            if(options&&options.headerView) {
                this.changeHeaderView(options.headerView);
            }
            if(options&&options.bottomView) {
                this.changeBottomView(options.bottomView);
            }
            if(options.loadingIndicatorImage) {
                this.loadingIndicatorImage = options.loadingIndicatorImage;
            }
        },
        
        hideBottomView: function() {
            this.bottomViewHidden = true;
        },
        
        showBottomView: function() {
            this.bottomViewHidden = false;
        },
        
        showLoadingIndicator: function () {
            if (app.embeddedMode) {
                setTimeout(function () {
                    parent.postMessage("SHOW_SPINNER", "*");
                }, 100);
            } else {
                this.$el.find('#loading-indicator').show();
            }
        },
        
        hideLoadingIndicator: function() {
            if (app.embeddedMode) {
                setTimeout(function () {
                    parent.postMessage("HIDE_SPINNER", "*");
                }, 100);
            } else {
                this.$el.find('#loading-indicator').hide();
            }
        },

        render: function(){
            clog('onRenderApplicationView:Start');
            if(this.loadingIndicatorInitialized==false&&this.loadingIndicatorImage) {
                this.$el.append('<div id="loading-indicator" class="loader" style="display:none; z-index: 100000;"/>');
            }
            this.loadingIndicatorInitialized = true;
            if(this.headerView!=null) {
                this.headerView.setTemplateFacoryIfNull(this.templateFactory);
                this.$el.find('#appHeader').html(this.headerView.$el);
                this.headerView.render();
                this.headerView.delegateEvents();
                if(this.height) {
                    this.$el.find('#appHeader').css('height',this.height)
                }
                if(!this.showInfoButton) {
            		$('#infoClick').parent()[0].removeChild($('#infoClick')[0]);
            	}
            }
            
            if(this.bottomView!=null) {
                this.bottomView.setTemplateFacoryIfNull(this.templateFactory);
                this.$el.find('#appBottom').html(this.bottomView.$el);
                if(this.bottomView&&this.bottomViewHidden) {
                    this.bottomView.$el.hide();
                    this.lastBottomHeight = this.$el.find('#appBottom').height();
                    this.$el.find('#appBottom').css('height','0px');
                    this.$el.find('#appContent').css('bottom','0px');
                } else {
                  	if (this.lastBottomHeight > 0) {
                    	this.$el.find('#appBottom').css('height',this.lastBottomHeight+'px');
                    	this.$el.find('#appContent').css('bottom',this.lastBottomHeight+'px');
                  	} else {
						this.$el.find('#appBottom').css('height','60px');
						this.$el.find('#appContent').css('bottom','60px');
                  	}
                    this.bottomView.$el.show();
                }
                this.bottomView.render();
                this.bottomView.delegateEvents();
            }

            if(this.contentView!=null) {
                //this.contentView._animated = true;
                this.contentView.$el.attr("id","innerContainer");
                this.$el.find('#appContent').html(this.contentView.$el);
                this.contentView.render();
                this.contentView.delegateEvents();
                if(this.contentView.niceScroll) {
                    this.$el.find('#appContent').niceScroll({
                        hwacceleration: true,
                        overflowx: false, 
                        enabletranslate3d: true,
                        touchbehavior: true, 
                        bouncescroll: false
                    });
                    this.$el.find('#appContent').getNiceScroll().resize();
                } else {
                    this.$el.find('#appContent').getNiceScroll().remove();
                }
                
            }
  
            
            clog('onRenderApplicationView:End');
        },
        
        changeBottomView: function(view, model, update) {
            if(view&&this.bottomView) {
                try {
                    this.bottomView.close();
                } catch(e) {
                    this.bottomView.undelegateEvents();
                    this.bottomView.remove();
                    this.bottomView.unbind();
                }
            }
            if(view) {
                this.bottomView = view;
            }
            if(model) {
                this.bottomView.model = model;
            }
        },

        changeHeaderView: function(view, model, update) {
            if(view&&this.headerView) {
                try {
                    this.headerView.close();
                } catch(e) {
                    this.headerView.undelegateEvents();
                    this.headerView.remove();
                    this.headerView.unbind();
                }
            }
            if(view) {
                this.headerView = view;
            }
            if(model) {
                this.headerView.model = model;
            }
        },

        changeContentView: function (view) {
            var self = this;
            if (this.contentView != null) {
                this.contentView.close(function () {
                    self.contentView = view;
                    self.contentView.setTemplateFacoryIfNull(self.templateFactory);
                    self.render();
                });
            } else {
                this.contentView = view;
                this.contentView.setTemplateFacoryIfNull(this.templateFactory);
                this.render();
            }
        }
       
    });
});



/*global define*/

define('view/BaseView',[
    'jquery',
    'underscore',
    'backbone',
    'logging/CoreLogging',
    'events/EventChannel'
], function ($, _, Backbone, clog, eventChannel) {
    'use strict';
    return Backbone.View.extend({

        _templateFactory: null,
        _applicationRouter: null,
        _animated: false,

        template: null,
        hbsTemplate: null,
        hbsStringTemplate: null,
        alternateTemplatePath: null,
        clickRoutes: null,
        
        addClasses: null,
        addClassesSelector: null,
        
        dataLoaded: false,
        loadData: null,
        
        niceScroll: window.parent && window.parent.ripple,
        
        model: {},

        /*
         * Local Event-Listeners
         */
        events: {
            'click' : 'clickEvent'
        },
        
        initialize: function (options) {
            var self = this;
            Backbone.View.prototype.initialize.call(this,options);
            if(options&&options.addClasses) {
                this.addClasses = options.addClasses;
            }
            if(options&&options.addClassesSelector) {
                this.addClassesSelector = options.addClassesSelector;
            }
            this.listenTo(this,'view:render',function() {
                self.render();
            });
            this.listenTo(this,'view:loadData',function() {
                if(self.loadData) {
                    self.loadData();
                }
            });
        },

        updateModel: function(model) {
            this.model = model;
            return this;
        },
        
        setTemplateFacoryIfNull: function(templateFactory) {
            if(!this._templateFactory) {
                this._templateFactory = templateFactory;
            }
            return this;
        },
        
        build: function() {
            this.trigger('view:initialized');
            return this;
        },

        /*
         * Render this view
         */
        render: function() {
            if(this.hbsTemplate!=null) {
                this.template = Handlebars.compile(this._templateFactory.retrieveTemplate(this.hbsTemplate, this.alternateTemplatePath));
            } else
            if(this.hbsStringTemplate!=null) {
                this.template = Handlebars.compile(this.hbsStringTemplate);
            }
            if(this.template) {
                this.$el.html(this.template(this.model));
            }
            this.updateClasses(false);
            this.animateIn();
            if(this.loadData&&!this.dataLoaded) {
                console.log('doLoadData');
                this.dataLoaded = true;
                this.trigger('view:loadData');
            }
            return this;
        },
        
        updateClasses: function(remove) {
            //console.log('updateClasses',this.addClasses,(this.addClassesSelector?$(this.addClassesSelector):this.$el));
            if(this.addClasses) {
                if(remove) {
                    //console.log('removeCLasses');
                    (this.addClassesSelector?$(this.addClassesSelector):this.$el).removeClass(this.addClasses);
                } else {
                    //console.log('addeCLasses');
                    (this.addClassesSelector?$(this.addClassesSelector):this.$el).addClass(this.addClasses);
                }
            }
        },

        /*
         * Collect local Click-Events
         */
        clickEvent: function(ev) {
            //ev.preventDefault();
            ev.stopPropagation();
            var localId = $(ev.target).attr("id");
            if (localId != null && this.clickRoutes != null && this.clickRoutes[localId]) {
                eventChannel.trigger('app:router:navigate',{route:this.clickRoutes[localId]});
            } else {
                if ($(ev.target).hasClass("backButton")) {
                    eventChannel.trigger('app:router:back');
                }
                else if($(ev.target).is('button')) {
                    this.buttonClicked(ev, localId);
                }
            }
        },
        
        animateIn: function() {
            if(this._animated) {
                this.$el.removeClass('animated bounceInRight');
                this.$el.addClass('animated bounceInRight');
            }
        },
        
        animateOut: function() {
            if(this._animated) {
                this.$el.removeClass('animated bounceOutLeft');
                this.$el.addClass('animated bounceOutLeft');
            }
        },
        
        beforeClose: function() {
            this.trigger('view:readytoclose');
        },

        close: function (callback) {
            var self = this;
            self.listenTo(self,'view:readytoclose',function() {
                self.updateClasses(true);
                if (self._animated) {
                    self.$el.bind("animationend webkitAnimationEnd oAnimationEnd MSAnimationEnd", function () {
                        self.undelegateEvents();
                        self.remove();
                        self.unbind();
                        if (callback) {
                            callback();
                        }
                    });
                    self.animateOut();
                } else {
                    self.undelegateEvents();
                    self.remove();
                    self.unbind();
                    if (callback) {
                        callback();
                    }
                }
            });
            self.beforeClose();
        },

        buttonClicked: function(event,buttonId) {},

    });
});

/*global define*/

define('router/BaseRouter',[
    'jquery',
    'backbone',
    'logging/CoreLogging',
    'events/EventChannel'
], function ($, Backbone, clog, eventChannel) {
    'use strict';
    return Backbone.Router.extend({

        routes: {},
        
        appInstance: null,
        currentRouteParams: null,

        initialize:function(options){
            var self = this;
            clog("Router initialized");
            eventChannel.on('app:router:navigate',function(options){
                self.performNavigation(options);
            });
            eventChannel.on('app:router:back',function(){
                Backbone.history.history.back();
                //windows.history.back();
            });
        },
        
        performNavigation: function(options) {
            this.currentRouteParams = options.params;
            this.navigate(options.route, true);
        },
        
        errorHandler:function(obj){
            //TODO:hier jquery und Serverspezifische Fehler im Übergabeobjekt abfangen und view-spezifisch handeln
            if(obj && obj.responseJSON && obj.responseJSON.message){
                alert(obj.responseJSON.message);
            }
        },

        showView: function(appView, view, headerModel, showBottomView) {
            if(appView.headerView) {
                appView.changeHeaderView(null,headerModel);
            }
            appView.bottomViewHidden = !showBottomView;
            appView.showInfoButton = headerModel.showInfoButton;
            appView.changeContentView(view);
        },
        
        initAndShowView: function(view, options, namespace) {        
            if(options.showBackButton==undefined||options.showBackButton==null) {
                options.showBackButton = true;
            }
            if(options.showBottomView==undefined||options.showBottomView==null) {
                options.showBottomView = false;
            }
            if(options.showInfoButton == false) {
				options.showInfoButton = false;
            } else {
            	options.showInfoButton = true;
            }
            var self = this;
            if (namespace) {
                console.log('loadNameSpace:', namespace);
                self.appInstance.loadNameSpace(namespace, function () {
                    self.listenTo(view, 'view:initialized', function () {
                        self.showView(self.appInstance.applicationView, view, options, options.showBottomView);
                    });
                    view.build();
                });
            } else {
                self.listenTo(view, 'view:initialized', function () {
                    self.showView(self.appInstance.applicationView, view, options, options.showBottomView);
                });
                view.build();
            }
        }
    });
});

/*global define*/

/*
 * --> How to use
 
    var rq = new RequestFactory({
        baseUrl: 'https://example.de'
    });
 
    // Builder with custom Request-Object 
    // and custom Parameters (as JSON)
 
    this.requestBuilder['<BuilderKey>'] = {
        buildRequestObject: function(requestObject,parameters) {
            requestObject.data = parameters;
            requestObject.type = 'POST';
            requestObject.crossDomain = true;
            requestObject.dataType = "json";
            requestObject.contentType = "application/json";
            return requestObject;
        },
        buildRequestParams: function(requestObject,parameters) {
            return JSON.stringify(parameters);
        }
    };
    
    // Define Requests for Builder
    // will result in Request: 'https://example.de/test/example' of Type 'POST'
    // with JSON-Parameters 
    
    this.requestObjects['<RequestName>'] = {
        requestType: '<BuilderKey>',
        url: '/test/example',
        parameters: {param1: '1', param2: '2'},

        // If you want to use the cache everytime ...
        cacheRequest: true
        
        // ... or with timeout (5 Seconds) ...
        cacheTimeout: 5000
        
        // ... or with custom function
        cacheCustomFunction: function() {
            // return true to use cached Object oder false for a real request
            return true;
        }
        
    };
    
    // Perform Request
    rq.performRequest('<RequestName>','<CustomParams>');
  
    // Or as Object-Request (ex. Collection, to get a Collection)
    rq.performRequest('<RequestName>','<CustomParams>',new Backbone.Collection());
  
 */

define('factory/RequestFactory',[
    'jquery',
    'underscore',
    'backbone',
    'logging/CoreLogging',
    'events/EventChannel'
], function ($, _, Backbone, clog, eventChannel) {
    return Backbone.Model.extend({
        
        logging: true,

        // Base URL for Requests
        baseUrl: null,

        // All Requests defined by developer
        requestObjects: {},

        // All Request-Builders defined by developer
        requestBuilder: {},
        
        requestStack: null,

        // Initialize RequestFactory with or without base URL
        initialize:function(options){
            if(options&&options.baseUrl) {
                this.baseUrl = options.baseUrl;
            }
            this.requestStack = new Backbone.Collection();
            this.listenTo(this.requestStack, "change reset add remove", this._stackChanged);
        },
        
        _stackChanged: function() {
            this.requestStack.each(function(queuedRequestObject) {
              if(queuedRequestObject.get('started')==false) {
                    var model = queuedRequestObject.get('model');
                    var rObj = queuedRequestObject.get('request');
                    if(!rObj.timeout) {
                        rObj.timeout = 50000;
                    }
                    if(model!=undefined&&model!=null) {
                        model.fetch(rObj);
                    } else {
                        $.ajax(rObj);
                    }
                    queuedRequestObject.set('started',true);
                }
            });
        },
        
        _pushRequestToQueue: function(key,rObj,model) { 
            eventChannel.trigger('app:busy:true');
            var found = this.requestStack.findWhere({'key': key});
            if(!found) {
                this.requestStack.add({
                    key: key,
                    started: false,
                    request: rObj,
                    model: model
                });
            }
        },
        
        _removeRequestFromQueue: function(key) {
            var found = this.requestStack.findWhere({'key': key});
            if(found) {
                this.requestStack.remove(found);
            }
            if(this.requestStack.length<1) {
                eventChannel.trigger('app:busy:false');
            }
        },

        /* *********
         * Interface
         * *********
         */
         
         performRequest: function(key, parameters, model, noCache) {
             var self = this;
             _.defer(function(){
                 self.performSyncedRequest(key, parameters, model, noCache);
             });
         },
         

        /*
         * Performing request.
         */
        performSyncedRequest: function(key, parameters, model, noCache) {

            var self = this;
            
            if(this.baseUrl==null||this.baseUrl==undefined) {
                return;
            }

            var useCache = this.requestObjects[key].cacheRequest;
            var responeAvailable = this.requestObjects[key].cachedResponse;
            var timeoutAvailable = this.requestObjects[key].cacheTimeout;
            var customCache = (typeof this.requestObjects[key].cacheCustomFunction == 'function');
            
            var returnCached = false;
            
            if(!returnCached&&useCache&&responeAvailable&&customCache&&this.requestObjects[key].cacheCustomFunction()) {
                if(this.logging) clog('Return '+key+':success' + ' cached because of Custom-Function');
                returnCached = true;
            } else if(!returnCached&&useCache&&responeAvailable&&timeoutAvailable&&(Date.now() - self.requestObjects[key].lastSuccessfullCall)<timeoutAvailable) {
                if(this.logging) clog('Return '+key+':success' + ' cached because of Timeout-Function');
                returnCached = true;
            } else if(!returnCached&&useCache&&responeAvailable&&!timeoutAvailable) {
                if(this.logging) clog('Return '+key+':success' + ' cached');
                returnCached = true;
            }
            
            if(noCache==true) {
                if(this.logging) clog('NOCACHE');
                returnCached = false;
            }
            
            if(!returnCached) {
                if(this.logging) clog('Try to retrieve '+key);
                var requestObject = this._buildRequestObject(
                    key,
                    this._buildRequestUrl(key),
                    this._buildRequestParams(key,parameters),
                    model!=undefined&&model!=null
                );
                this._pushRequestToQueue(key,requestObject,model);
            } else {
                var cachedReference = this.requestObjects[key].cachedResponse;
                this._sendSuccess(key,JSON.parse(JSON.stringify(cachedReference)));
            }
        },
        
        /* ********
         * Internal
         * ********
         * /
         
        /*
         * Build a default request.
         * Updates the request by a specific builder.
         */
        _buildRequestObject: function(key,url,params,isModel) {
            
            /*
             * Create Base-Request
             */
            var requestObject = this._buildBaseRequestObject(key,url,params);
            
            /*
             * If requestObject is a Model,
             * define alternate Success/Error-Handlers
             */
            this._defineModelHandlers(key,requestObject,isModel);

            /*
             * Use Custum-Builder im defined,
             * if not return default Request
             */
            try {
                return (this._getBuilderForType(this.requestObjects[key].requestType)).buildRequestObject(requestObject,params);
            } catch(e) {
                return requestObject;
            }
     
            return requestObject;
        },
        
        /*
         * Returns the Builder for
         * a specific Request-Type.
         */
        _getBuilderForType: function(type) {
            var builder = this.requestBuilder[type];
            if(builder!=undefined&&builder!=null) {
                return builder;
            }
            return null;
        },
        
        _sendError: function(key,response,collection) {
            var builder = this._getBuilderForType(this.requestObjects[key].requestType);
            if(builder!=null) {
                try {
                    builder.responseHandler(key,response,false);
                } catch(e) {
                    this.trigger(key+':error', response);
                }
            } else {
                this.trigger(key+':error', response);
            } 
        },
        
        _sendSuccess: function(key,response,collection) {
            var builder = this._getBuilderForType(this.requestObjects[key].requestType);
            if(builder!=null) {
                try {
                    builder.responseHandler(key,response,true);
                } catch(e) {
                    this.trigger(key+':success', response);
                }
            } else {
                this.trigger(key+':success', response);
            }  
        },
        
        _cacheResponseForKey: function(response,key) {
            if(this.requestObjects[key].cacheRequest) {
                response = JSON.parse(JSON.stringify(response));
                (this.requestObjects[key])['cachedResponse'] = response;
            }
        },
        
        _defineModelHandlers: function(key,requestObject,isModel) {
            var self = this;
            if(isModel) {
                requestObject.success = function(collection,response,options){
                    if(self.logging) clog(key+':success1');
                    self.requestObjects[key].lastSuccessfullCall = Date.now();
                    self._cacheResponseForKey(response,key);
                    self._removeRequestFromQueue(key);
                    self._sendSuccess(key,response,collection);
                };
                requestObject.error = function(model, response, jqxhr){
                    var responseText = (jqxhr && jqxhr.xhr && jqxhr.xhr.responseText)?jqxhr.xhr.responseText:"no responseText";
                    if(self.logging) clog(key+':error1'+' '+ responseText);
                    self.requestObjects[key].lastErrorCall = Date.now();
                    self.requestObjects[key].cacheRequest = null;
                    self._removeRequestFromQueue(key);
                    self._sendError(key,response);
                };
            }
        },
        
        _buildBaseRequestObject: function(key,url,params) {
            var self = this;
            return {
                url: url,
                type:'GET',
                data: params,
                success:function(response){
                    if(self.logging) clog(key+':success2');
                    self.requestObjects[key].lastSuccessfullCall = Date.now();
                    self._cacheResponseForKey(response,key);
                    self._removeRequestFromQueue(key);
                    self._sendSuccess(key,response);
                },
                error:function(response){
                    if(self.logging) clog(key+':error2');
                    self.requestObjects[key].lastErrorCall = Date.now();
                    self.requestObjects[key].cacheRequest = null;
                    self._removeRequestFromQueue(key);
                    self._sendError(key,response);
                }
            };
        },
        
        /*
         * Build url for request.
         */
        _buildRequestUrl: function(key) {
            var url = null;
            var useBaseUrl = true;
            if(this.requestObjects[key].useBaseUrl!=undefined&&this.requestObjects[key].useBaseUrl!=null) {
                useBaseUrl = this.requestObjects[key].useBaseUrl;
            }
            var builder = this._getBuilderForType(this.requestObjects[key].requestType);
            if(builder!=null) {
                try {
                    var localUrl = useBaseUrl?this.baseUrl:'';
                    if(builder.baseUrl) {
                        localUrl = builder.baseUrl;
                    }
                    url = builder.buildRequestUrl(localUrl,this.requestObjects[key].url);
                } catch(e) {
                    url = (useBaseUrl?this.baseUrl:'')+this.requestObjects[key].url;
                }
            } else {
                url = (useBaseUrl?this.baseUrl:'')+this.requestObjects[key].url;
            }
            return url;
        },
        
        /*
         * Build parameters for request.
         * Updates the request by a specific builder.
         */
        _buildRequestParams: function(key,parameters) {
            var directReturn = false;
            var builder = this._getBuilderForType(this.requestObjects[key].requestType);
            if(builder!=null) {
                try {
                    if(parameters) {
                        return builder.buildRequestParams(key,this.requestObjects[key],parameters);
                    } else {
                        return builder.buildRequestParams(key,this.requestObjects[key],this.requestObjects[key].parameters);
                    }
                } catch(e) {
                    directReturn = true;
                }
            } else {
                directReturn = true;
            }
            if(directReturn) {
                if(parameters) {
                    return parameters;
                } else {
                    return this.requestObjects[key].parameters;
                }
            }
            return null;
        }

    });
});

/*global define*/

define('view/BottomMenu',[
    'jquery',
    'underscore',
    'backbone',
    'view/BaseView',
    'events/EventChannel'
], function ($, _, Backbone, BaseView, eventChannel) {
    'use strict';
    return BaseView.extend({

        items: [],
        
        moreItemIndex: 2,
        moreItem: null,
        
        moreIsDisplayed: function() {
            var hasClass = false;
            var moreItems = $('.pcore-bottom-menu-item-more-item');
            if(moreItems.length>0) {
                hasClass = moreItems.hasClass('pcore-bottom-menu-item-more-item-hiden');
            }
            return !hasClass;
        },
        
        showMore: function() {
            $('.pcore-bottom-menu-item-more-item').removeClass('pcore-bottom-menu-item-more-item-hiden');
        },
        
        hideMore: function() {
            //console.log('HideMore');
            $('.pcore-bottom-menu-item-more-item').addClass('pcore-bottom-menu-item-more-item-hiden');
        },

        model: null,
        
        maxItems: 3,

        ItemClass: BaseView.extend({
            events: {
                "click": "vehicleTypeChanged"
            },
            vehicleTypeChanged: function(ev) {
                //console.log('MenuClick');
                var route = $(ev.currentTarget).find('.pcore-bottom-menu-item')[0].id;
                if(route!='_moreView') {
                    $('.pcore-bottom-menu-item-more-item').addClass('pcore-bottom-menu-item-more-item-hiden');
                    eventChannel.trigger('app:router:navigate',{route:route});
                } else {
                    eventChannel.trigger('display:more');
                }
            },
            
            hbsTemplate: 'BottomMenuItemView',
            alternateTemplatePath: './scripts/templates/ui/',
            itemWidth: "0px",
            morePosX: 0,
            morePosY: 0,
            render: function() {
                BaseView.prototype.render.call(this);
                this.delegateEvents();
                this.$el.find('.pcore-bottom-menu-item').css("width", this.itemWidth);
                if(!this.isDisplayed&&!this.isMoreItem) {
                    this.$el.find('.pcore-bottom-menu-item').addClass('pcore-bottom-menu-item-more-item');
                    this.$el.find('.pcore-bottom-menu-item').css("bottom", this.morePosY+'px');
                    this.$el.find('.pcore-bottom-menu-item').css("left", this.morePosX+'px');
                    this.$el.find('.pcore-bottom-menu-item').addClass('pcore-bottom-menu-item-more-item-hiden');
                }
                if(this.model.icon) {
                    this.$el.find('#menu_item').addClass('glyphicon '+this.model.icon);
                }
                if(this.model.route) {
                    this.$el.find('.pcore-bottom-menu-item').attr("id",this.model.route);
                }
            }

        }),

        initialize: function(options) {
            BaseView.prototype.initialize.call(this,options);
            this.hideMore();
            var self = this;
            this.items = [];
            this.model = options.model;
            this.setTemplateFacoryIfNull(options.templateFactory);
            this.$el.addClass("appBottomDiv");

            this.maxItems = options.maxItems != null ? options.maxItems : 3;
            this.moreItemIndex = this.maxItems > 0 ? this.maxItems -1 : 0;

            if(options.items.length>this.maxItems) {
                var menuItem = new this.ItemClass();
                menuItem.model = {
                    text: 'Mehr', 
                    icon: 'glyphicon glyphicon-option-horizontal', 
                    route: '_moreView'
                };
                self.moreItem = menuItem;
            }

            _.each(options.items, function(item){
                self.addItem(item);
            });
            
            eventChannel.off('display:more').on('display:more',function(){
                if(self.moreIsDisplayed()==false) {
                    self.showMore();
                } else {
                    self.hideMore();
                }
            });

            $(window).on("resize", self.orientationChanged.bind(self));
            
            return this;
        },

        render: function() {
            BaseView.prototype.render.call(this);
            var maxDisplayIcons = this.maxItems<=this.items.length?this.maxItems:this.items.length;
            var itemIndex = 0;
            this.delegateEvents();
            for(var i=0; i<maxDisplayIcons; i++) {
                if(i!=this.moreItemIndex||!this.moreItem) {
                    this.items[itemIndex].setTemplateFacoryIfNull(this._templateFactory);
                    this.items[itemIndex].isDisplayed = true;
                    this.$el.append(this.items[itemIndex].el);
                    itemIndex++;
                } else {
                    this.moreItem.setTemplateFacoryIfNull(this._templateFactory);
                    this.moreItem.isMoreItem = true;
                    this.$el.append(this.moreItem.el);
                }
            }
            var localCount = 0;
            for(var i=0; i<this.items.length; i++) {
                if(!this.items[i].isDisplayed) {
                    this.items[i].setTemplateFacoryIfNull(this._templateFactory);
                    app.applicationView.$el.append(this.items[i].el);
                    localCount++;
                }
            }
            this.renderItems();
            return this;
        },

        renderItems: function(){
            var localCount = 0;
            var itemIndex = 0;
            var maxDisplayIcons = this.maxItems<=this.items.length?this.maxItems:this.items.length;
            for(var i=0; i<maxDisplayIcons; i++) {
                if(i!=this.moreItemIndex||!this.moreItem) {
                    this.items[itemIndex].itemWidth = this.calcItemWidth()+"px";
                    itemIndex++;
                } else {
                    this.moreItem.itemWidth = this.calcItemWidth()+"px";
                    this.moreItem.render();
                }
            }
            for(var i=0; i<this.items.length; i++) {
                if(!this.items[i].isDisplayed) {
                    this.items[i].itemWidth = this.calcItemWidth()+"px";
                    this.items[i].morePosX = this.calcItemWidth()*this.moreItemIndex;
                    this.items[i].morePosY = 60*(localCount+1);
                    localCount++;
                }
                this.items[i].render();
            }
        },

        orientationChanged:function(){
            this.renderItems();
        },
        
        calcItemWidth: function() {
            var maxDisplayIcons = this.maxItems<=this.items.length?this.maxItems:this.items.length;
            var width = ($('.appBottomDiv').innerWidth()/maxDisplayIcons);
            return width;
        },

        addItem: function(model) {
            //console.log('addNewItem');
            var menuItem = new this.ItemClass();
            menuItem.model = model;
            this.items.push(menuItem);
            return this;
        }

    });
});

/*global EWAAppNeu, Backbone, JST*/

define('view/GoogleMapView',[
        'jquery',
        'underscore',
        'backbone',
        'view/BaseView',
        'events/EventChannel',
        'service/LocationService'
    ],
    function ($, _, Backbone, BaseView, eventChannel, locationService) {
        'use strict';
        return BaseView.extend({
            
            hbsStringTemplate: '<div id="map-canvas" class="google-map-canvas"></div>',
            
            mapSelector:'map-canvas',
            height: '100px',
            mapAttributes: {},    

            circleMap: {},
            markerWindowMap: {},
            markerMap: {},
            map: null,

            mapMoved: false,
            niceScroll: false,
            
            autoReverseGeocoding: false,
            currentReverseGeocodingAddress: null,
            reverseGeocodingFinished: null,

            updateMap: function() {
                this.renderGoogleMap();
                this.renderOverlay();
            },

            centerMapTo: function(cl) {
                this.mapMoved = false;
                if(this.map) {
                    this.map.setCenter(cl);
                }
                this.performAutoReverseGeocoding();
            },

            render: function() {
                var self = this;
                BaseView.prototype.render.call(this);
                this.renderGoogleMap();
                if(this.height) {
                    $('#'+this.mapSelector).css('height',this.height);
                }
                setTimeout(function(){
            		self.height = (window.innerHeight)-$('#appHeader').height()-$('#appBottom').height();
            		$('#'+self.mapSelector).css('height', self.height);
				}, 100);

                var locServ = new locationService({
                    defaultLatitude: 0,
                    defaultLongitude: 0
            });

                var positionButton = document.createElement("div");
                positionButton.setAttribute("id", "updatePositionButton");
                self.map.controls[google.maps.ControlPosition.RIGHT_BOTTOM].push(positionButton);
                positionButton.addEventListener("click", function(){
                    eventChannel.trigger('app:busy:true');
						  app.requestFactory.updatePositionAndRequestPlaces(function(location) {
							    locServ.updateLocation(function(location){
							        self.centerMapTo(new google.maps.LatLng(location.latitude, location.longitude));
							        self.updateMap();
							        eventChannel.trigger('app:busy:false');
							    });
						  })

                });

                return this;
            },
            
            resize: function () {
                var self = this;
                if(self.map) {
                    var center = self.map.getCenter();
                    google.maps.event.trigger(self.map, "resize");
                    self.map.setCenter(center);
                }
            },

            renderGoogleMap: function() {
                var self = this;
                if(!this.map&&this.model&&this.mapSelector&&this.mapAttributes) {
                    console.log('MAPATTR:',this.mapAttributes);
                    this.map = new google.maps.Map(
                        document.getElementById(this.mapSelector),
                        this.mapAttributes
                    );
                    this.mapMoved = false;
                    google.maps.event.addListenerOnce(self.map, 'idle', function(){
                        self.performAutoReverseGeocoding();
                    });
                    google.maps.event.addListener(this.map,'dragend',function() {
                        self.mapMoved = true;
                        self.performAutoReverseGeocoding();
                        if(self.dragListener) {
                            self.dragListener(self);
                        } else {
                            self.updateMap();
                        }
                    });
                    google.maps.event.addListener(self.map, 'tilesloaded', function() {
                        self.resize();
                    });
                    google.maps.event.addListener(this.map,'zoom_changed',function() {
                        self.mapMoved = true;
                        self.performAutoReverseGeocoding();
                        if(self.zoomListener) {
                            self.zoomListener(self);
                        } else {
                            self.updateMap();
                        }
                    });
                    self.afterMapInitialization();
                }
            },
            
            afterMapInitialization: function() {
                //console.log('MapInitialized');    
            },
            
            getSelectedPosition: function() {
                var selectedPosition = this.map.getCenter();
                return new google.maps.LatLng(selectedPosition.lat(),selectedPosition.lng());
            },

            markerCluster: null,
    
            renderOverlay: function() {
                //console.log('renderOverlay');
                var self = this;
                if(this.model&&this.updateMapOverlay) {
                    if(self.markerMap) {
                        _.each(self.markerMap,function(marker,name){
                            marker.setMap(null);
                        });
                        self.markerMap = {};
                    }
                    this.updateMapOverlay(self);
                    var markerArray = [];
                    if(self.markerMap&&self.markerClickListener) {
                        _.each(self.markerMap,function(marker,name){
                            google.maps.event.addListener(marker, 'click', function() {
                                marker.name = name;
                                self.markerClickListener(self,marker);
                            });
                            if(!marker.getMap()) {
                                markerArray.push(marker);
                            }
                        });
                    }
                    var mcOptions = {
                        gridSize: 50,
                        maxZoom: 13,
                        imagePath: 'images/m'
                    };
                    if(self.markerCluster==null) {
                        self.markerCluster = new MarkerClusterer(self.map, markerArray, mcOptions);
                    } else {
                        self.markerCluster.clearMarkers();
                        self.markerCluster.addMarkers(markerArray);
                    }
                }
                self.resize();
            },

            addCenterMarker: function(icon,shape) {
                if(!this.markerMap['centerMarker']) {
                    this.markerMap['centerMarker'] = new google.maps.Marker({
                        icon: icon,
                        shape: shape,
                        map: this.map
                    });
                    this.markerMap['centerMarker'].bindTo('position', this.map, 'center');
                }
            },

            addPositionMarker: function(latitude,longitude) {
                // Als Check of ein entsprechendes Bild vorhanden ist
                var self = this;
                if(!this.markerMap['positionMarker']) {
                    $.ajax({
                        url: 'images/icon_position.png',
                        type: 'HEAD',
                        error: function () {
                            self.markerMap['positionMarker'] = new google.maps.Marker({
                                position: new google.maps.LatLng(latitude,longitude),
                                map: self.map,
                            });
                        },
                        success: function () {
                            var positionImage = {
                                url: "images/icon_position.png",
                                scaledSize: new google.maps.Size(21, 34),
                                size: new google.maps.Size(44, 74),
                                anchor: new google.maps.Point(10, 34)
                            };
                            self.markerMap['positionMarker'] = new google.maps.Marker({
                                position: new google.maps.LatLng(latitude,longitude),
                                map: self.map,
                                icon: positionImage,
                            });
                        }
                    });
                }
            },

            showInfoWindow: function(marker,view) {
                var self = this;
                _.each(self.markerWindowMap,function(infoWindow,key){
                    infoWindow.setMap(null);
                    infoWindow.close();
                });
                self.markerWindowMap = {};
                self.markerWindowMap['markerWindow_'+marker.name] = new google.maps.InfoWindow({
                    content: view?'<div id="markerWindowDiv_'+marker.name+'">'+$(view.render().el.childNodes).html()+'</div>':'<div>'+marker.name+'</div>'
                });
                self.markerWindowMap['markerWindow_'+marker.name].open(self.map,marker);
                self.markerWindowMap['markerWindow_'+marker.name].marker = marker;
                google.maps.event.addListener(self.markerWindowMap['markerWindow_'+marker.name], 'domready', function(){
                    $('#markerWindowDiv_'+marker.name+' button').off('click').on('click',function(ev){
                        if(self.buttonClickListener) {
                            self.buttonClickListener(self,marker,ev);
                        }
                    });
                });
            },

            addCircle: function(name,options) {
                if(this.circleMap[name]) {
                    this.circleMap[name].setMap(null);
                    this.circleMap[name] = null;
                }
                options.map = this.map;
                this.circleMap[name] = new google.maps.Circle(options);
            },

            showStreetView: function(marker,maxDistance) {
                var point = new google.maps.LatLng(marker.position.A,marker.position.F);
                var streetViewService = new google.maps.StreetViewService();
                var panorama = this.map.getStreetView();
                streetViewService.getPanoramaByLocation(point, maxDistance, function (streetViewPanoramaData, status) {
                    if(status === google.maps.StreetViewStatus.OK){
                        var oldPoint = point;
                        point = streetViewPanoramaData.location.latLng;
                        var heading = google.maps.geometry.spherical.computeHeading(point,oldPoint);
                        panorama.setPosition(point);
                        panorama.setPov({
                            heading: heading,
                            zoom: 1,
                            pitch: 0
                        });
                        panorama.setVisible(true);
                    }else{
                        console.log("Sorry! Street View is not available.");
                    }
                });
            },
            
            geocoder: null,
            currentGeocodingAddress: null,
            currentGeocodingResults: [],
            
            performGeocoding: function (addressStr,callback) {
                var self = this;
                if(self.geocoder==null) {
                    self.geocoder = new google.maps.Geocoder();
                }
                console.log('GeoCode: '+addressStr);
                self.geocoder.geocode({ 'address':addressStr+" deutschland", region:"de"}, function (results, status) {
                    if (status == google.maps.GeocoderStatus.OK) {
                        console.log('RESULTS:',results);
                        self.currentGeocodingResults = results;
                        self.currentGeocodingAddress = new self.AddressModel({result:results[0]});
                        //self.map.setCenter(results[0].geometry.location);
                        callback(self.currentGeocodingAddress,status);
                    } else {
                        console.log('geocoding:error1 ',status);
                        callback(null,status);
                    }
                });
            },
            
            performAutoReverseGeocoding: function() {
                console.log('performAutoReverseGeocoding');
                if(!this.map) {
                    return;
                }
                var self = this;
                if(self.geocoder==null) {
                    self.geocoder = new google.maps.Geocoder();
                }
                if(this.autoReverseGeocoding) {
                    self.geocoder.geocode({'location': self.getSelectedPosition()}, function(results, status) {
                        console.log('performAutoReverseGeocodingResult: ',results,status);
                        
                        if (status == google.maps.GeocoderStatus.OK) {
                            self.parseReverseGeocodingResponse(results);
                        } else {
                            console.log('geocoding:error2');
                        }
                    });
                }
            },
            
            AddressModel: Backbone.Model.extend({ 
                getAddressComponent: function (component) {
                    var component = _.find(
                        this.get('result').address_components, function (ac) { 
                            return ac.types[0] == component 
                        }
                    );
                    if(component&&component.short_name) {
                        return component.short_name;
                    } else {
                        return '';
                    }
                },
                getAddressString: function (locale) {
                    var addressStr = this.getAddressComponent('route') + ' ' +
                        this.getAddressComponent('street_number') + ', ' +
                        this.getAddressComponent('postal_code') + ' ' +
                        this.getAddressComponent('locality');
                    return addressStr;
                }
            }),
            
            parseReverseGeocodingResponse: function(results) {
                this.currentReverseGeocodingAddress = new this.AddressModel({result:results[0]});
                this.trigger('reversegeocoding:finished',this.currentReverseGeocodingAddress);
                if(this.reverseGeocodingFinished) {
                    this.reverseGeocodingFinished(this.currentReverseGeocodingAddress);
                }
            },
            
            /*
             * Directions API
             */
             
            directionsDisplay: null,
            directionsService: null,

            calcRoute: function (start,end,travelMode,display,callback) {
                var self = this;
                if(!this.directionsService) {
                    this.directionsService = new google.maps.DirectionsService();
                    this.directionsDisplay = new google.maps.DirectionsRenderer();
                }
                this.directionsDisplay.setMap(this.map);
                this.directionsService.route({
                    origin:start,
                    destination:end,
                    travelMode: travelMode
                }, function(result, status) {
                    if(display&&(status == google.maps.DirectionsStatus.OK)) {
                        self.directionsDisplay.setDirections(result);
                    }
                    if(callback) callback(result, status);
                });
            }

        });
    });

/*global define*/

define('service/LocationService',[
    'jquery',
    'underscore',
    'backbone',
    'logging/CoreLogging',
    'events/EventChannel'
], function ($, _, Backbone, clog, eventChannel) {
    'use strict';
    return Backbone.Model.extend({
        
        location: null,
        
        initialize:function(options){
            Backbone.Model.prototype.initialize.call(this,options);
            this.triggerLocation(this.get('defaultLatitude'),this.get('defaultLongitude'));
            this.updateLocation();
        },
        
        triggerLocation: function(latitude,longitude,state,callback) {
            this.set('location', {
                state: state,
                latitude: latitude,
                longitude: longitude
            });
            if(callback) {
                callback(this.get('location'));
            }
            eventChannel.trigger('location:success', this.get('location'));
        },

        updateLocation: function(callback) {
            var self = this;
            console.log(self);
            if (navigator.geolocation) {
                navigator.geolocation.getCurrentPosition(
                    function(location) {
                        self.triggerLocation(location.coords.latitude,location.coords.longitude,0,callback);
                    },
                    function(error) {
                        self.triggerLocation(self.get('defaultLatitude'),self.get('defaultLongitude'),1,callback);
                    },
                    {enableHighAccuracy: true,timeout: 15000}
                );
            } else {
                self.triggerLocation(self.get('defaultLatitude'),self.get('defaultLongitude'),2,callback);
            }
        }
        
    });
});



/*global EWAAppNeu, Backbone, JST*/

define('view/AlertBoxView',[
        'jquery',
        'underscore',
        'backbone',
        'view/BaseView',
        'events/EventChannel'    
    ],
    function ($, _, Backbone, BaseView, eventChannel) {
        'use strict';
        return BaseView.extend({
            hbsStringTemplate: '',
            render: function() {
                BaseView.prototype.render.call(this);
                var html = ''
                +'<div id="local_alert" class="alert alert-danger" role="alert">'
                +'<button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>'
                +'<span class="sr-only">Error:</span>'
                +this.model.message
                +'</div>';
                this.$el.html(html);
                return this;
            }
        });
    });

define('pcorelib',[
    'Application',
    'view/ApplicationView',
    'view/BaseView',
    'router/BaseRouter',
    'factory/RequestFactory',
    'factory/TemplateFactory',
    'events/EventChannel',
    'view/BottomMenu',
    'view/GoogleMapView',
    'service/LocationService',
    'view/AlertBoxView'
], function (Application,ApplicationView,BaseView,BaseRouter,RequestFactory,TemplateFactory,EventChannel,BottomMenu,GoogleMapView,LocationService,AlertBoxView) {
    return {
        Application: Application,
        ApplicationView: ApplicationView,
        BaseView: BaseView,
        BaseRouter: BaseRouter,
        RequestFactory: RequestFactory,
        TemplateFactory: TemplateFactory,
        EventChannel: EventChannel,
        BottomMenu: BottomMenu,
        GoogleMapView: GoogleMapView,
        LocationService: LocationService,
        AlertBoxView: AlertBoxView
    };
});
