/*global define*/

define([
	'jquery',
	'underscore',
	'backbone',
	'pcorelib',
	'moment',
	'bookingService',
	'renderService',
	'serverConfigBuilder',
	'application/ErrorHandling'
], function ($, _, Backbone, pcorelib, moment, BookingService, RenderService, ServerConfigBuilder, errorHandling) {
	'use strict';
	return pcorelib.Application.extend({

		getProvidedHeight: function() {
			return parseInt($('#heightContainer').text());
		},

		onMessage: function(e) {
			if(e&&e.data) {
				if(e.data === 'EVENT_GOBACK') {
					if(!isHomeView) {
						Backbone.history.history.back();
					} else {
						parent.postMessage('CANTAMEN_HOME_BACK', '*');
					}
				}
				if(e.data === 'EVENT_LOGOUT') {
					app.configuration.saveLoginData = false;
					app.saveConfiguration();
					pcorelib.EventChannel.trigger('app:router:navigate',{route:'login'});
				}
			}
		},

		build: function(options,buildCallback) {
			var self = this;
			pcorelib.Application.prototype.build.call(this,options,buildCallback);
			setTimeout(function(){ // load namespaces to use it in index.html despite it not being .hbs
				app.loadNameSpace(['999-err-msg','236-new-b-directions'])
			}, 1000);

			// registers Eventlisteners for Errorhandling
			errorHandling.registerEvents();

			// pcorelib.EventChannel.on('error:true:prepareBooking', function() {
			// 	self.showErrorMessage('prepareBooking');
			// });
			// pcorelib.EventChannel.on('error:true:performBooking', function() {
			// 	self.showErrorMessage('performBooking');
			// });

			$('#errorMessage').on('click', function() {
				pcorelib.EventChannel.trigger('error:hide');
			});

			/*
			// tests if loginbutton exists
			// true --> splashscreen is hidden
			var contentLoadedInterval = setInterval(function(){
				if($('#loginBtn').html() || $('#noConnectionForm').html()) {
					pcorelib.EventChannel.trigger('splashscreen:hide');
				}
			},100);
			*/

			pcorelib.EventChannel.on('splashscreen:hide', function() {
				$('#loadingDiv').hide();
				if(navigator&&navigator.splashscreen) {
					navigator.splashscreen.hide();
				}
				if (typeof variable !== 'undefined') {
					clearInterval(contentLoadedInterval);
				}
			});

			pcorelib.EventChannel.on('app:router:navigate',function(options){
				if(window.location.hash != "#infoView") {
					var hash = window.location.hash;
					// delete leading '#' of window.location.hash
					if(hash.indexOf('#') === 0) {
						hash = hash.substring(1);
					}
					// last visited page gets stored in app.previousView[]
					app.previousView = hash;
				}
			});

			// Helper for Handlebars to perform a comparisson inside the template
			Handlebars.registerHelper('if_eq', function(a, b, opts) {
			    if(a == b)
			        return opts.fn(this);
			    else
			        return opts.inverse(this);
			});
        },

        deleteSessionData: function() {
        	app.bookingSuggestions = null;
    		app.estimatedPrice = null;
			app.newFilterObj = [];
			app.orgSettings = null;
			app.selectedBookee = null;
			app.selectedBooking = null;
			app.suggestionCount = 0;
			app.summarizedBookingSuggestions = null;
			app.userData = null;
			app.userFavPlaces = [];
			app.userNewFavPlaces = [];
			app.userOrgas = [];
			app.selectedOrga = null;
			app.vehicleBookeeTypes = null;
			app.previousView = "";
			app.userSettings.settings = null;
			app.service.booking.startDate = new Date(),"min";
			app.service.booking.endDate = new Date(),"min";
			app.prelimActive = false;
        },

		constants: {
			distanceUnit: {
				"KILOMETERS": "km",
				"METERS": "m",
				"STATUTEMILES": "sm",
				"NAUTICALMILES": "nm"
			}
		},

		service: {
			booking: null
		},

		initServices: function() {
			this.service.booking = BookingService;
			_.extend(this.service.booking, Backbone.Events);
			this.service.render = RenderService;
			_.extend(this.service.render, Backbone.Events);
		},

		/*
		 * Responses received from server
		 */
		userData: null,
		userOrgas: null,
		userFavPlaces: null,
		userNewFavPlaces: [],
		orgSettings: null,
        bookeeLimit: 20,
		/**
		 * Response from get_member_settings
		 */
		vehicleBookeeTypes: null,
		vehicleAttributes: null,
		newFilterObj: [],

		locationService: null,

		embeddedMode: true,
		startupType: null,
		startupX: null,
		startupY: null,
		startupPlace: null,
		startupPlaceName: null,
		startupStart: null,
		startupEnd: null,

		data: {
			"langParameters": {
				"selectedLang": "de"
			},
			"userParameters": {
				"orgNr": null,
				"password": null,
				"user": null
			},
			"adminParameters": {

				"orgNr": "14",
				"password": "Ebus23.4",
				"adminLogin": "_a_a"
			},
			"baseUrl" : "",
			"rangeParameters": [0.5, 1, 2, 5, 10, 20, 30, 50]
		},

		auth: {
			"adminAuthentication": {
				"authentication": null
			},
			"fullAuthentication": {
				"authentication": null
			},
			"token": null
		},

		userSettings: {
			settings: []
		},

		bookingData: {
			startDate: null,
			endDate: null,
			selectedFilter: null,
			selectedPlace: null,
			selectedRange: null
		},

		bookingSuggestions: null,
		summarizedBookingSuggestions: null,
		selectedBookee: null,
        selectedBooking: null,
		imageCache: [],
		estimatedPrice:null,

		previousView: "",

		buildAuth: function () {

			this.auth.adminAuthentication.authentication = {"admin": this.data.adminParameters};
			this.auth.fullAuthentication.authentication = {
				"admin": this.data.adminParameters,
				"member": this.data.userParameters
			};
			var self = this;

		},

		accessLocationService: function() {
			if(this.locationService==null) {
				this.locationService = new pcorelib.LocationService({
					defaultLatitude: 52.362926,
					defaultLongitude: 9.728683
				});
			}
			return this.locationService;
		},

		helper: {
			parseDateDigit: function (digit) {
				digit = digit.toString();
				if (digit.length == 1) {
					return "0" + digit;
				}
				else {
					return digit;
				}
			}
		},

		suggestionCount: 0,

		buildSummarizedBookees: function() {
			var self = this;
			self.suggestionCount = 0;
			this.summarizedBookingSuggestions = this.bookingSuggestions.groupBy(function(bookee){
				if(bookee.get('bookee').place) {
					var placeNr = bookee.get('bookee').place.nr;
					if(self.suggestionCount<=placeNr) {
						self.suggestionCount = placeNr;
					}
					return placeNr;
				} else {
					return 0;
				}
			});

			var startIndex = self.suggestionCount+1;
			_.each(this.summarizedBookingSuggestions[0], function(item) {
				self.summarizedBookingSuggestions[startIndex] = [item];
				startIndex++;
			});
			delete this.summarizedBookingSuggestions[0];

			if(this.bookingSuggestions.length<1) {
				self.summarizedBookingSuggestions = null;
			}
			return this.summarizedBookingSuggestions;
		}

	});
});
