/*global define*/

define([
	'jquery',
	'moment'
], function ($, Moment) {
	'use strict';
	return {

		_momentAsGermanString: function(currentDate) {
			var weekday = new Array(7);
			weekday[0]=  "So";
			weekday[1] = "Mo";
			weekday[2] = "Di";
			weekday[3] = "Mi";
			weekday[4] = "Do";
			weekday[5] = "Fr";
			weekday[6] = "Sa";
			var months = new Array(7);
			months[0]=  "Januar";
			months[1] = "Februar";
			months[2] = "März";
			months[3] = "April";
			months[4] = "Mai";
			months[5] = "Juni";
			months[6] = "Juli";
			months[7] = "August";
			months[8] = "September";
			months[9] = "Oktober";
			months[10] = "November";
			months[11] = "Dezember";
			var strDay = weekday[currentDate.getDay()];
			var day = currentDate.getDate();
			var month = currentDate.getMonth();
			var minutes = currentDate.getMinutes()<10?'0'+currentDate.getMinutes():currentDate.getMinutes();
			var hours = currentDate.getHours()<10?'0'+currentDate.getHours():currentDate.getHours();
			var strDateTime = '' + strDay + ' ' + day + '. ' + months[month] + ' ' + currentDate.getFullYear() + ' ' + hours + ':' + minutes;
			return strDateTime;
		},

		_formatDateString: function (date, type) {
			var dateString;
			if (date == "sofort" || date == "") {
				date = new Date();
			}
			switch (type) {
				case "sec":
					dateString = app.helper.parseDateDigit(date.getDate()) + "." + app.helper.parseDateDigit(date.getMonth() + 1) + "." + date.getFullYear() + " " + app.helper.parseDateDigit(date.getHours()) + ":" + app.helper.parseDateDigit(date.getMinutes()) + ":" + app.helper.parseDateDigit(date.getSeconds());
					break;
				case "url":
					dateString = date.getFullYear() + "-" + app.helper.parseDateDigit(date.getMonth() + 1) + "-" + app.helper.parseDateDigit(date.getDate()) + " " + app.helper.parseDateDigit(date.getHours()) + ":" + app.helper.parseDateDigit(date.getMinutes());
					break;
				case "min":
					dateString = app.helper.parseDateDigit(date.getDate()) + "." + app.helper.parseDateDigit(date.getMonth() + 1) + "." + date.getFullYear() + " " + app.helper.parseDateDigit(date.getHours()) + ":" + app.helper.parseDateDigit(date.getMinutes());
					break;
				case "short":
					dateString = app.helper.parseDateDigit(date.getHours()) + ":" + app.helper.parseDateDigit(date.getMinutes());
					break;
				case "iso8601":
					dateString = this._dateToIso8601(date);
					break;
				case "ger":
					dateString = this._momentAsGermanString(date);
					break;
			}
			return dateString;
		},

		_dateToIso8601: function(localDate) {
			var utcTime = Moment(localDate).utc().toDate();
			utcTime.setHours(utcTime.getHours() - utcTime.getTimezoneOffset() / 60);
			var dateString = JSON.stringify(utcTime);
			var timeZoneString = '.000';
			var tzValue = ((utcTime.getTimezoneOffset()/60)*-1);
			var isSet = false;
			if(tzValue>=0&&tzValue<10) {
				timeZoneString = timeZoneString + '+0' + tzValue + ':00';
				isSet = true;
			}
			if(tzValue>=0&&tzValue>9) {
				timeZoneString = timeZoneString + '+' + tzValue + ':00';
				isSet = true;
			}
			if(tzValue<0&&tzValue>=-9) {
				timeZoneString = timeZoneString + '-0' + tzValue + ':00';
				isSet = true;
			}
			if(tzValue<0&&tzValue<-9) {
				timeZoneString = timeZoneString + '-' + tzValue + ':00';
				isSet = true;
			}
			if(dateString.substr(1,2)=='00') {
				var retStr = dateString.substring(1, dateString.length-6)+timeZoneString;
				return '20'+retStr.substring(2);
			}
			return dateString.substring(1, dateString.length-6)+timeZoneString;
		},
        
        _getPreparedStartDate: function(startDate,endDate,intervallMinutes,minBookingMinutes,setByHand) {
            var minutes = startDate.getMinutes();
            var hours = startDate.getHours();
            var calculatedMinutes = (Math.ceil(minutes/intervallMinutes)) * intervallMinutes;
            if(setByHand) {
                calculatedMinutes = (Math.floor(minutes/intervallMinutes)) * intervallMinutes;
            }
            var resultDate = new Date(startDate.getTime());
            resultDate.setHours(hours);
			resultDate.setMinutes(calculatedMinutes);
			resultDate.setSeconds(0);
            return resultDate;
        },
        
        _getPreparedEndDate: function(startDate,endDate,intervallMinutes,minBookingMinutes,setByHand) {
            var timeDiff = Math.floor((endDate-startDate)/1000/60);
            if(timeDiff<minBookingMinutes) {
                var calcStartDate = this._getPreparedStartDate(startDate,endDate,intervallMinutes,minBookingMinutes,setByHand);
                var resultDate = new Date(endDate.getTime());
					 // Tagessprung.beachten
					 if (calcStartDate.getDate() > resultDate.getDate()) {
						resultDate.setDate(calcStartDate.getDate());
					 }
                resultDate.setHours(calcStartDate.getHours());
                resultDate.setMinutes(calcStartDate.getMinutes()+minBookingMinutes);
                resultDate.setSeconds(0);
                return resultDate;
            } else {
                return endDate;
            }
        },

		_getPreparedDate: function(baseDate,intervallMinutes,minBookingMinutes,isFloor,startDate,isEndDate) {
            
            var nextHour = false;
            var minutes = baseDate.getMinutes();
            var hours = baseDate.getHours();
            if(minutes==0||minutes==intervallMinutes) {
                minutes = minutes + 1;
            }
            if(((60-minutes)<intervallMinutes)&&!isFloor) nextHour = true;
			var m = ((isFloor?(Math.floor(minutes/intervallMinutes)):(Math.ceil(minutes/intervallMinutes))) * intervallMinutes) % 60;
            var h = nextHour ? (hours+1) : hours;
			baseDate.setHours(h);
			baseDate.setMinutes(m);
			baseDate.setSeconds(0);
			return baseDate;
            
		},

		_getDurationValue: function (str,id) {
			var idIndex = str.indexOf(id);
			if (idIndex > -1) { // Found
				var value = '00';
				var firstTry = str.substring(idIndex - 2, idIndex);
				var secondTry = str.substring(idIndex - 1, idIndex);
				if (!isNaN(firstTry)) { //Zweistellig
					value = firstTry;
				} else if (!isNaN(secondTry)) { //Einstellig
					value = secondTry;
				}
				if(value.length<2) {
					value = '0'+value;
				}
				return value;
			}
		},

		_convertDuration: function (str) {
			return {
				days: this._getDurationValue(str,'D'),
				hours: this._getDurationValue(str,'H'),
				minutes: this._getDurationValue(str,'M'),
				seconds: this._getDurationValue(str,'S'),
				inMinutes: function() {
					return (parseInt(this.days*24*60)+parseInt(this.hours*60)+parseInt(this.minutes));
				},
				inDifferenceFromNow: function() {
					return new Date(Math.round((new Date()).getTime() + (this.inMinutes()*60000)));
				}
			};
		},

		_maxBookingLength: function(iso, diff, bool) {
			if(bool) {
				return iso;
			} else {
				return diff;
			}
		},

		_buildSetMemberSettingsRequest: function(organisation, filterArray) {
			var settings = {};
			settings[organisation] = filterArray;

			app.requestFactory.performRequest('set_member_settings', {
				"settings" : settings
            });
		}
	};
});
