/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'pcorelib',
    'serverConfigBuilder'
], function ($, _, Backbone, pcorelib, ServerConfigBuilder) {
    'use strict';
    return pcorelib.RequestFactory.extend({

        token: null,

        buildRequestDataForKey: function(key,parameters) {
            if(key=='list_available_bookees') {
                var start = app.service.booking.bookingData.startDate;
                var end = app.service.booking.bookingData.endDate;
                var orgNr = (app.selectedOrga) ? app.selectedOrga.nr : app.data.userParameters.orgNr;
				var lat = null;
				var long = null;
				if(app.service.booking.bookingData.selectedPlace) {
					var lat = app.service.booking.bookingData.selectedPlace.geoRange.coordinate.latitude;
					var long = app.service.booking.bookingData.selectedPlace.geoRange.coordinate.longitude;
				}
                var params = {
                    "bookeeOrgNr":orgNr,
                    "limit": app.bookeeLimit != null ? '' + app.bookeeLimit : "500",
                    "start":start,
                    "end":end,
                    "criteria":{
                        "range":{
                            "coordinate":{
                                "latitude":lat,
                                "longitude":long
                            },
                            "rangeUnit":"KILOMETERS"
                        }
                    }
                };
				if(app.service.booking.bookingData.selectedRange) {
					params["criteria"]["range"]["range"] = app.service.booking.bookingData.selectedRange;
				}
				if(app.service.booking.bookingData.selectedFilter) {
					params["criteria"]["bookeeTypeNrs"] = app.service.booking.bookingData.selectedFilter.WTYP;
					params["criteria"]["wantedAttributeNrs"] = app.service.booking.bookingData.selectedFilter.WAT;
					params["criteria"]["unwantedAttributeNrs"] = app.service.booking.bookingData.selectedFilter.UWAT;
				}
				if(app.service.booking.bookingData.selectedStation!=null) {
					params["criteria"]["placeNr"] = app.service.booking.bookingData.selectedStation.placeNr;
					params["criteria"]["range"]["coordinate"]["latitude"] = app.service.booking.bookingData.selectedStation.latitude;
					params["criteria"]["range"]["coordinate"]["longitude"] = app.service.booking.bookingData.selectedStation.longitude;
					params["criteria"]["range"]["range"] = 0.5;
					delete params.criteria.bookeeTypeNrs;
					delete params.criteria.wantedAttributeNrs;
					delete params.criteria.unwantedAttributeNrs;
				}
                return params;
            }
			return parameters;
        },

        buildVerbosityForKey: function(key,parameters) {
            if(key=='list_available_bookees') {
                if(app.orgSettings.settings.showRemarks) {
                    return {
                        "BOOKEE_WITH_REMARK_INFO":true,
                        "PLACE_WITH_REMARK_INFO":true,
                        "BOOKEE_WITH_PLACE":true
                    }
                } else {
                    return {
                        "BOOKEE_WITH_REMARK_INFO":false,
                        "PLACE_WITH_REMARK_INFO":false,
                        "BOOKEE_WITH_PLACE":true
                    }
                }
            }
            if(key=='get_bookee_details') {
                return {
                    "BOOKEE_WITH_PLACE": true,
                    "BOOKEE_WITH_COMPLETE_REMARKS": true,
                    "PLACE_WITH_COMPLETE_REMARKS": true,
                    "BOOKEE_WITH_REALBOOKEE_DETAILS": true
                }
            }
            if(key=='book_prelim') {
                return {
                    "BOOKING_WITH_PRICE":false,
                    "PLACE_WITH_REMARK_INFO":false,
                    "PLACE_WITH_COMPLETE_REMARKS":false,
                    "PLACE_WITH_REMARK_CONTENTS":false
                }
            }
            if(key=='get_org_settings') {
                return {
                    "SETTINGS_WITH_GEO_MARKERS":true
                }
            }
			if(key=='list_bookings') {
				return {
					"BOOKEE_WITH_REMARK_INFO": true,
					"PLACE_WITH_REMARK_INFO": true,
					"BOOKEE_WITH_PLACE": true,
					"BOOKING_WITH_REMARKS": true
				}
			}
			return {
				"PLACE_WITH_REMARK_INFO":false,
				"PLACE_WITH_COMPLETE_REMARKS":false,
				"PLACE_WITH_REMARK_CONTENTS":false
			}
        },

        buildParametersForKey: function(key,parameters) {
			return {
				"authentication":app.auth.fullAuthentication.authentication,
				"verbosity": this.buildVerbosityForKey(key,parameters),
				"parameters": this.buildRequestDataForKey(key,parameters)
			}
        },

        initialize:function(){
            var self = this;
            pcorelib.RequestFactory.prototype.initialize.call(this);
            // http://10.20.4.145:9000/eif_1/providers/list_organisations
//            this.baseUrl = "https://eif-test-intern.cantamen.de/eif_1";
            var serverParameters = ServerConfigBuilder.buildParameters(ServerConfigBuilder.CONFIG_GVH);
            app.data.adminParameters = {
                "orgNr": serverParameters.orgNr,
                "password": serverParameters.password,
                "adminLogin": serverParameters.adminLogin
            };
            app.data.baseUrl = serverParameters.baseUrl;
            this.baseUrl = serverParameters.baseUrl;
//            this.baseUrl = "http://gvhapp.projektionisten.intern/www/xdomain_proxy.php?csurl=https://moca-test.cantamen.de/eif_1",


   		    this.requestBuilder['EWA'] = {
                buildRequestObject: function(requestObject,parameters) {
                    requestObject.data = parameters;
                    requestObject.type = 'POST';
                    requestObject.crossDomain = true;
                    requestObject.dataType = "json";
                    requestObject.contentType = "application/json";
                    requestObject.headers = {
                        Accept : "text/plain; charset=utf-8",
                        "Content-Type": "text/plain; charset=utf-8"
                    };
                    return requestObject;
                },
                buildRequestParams: function(key,requestObject,parameters) {
                    var op = self.overrideParams[key];
                    if(!op&&requestObject.buildParameters) {
						//console.log(JSON.stringify(parameters));
                        parameters = self.buildParametersForKey(key,parameters);
                    }
                    if(op) {
                        parameters = op;
                    }
                     var localParams = JSON.parse(JSON.stringify(parameters));
					console.log(JSON.stringify(localParams));
                    return JSON.stringify(localParams);
                },
                responseHandler: function(key,response,success) {
                    if(success) {
                        this.token = response.token;
                        //console.log(response);
                        this.trigger(key+':success', response);

                    } else {
                        this.token = null;
						this.trigger(key+':error', response);
                    }
                }
            };

            /*
             * Define Requests
             */
            
            this.requestObjects['execute_virtual_card_swipe'] = {
                requestType: 'EWA',
                url: '/bookings/execute_virtual_card_swipe',
				buildParameters: false
            };
            this.requestObjects['get_virtual_card_swipe_possibility'] = {
                requestType: 'EWA',
                url: '/bookings/get_virtual_card_swipe_possibility',
				buildParameters: false
            };

            this.requestObjects['list_bookings'] = {
                requestType: 'EWA',
                url: '/bookings/list_bookings',
				buildParameters: true
            };
            this.requestObjects['change_booking_prelim'] = {
                requestType: 'EWA',
                url: '/bookings/change_booking_prelim',
				buildParameters: true
            };
            this.requestObjects['add_booking_remark'] = {
                requestType: 'EWA',
                url: '/bookings/add_booking_remark',
				buildParameters: false
            };
			this.requestObjects['list_places_by_area'] = {
				requestType: 'EWA',
				url: '/bookings/list_places_by_area',
				buildParameters: true
			};
            this.requestObjects['list_available_bookees'] = {
                requestType: 'EWA',
                url: '/bookings/list_available_bookees',
                buildParameters: true,
                cacheRequest: false,
                useToken: false
            };
            this.requestObjects['get_bookee_details'] = {
                requestType: 'EWA',
                url: '/bookings/get_bookee_details',
                buildParameters: true
            };
            this.requestObjects['book_prelim'] = {
                requestType: 'EWA',
                url: '/bookings/book_prelim',
                buildParameters: true
            };
			this.requestObjects['cancel_booking_prelim'] = {
				requestType: 'EWA',
				url: '/bookings/cancel_booking_prelim',
				buildParameters: true
			};
			this.requestObjects['retract_booking'] = {
				requestType: 'EWA',
				url: '/bookings/retract_booking',
				buildParameters: true
			};
			this.requestObjects['confirm_booking'] = {
				requestType: 'EWA',
				url: '/bookings/confirm_booking',
				buildParameters: true
			};
            this.requestObjects['get_org_settings'] = {
                requestType: 'EWA',
                url: '/providers/get_org_settings',
                buildParameters: true
            };
            this.requestObjects['get_member'] = {
                requestType: 'EWA',
                url: '/members/get_member',
                parameters: app.auth.fullAuthentication,
                useToken: false
            };
            this.requestObjects['list_organisations'] = {
                requestType: 'EWA',
                url: '/providers/list_organisations',
                cacheRequest: false
            };
            this.requestObjects['list_cities'] = {
                requestType: 'EWA',
                url: '/bookings/list_cities',
                parameters: app.auth.adminAuthentication,
                cacheRequest: true
            };
            this.requestObjects['get_favorite_places_of_member'] = {
                requestType: 'EWA',
                url: '/members/get_favorite_places_of_member',
                parameters: app.auth.fullAuthentication,
                cacheRequest: false
            };
			this.requestObjects['set_favorite_places_of_member'] = {
				requestType: 'EWA',
				url: '/members/set_favorite_places_of_member',
				buildParameters: true
			};
            this.requestObjects['get_member_settings'] = {
                requestType: 'EWA',
                url: '/members/get_member_settings',
                parameters: app.auth.fullAuthentication
            };
            this.requestObjects['list_bookee_types'] = {
                requestType: 'EWA',
                url: '/bookings/list_bookee_types',
                parameters: app.auth.fullAuthentication,
                cacheRequest: true
            };
            this.requestObjects['list_attributes'] = {
                requestType: 'EWA',
                url: '/bookings/list_attributes',
                parameters: app.auth.fullAuthentication,
                cacheRequest: true
            };
            this.requestObjects['set_member_settings'] = {
                requestType: 'EWA',
                url: '/members/set_member_settings',
                parameters: app.auth.fullAuthentication,
                buildParameters: true
            };
            this.requestObjects['compute_ride_price_of_booking'] = {
                requestType: 'EWA',
                url: '/bookings/compute_ride_price_of_booking',
                parameters: app.auth.fullAuthentication
            };

            this.requestObjects['get_default_booking_model_for_orga'] = {
                requestType: 'EWA',
                url: '/bookings/get_default_booking_model_for_orga',
                parameters: app.auth.fullAuthentication
            };

            this.requestObjects['change_booking_prelim'] = {
                requestType: 'EWA',
                url: '/bookings/change_booking_prelim',
                parameters: app.auth.fullAuthentication
            };

            this.requestObjects['get_booking_change_data'] = {
                requestType: 'EWA',
                url: '/bookings/get_booking_change_data',
                parameters: app.auth.fullAuthentication
            };

            this.requestObjects['get_org_data'] = {
                requestType: 'EWA',
                url: '/providers/get_org_data',
                parameters: app.auth.fullAuthentication
            };
				
				this.requestObjects['list_booking_additional_data'] = {
					 requestType: 'EWA',
					 url: '/bookings/list_booking_additional_data',
					 parameters: app.auth.fullAuthentication
				};

        },

        requestPlacesForPosition: function(range,callback) {
            this.listenToOnce(app.requestFactory,'list_places_by_area:success',function(response){
                var placesCollection = new Backbone.Collection(response.result.places);
                // Places, die nicht zur aktuellen Organisation geh??ren, werden augefiltert und auf den Karte nicht dargestellt
               //  var orgNr = (app.selectedOrga != null) ? app.selectedOrga.nr : app.data.userParameters.orgNr;
               //  _.each(placesCollection.models, function(place) {
               //      if(place.attributes.orgnr != orgNr) {
               //          placesCollection.remove(place.cid);
               //      }
               //  });
                callback(range.coordinate,placesCollection);
            });
            this.listenToOnce(app.requestFactory,'list_places_by_area:error',function(response){
                callback(null);
            });
            app.requestFactory.performRequest('list_places_by_area',{
				"range": range
            },new Backbone.Collection());
        },

        updatePositionAndRequestPlaces: function(callback,range) {
            var self = this;
            if(range) {
                self.requestPlacesForPosition(range,callback);
            } else {
                app.accessLocationService().updateLocation(function(location){
                    self.requestPlacesForPosition({
                        "coordinate": {
                            "latitude": location.latitude,
                            "longitude": location.longitude
                        },
                        "range": 1,
                        "rangeUnit":"KILOMETERS"
                    },callback);
                });
            }
        },

        overrideParams: {}

    });
});
