/*global define*/

define([
    'jquery',
    'backbone',
    'pcorelib',
    '../views/login/OrganisationsView',
    '../views/search/NewBookingView',
    '../views/search/NewFilterView',
    '../views/bookees/BookeeListView',
    '../views/bookees/BookeeDetailView',
    '../views/bookees/BookeeView',
    '../views/bookings/MyBookingsView',
    '../views/search/StationFinderView',
    '../views/search/DeterminePlaceView',
    '../views/bookees/BookeeSuccessView',
    '../views/bookings/BookingSelectedView',
    '../views/search/ScrollTestView',
    '../views/settings/SettingsView',
    '../views/service/ServiceView',
    '../views/service/ImprintView',
	'../views/application/InfoView',
	'../components/InfoPanel',
	'moment'
], function ($, Backbone, pcorelib, OrganisationsView, NewBookingView, NewFilterView, ListAvailableBookeesView, BookeeDetailView, BookeeView, MyBookingsView, StationFinderView, DeterminePlaceView, BookeeSuccessView, BookingSelectedView, ScrollTestView, SettingsView, ServiceView, ImprintView, InfoView, InfoPanel, Moment) {
    'use strict';

    return pcorelib.BaseRouter.extend({

        routes: {
            "":"routeStartupPage",
            "scrollTest":"routeScrollTestPage",
			"noConnectionOrError":"noConnectionView",
            "login(/:place/:start/:end)":"routeStartupPage",
			"loginEmbedded(/:type/:x_coord/:y_coord/:place/:name/:start/:end)":"routeEmbeddedStartupPage",
            "reloginEmbedded":"routeEmbeddedStartupPage",
            "mainMenu":"routeMainMenu",
            "newBooking":"routeNewBooking",
            "newFilter":"routeNewFilter",
            "findStations":"showMapView",
            "newPlace(/:nextView)":"showDeterminePlaceView",
	        "bookies":"bookies",
            "bookiesEmbedded":"bookiesEmbedded",
            "bookeeSelected":"bookeeSelected",
            "bookeeDetails":"bookeeDetails",
            "infoView":"infoView",
            "showImage(/:src)":"showImageView",
			"myBookingsView":"myBookingsView",
            "bookeeSuccessView":"bookeeSuccessView",
            "bookingSelected":"bookingSelected",
            "settings":"settings",
            "service":"service",
            "imprint": "imprint"
        },

        performNavigation: function(options) {

            this.currentRouteParams = options.params;
            /*
            if(options.route=='imprint') {
                this.imprint();
            } else if(options.route=='service') {
                this.service();
            } else
            */
            if( options.route!=='newBooking'&&
                options.route!=='login'&&
                options.route!=='loginEmbedded'&&
                options.route!=='noConnectionOrError') {
                    isHomeView = false;
            } else {
                isHomeView = true;
            }
            if(options.route.indexOf('showImage') > -1) {
                console.log("ROUTEE1:",JSON.stringify(options));
                console.log("ROUTEE2:",JSON.stringify(options.route.split('/')));
                var image = options.route.split('/')[1];
                this.showImageView(image);
            } else {
                this.navigate(options.route, true);
            }
        },

        /*
         * Routes
         */

        routeScrollTestPage: function() {
            this.initAndShowView(
				new ScrollTestView(),
				{title:"ScrollTest", showBottomView: true, showBackButton: false}, ['999-err-msg']
			);
        },

		noConnectionView: function() {
			this.initAndShowView(
				new (pcorelib.BaseView.extend({
					hbsTemplate: 'NoConnectionView',
					alternateTemplatePath: './scripts/templates/login/',
					showWarning: function(message) {
						this.$el.find('#errorMessages').append((new pcorelib.AlertBoxView({model:{message:message}})).render().$el);
						$('#local_alert').alert();
					},
					render: function() {
						pcorelib.BaseView.prototype.render.call(this);
						this.$el.find('#noConnectionForm').css('display','block');
						this.showWarning('Keine Verbindung!');
						return this;
					},
                    buttonClicked: function(event,buttonId) {
                        if(buttonId=='reloadBtn') {
                            pcorelib.EventChannel.trigger('app:router:navigate',{route:'login'});
                        }
                    }
				}))(),
				{title:"999-err-msg:txt.title", showBottomView: false, showBackButton: false} , ['999-err-msg']
			);
		},

		myBookingsView: function() {
			this.initAndShowView(
				new MyBookingsView(),
				{title:"300-mybookings:txt.title", showBottomView: true, showBackButton: false}, ['100-home', '300-mybookings', '320-booking-info', '999-err-msg']
			);
		},
        routeMainMenu: function() {
            this.initAndShowView(
                new MainMenuView({
                    model: {
                        user:app.userData,
                        hasOnlyOneOrgas: (app.userOrgas && app.userOrgas.length <= 1),
                        orgas:app.userOrgas,
                        loginOrga : app.data.userParameters.orgNr
                    }
                }),
                {title:"100-home:txt.title", showBottomView: true},
                ['100-home','101-about', '999-err-msg']
            );
        },

		routeEmbeddedStartupPage: function(type,x_coord,y_coord,place,name,start,end) {
			app.startupPlace = undefined;
	        app.embeddedMode = true;
			if(place) {
                if(type=='STADTFLITZER') {
                    type = null;
                }
				app.startupType = type;
				app.startupX = x_coord;
				app.startupY = y_coord;
				app.startupPlace = place;
				app.startupPlaceName = name;
				app.startupStart = start;
                app.startupEnd = end;

				if (app.startupType == 'STADTMOBIL' || app.startupType == 'Stadtmobil') {
					app.service.booking.bookingData.startDate = app.service.booking.getDate('start',true,'iso8601', new Date(parseInt(start)));
					app.service.booking.bookingData.endDate = app.service.booking.getDate('end',true,'iso8601', new Date(parseInt(end)));
				} else {
					app.service.booking.bookingData.startDate = app.service.booking.getDate('start',false,'iso8601', new Date(parseInt(start)));
					app.service.booking.bookingData.endDate = app.service.booking.getDate('end',false,'iso8601', new Date(parseInt(end)));
                }
				$('#appContent').css('visibility','hidden');
			}
			var self = this;
			if(overrideLoginForTesting) {
				this.initAndShowView(
					new (pcorelib.BaseView.extend({
						buttonClicked: function(event,buttonId) {
							if(buttonId=="tb001") {
								self.showDeterminePlaceView();
							}
							if(buttonId=="tb002") {
								self.showMapView();
							}
							if(buttonId=="tb003") {
								self.infoView();
							}
						},
						hbsStringTemplate: '' +
							'<div class="btn-group-vertical" role="group" style="width: 100%">' +
							'<button id="tb001" type="button" class="btn btn-default">DeterminePlaceView</button>' +
							'<button id="tb002" type="button" class="btn btn-default">StationFinderView</button>' +
							'<button id="tb003" type="button" class="btn btn-default">InfoView</button>' +
							'</div>'
					}))(),
					{title:"TestView", showBottomView: true, showBackButton: false}, ['999-err-msg']
				);
    		} else {
                app.relogin = true;
    			if(app.applicationView.bottomView) {
    				app.applicationView.bottomView.close();
    			}
    			if($.i18n.isInitialized()){
    				this.initLoginView();
    			} else {
    				this.on("i18:initialized", this.initLoginView);
    			}
    		}
    	},

        routeStartupPage: function(place,start,end) {
            if(place) {
				app.startupPlace = place;
				app.startupStart = start;
				app.startupEnd = end;
			}
            var self = this;
            if(overrideLoginForTesting) {
                this.initAndShowView(
                    new (pcorelib.BaseView.extend({
                        buttonClicked: function(event,buttonId) {
                            if(buttonId=="tb001") {
                                self.showDeterminePlaceView();
                            }
                            if(buttonId=="tb002") {
                                self.showMapView();
                            }
                            if(buttonId=="tb003") {
                                self.infoView();
                            }
                        },
                        hbsStringTemplate: '' +
                        '<div class="btn-group-vertical" role="group" style="width: 100%">' +
                            '<button id="tb001" type="button" class="btn btn-default">DeterminePlaceView</button>' +
                            '<button id="tb002" type="button" class="btn btn-default">StationFinderView</button>' +
                            '<button id="tb003" type="button" class="btn btn-default">InfoView</button>' +
                        '</div>'
                    }))(),
                    {title:"TestView", showBottomView: true, showBackButton: false}, ['999-err-msg']
                );
            } else {
                if(app.applicationView.bottomView) {
                    app.applicationView.bottomView.close();
                }
                if($.i18n.isInitialized()){
                    this.initLoginView();
                } else{
                    this.on("i18:initialized", this.initLoginView);
                }
            }
        },

        initLoginView:function(){
            this.initAndShowView(
                new OrganisationsView(),
                {title:"000-login:txt.title", showBackButton: false}, ['999-err-msg']
            );
        },

        bookeeDetails: function() {
            this.initAndShowView(
                new BookeeDetailView(),
                {title:"320-booking-info:txt.title1", showBottomView: true, showInfoButton: false},
                ['320-booking-info', '321-booking-info-image', '999-err-msg']
            );
        },

        infoView: function() {
            this.initAndShowView(
                new InfoView(),
                {title:"100-home:btn.about", showBackButton: true, showInfoButton: false},
                ['000-login-help', '100-home', '101-about', '200-newbooking-help', '220-new-b-vehicles-help', '300-mybookings-help', '310-booking-status-help', '315-change-booking-time-help', '318-edit-booking-remark-help', '330-del-booking-review-help', '500-settings-help', '550-settings-loc-help', '550-settings-loc-new-help', '540-settings-filter-help', '546-settings-filter-edit-help', '550-settings-loc-edit-help', '550-settings-loc-find-help', '999-err-msg']
            );
        },

        showMapView: function() {
            this.initAndShowView(
                new StationFinderView(),
                {title:"550-settings-loc-find:txt.title", showBottomView: true}, ['550-settings-loc-find', '999-err-msg']
            );
        },

        showDeterminePlaceView: function(nextPage) {
            this.initAndShowView(
                new DeterminePlaceView({page:nextPage}),
                {title:"550-settings-loc-new:txt.title", showBottomView: true}, ['550-settings-loc-new', '999-err-msg']
            );
        },

        bookies: function() {
            this.initAndShowView(
                new ListAvailableBookeesView(),
                {title:"220-new-b-vehicles:txt.title3", showBottomView: true},
                ['220-new-b-vehicles', '999-err-msg']
            );
        },

        bookiesEmbedded: function() {
        	app.embeddedMode = true;
			if($.i18n.isInitialized()){
				this.bookies();
			} else{
				this.on("i18:initialized", this.bookies);
			}
        },

        routeNewBooking: function() {
            this.initAndShowView(
                new NewBookingView(),
                {title:"200-newbooking:txt.title", showBottomView: true, showBackButton: false},
                ['200-newbooking', '100-home', '550-settings-loc-edit', '999-err-msg']
            );
        },

        routeNewFilter:function(){
            this.initAndShowView(
                new NewFilterView(),
                {title:"546-settings-filter-edit:txt.title_n", showBottomView: true},
                ['546-settings-filter-edit', '540-settings-filter', '999-err-msg']
            );
        },

        bookeeSelected: function() {
            this.initAndShowView(
                new BookeeView(),
                {title:"230-new-b-review:txt.title1", showBottomView: true, showInfoButton: false},
                ['230-new-b-review','545-settings-filter-del', '320-booking-info', '999-err-msg']
            );
        },

        bookeeSuccessView: function() {
            this.initAndShowView(
                new BookeeSuccessView(),
                {title:"235-new-b-confirm:txt.title", showBottomView: true, showBackButton: false, showInfoButton: false},
                ['235-new-b-confirm','236-new-b-directions','320-booking-info', '321-booking-info-image', '999-err-msg']
            );
        },
        bookingSelected:function(){
            this.initAndShowView(
                new BookingSelectedView(),
                {title:"310-booking-status:txt.title", showBottomView: true, showBackButton: true},
                ['310-booking-status', '315-change-booking-time', '316-change-booking-time-review', '318-edit-booking-remark',
                    '310-booking-car-swipe', '330-del-booking-review', '999-err-msg']
            );
        },
        settings:function() {
            this.initAndShowView(
                new SettingsView(),
                {title:"500-settings:txt.title", showBottomView: true, showBackButton:false},
                ['500-settings', '540-settings-filter', '545-settings-filter-del', '546-settings-filter-edit',
                '550-settings-loc-del', '550-settings-loc-edit', '550-settings-loc-find', '550-settings-loc-new',
                '550-settings-loc', '999-err-msg']
            );

        },
        service:function() {
            this.initAndShowView(
            	new ServiceView(),
            	{title:"000-login:menu.service", showBottomView: true, showBackButton:true},
            	['999-err-msg']
            );
            //var ref = window.open(app.orgData.orgData.imprintUrl != "" ? app.orgData.orgData.imprintUrl : "http://www.projektionisten.de", '_blank', 'location=no,EnableViewPortScale=yes');
        },
        imprint:function() {
            this.initAndShowView(
                new ImprintView(),
                {title:"000-login:menu.imprint", showBottomView: true, showBackButton:true},
                ['999-err-msg']
            );
            //var ref = window.open(app.orgData.orgData.contactUrl != "" ? app.orgData.orgData.contactUrl : "http://www.projektionisten.de", '_blank', 'location=no,EnableViewPortScale=yes');
        },
        showImageView: function(image) {
            console.log('SHOWIMAGE:'+image);
            PhotoViewer.show(app.imageCache[image].image, app.imageCache[image].title,  {share:false});
        }

    });
});
