/*global define*/

define([
	'jquery',
	'underscore',
	'backbone',
	'templates',
	'pcorelib',
	'moment',
	'helper'
], function ($, _, Backbone, JST, pcorelib, Moment, Helper) {
	'use strict';
	return {

		/*
		 * -------------
		 */

        startDate: null,
		endDate: null,

		bookingModelByOrga: {
			"minBookingLengthISO8601": "P0D00H30M00S",
			"maxBookingLengthISO8601": "P22D00H00M00S",
			"intervalISO8601": "P0D00H30M00S"
		},

		bookingData: {
			startDate: null,
			endDate: null,
			selectedFilter: null,
			selectedPlace: null,
			selectedRange: null,
			selectedStation: null,
			selectedOrga: null
		},

		showPlaceWithStartupData: function(bookeeTypeNrs,placeNr) {
			var params = {
				"authentication": app.auth.fullAuthentication.authentication,
				"verbosity": {
					"BOOKEE_WITH_REMARK_INFO": true,
					"PLACE_WITH_REMARK_INFO": true,
					"BOOKEE_WITH_PLACE": true
				},
				"parameters": {
					"start": Helper._formatDateString(new Date(parseInt(app.startupStart)),'iso8601'),
					"end": Helper._formatDateString(new Date(parseInt(app.startupEnd)),'iso8601'),
					"criteria": {
						"range": {
							"coordinate": {
								latitude: app.startupY,
								longitude: app.startupX
							},
							"rangeUnit" : "KILOMETERS",
							"range" : 0.2
						}
					}
				}
			};
			if(bookeeTypeNrs) {
				params.parameters.criteria.bookeeTypeNrs = bookeeTypeNrs;
			}
			if(bookeeTypeNrs) {
				params.parameters.criteria.placeNr = placeNr;
			}
			app.requestFactory.overrideParams['list_available_bookees'] = params;
			pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookies',params:{noCache:true}});
		},

		updateBookingData: function(start,end,filter,place,range,station,orga) {
			this.startDate = start;
			this.endDate = end;
			this.bookingData.selectedOrga = orga;
			this.bookingData.startDate = this.getDate('start',true,'iso8601');
			this.bookingData.endDate = this.getDate('end',true,'iso8601');
			if(filter) {
				this.bookingData.selectedFilter = filter;
			}
			if(place) {
				this.bookingData.selectedPlace = place;
			}
			if(range) {
				this.bookingData.selectedRange = range;
			}
			if(station) {
				this.bookingData.selectedStation = station;
			} else {
				this.bookingData.selectedStation = null;
			}
		},

		getDate: function(dateType,rounded,returntype,baseDate,isFloor) {
			var bookingModel = this.getBookingModel();
			var localDate = null;
			if(dateType=='start') {
				if(baseDate) {
					this.startDate = baseDate;
				}
				if(this.startDate==null) {
					this.startDate = new Date();
				}
				localDate = rounded?Helper._getPreparedStartDate(this.startDate,this.endDate,bookingModel.interval.inMinutes(),0,isFloor):this.startDate;
				return Helper._formatDateString(localDate,returntype);
			}
			if(dateType=='end') {
				if(baseDate) {
					this.endDate = baseDate;
				}
				if(this.endDate==null) {
					this.endDate = new Date();
				}
				var diff = Math.floor((this.endDate.getTime() - this.startDate.getTime()) / 60000);
				localDate = rounded?Helper._getPreparedEndDate(this.startDate,this.endDate,bookingModel.interval.inMinutes(),bookingModel.min.inMinutes(),isFloor):this.endDate;
				return Helper._formatDateString(localDate,returntype);
			}
		},

		getBookingModel: function() {
			if(app.bookingModelByOrga) {
				this.bookingModelByOrga = app.bookingModelByOrga;
			}
			var diff = {
				days: "1000",
				hours: "00",
				minutes: "00",
				seconds: "00",
				inMinutes: function() {
					return (parseInt(this.days*24*60)+parseInt(this.hours*60)+parseInt(this.minutes));
				},
				inDifferenceFromNow: function() {
					return new Date(Math.round((new Date()).getTime() + (this.inMinutes()*60000)));
				}
			};

			return {
				min: Helper._convertDuration(this.bookingModelByOrga.minBookingLengthISO8601),
				max: Helper._maxBookingLength(Helper._convertDuration(this.bookingModelByOrga.maxBookingLengthISO8601), diff, false),
				interval: Helper._convertDuration(this.bookingModelByOrga.intervalISO8601)
			}
		},

		getBookingDetails: function(selectedBookee,callback) {
			var model = {
				selectedBooke: selectedBookee
			}
			this.listenToOnce(app.requestFactory, 'get_bookee_details:success', function (response) {
				model.bookeDetails = response.result.bookee;
				model.selectedBooke.bookeDetails = model.bookeDetails;
				callback(model,true);
			});
			this.listenToOnce(app.requestFactory, 'get_bookee_details:error', function (response) {
				callback(model,false);
			});
			app.requestFactory.performRequest('get_bookee_details', {"bookeeNr": model.selectedBooke.get('bookee').nr});
		},

		prepareBooking: function(selectedBookee,callback,constraints) {
			var self = this;
			var model = null;
            var requestData = {
                "start": null,
                "end": null,
                "bookeeNr": null
            };

            self.listenToOnce(app.requestFactory, 'book_prelim:success', function (response) {
                model.prelimError = false;
				model.prelim = {
					hash: response.result.booking.hash,
					nr: response.result.booking.nr,
					seqinorg: response.result.booking.seqinorg,
					result: response.result
				};
				callback(model,true);
			});

            self.listenToOnce(app.requestFactory, 'book_prelim:error', function (response) {
                if(response.hasOwnProperty("responseJSON")){
                    if(response.responseJSON.errorCode == 14){
                        response.prelimError = false;
                        response.batteryWarning = true;
                    }else{
                        response.prelimError = true;
                        response.batteryWarning = false;
                    }
                }else {
                    response.prelimError = true;
                    response.batteryWarning = false;
                }

				callback(response,false);
			});

			self.getBookingDetails(selectedBookee,function(newModel){
				model = newModel;
				requestData.start = selectedBookee.attributes.start;
                requestData.end = selectedBookee.attributes.end;
                requestData.bookeeNr = model.bookeDetails.nr;
				if(constraints){
					requestData.constraints = constraints;
				}
				/*
                if(selectedBookee.hasOwnProperty("batteryConstraint") && selectedBookee.batteryConstraint >= 0){
                    requestData.constraints = {"batteryCharge": selectedBookee.batteryConstraint};
                }
				*/
				app.requestFactory.performRequest('book_prelim',requestData);
			})
		},

		retractBooking: function(hash,callback) {
			var self = this;
			self.listenTo(app.requestFactory, 'retract_booking:success', function (response) {
				callback(response,true);
			});
			self.listenTo(app.requestFactory, 'retract_booking:error', function (response) {
				callback(response,false);
			});
			app.requestFactory.performRequest('retract_booking', {
				"hash": hash
			});
		},

		performBooking: function(hash,view,callback) {
			var self = this;
			self.listenToOnce(app.requestFactory, 'confirm_booking:success', function (response) {
				callback(response,true);
			});
			self.listenToOnce(app.requestFactory, 'confirm_booking:error', function (response) {
				callback(response,false);
			});
			var requestObj = {"hash": hash};
            if(view=="BookeeView"){
                var remarkPanel = $('#remarkPanel').find("input");
				if(remarkPanel && remarkPanel.val() && remarkPanel.val()!=""){
                    requestObj.bookingRemark = remarkPanel.val();
                }
				var promotionPanel = $('#promotionPanel').find("input");
                if(promotionPanel && promotionPanel.val() && promotionPanel.val()!=""){
                    requestObj.promotionCode = promotionPanel.val();
                }
					 
					 var $addProps = $('#addProp').find('input');
		 			if ($addProps) {
		 				requestObj.changeBooking = {additionalData: []};
		 				console.log($addProps);
		 				$addProps.each(function(i, addProp){
		 					var $addProp = $(addProp);
		 					console.log($addProp);
		 					if ($addProp.val() && $addProp.val()!='') {
		 						requestObj.changeBooking.additionalData.push({
		 							"typeNrInOrg": $addProp.attr('data-nrinorg'),
		 							"value": $addProp.val()
		 						})
		 					}
		 				});
		 			}
            }
            else if(view=="BookingSelectedView"){
                //Aus der BookingSelectedView auslesen
            }
			app.requestFactory.performRequest('confirm_booking', requestObj);
		},

		cancelBooking: function(orgNr,seqInOrg, callback) {
			var params = {
				"orgNr": orgNr,
				"seqInOrg":seqInOrg,
                "allowShortening": true
			};

			this.listenToOnce(app.requestFactory, 'cancel_booking_prelim:success', function (response) {
                callback({"hash": response.result.booking.hash})
			});
			this.listenToOnce(app.requestFactory, 'cancel_booking_prelim:error', function (response) {
				if(response.responseText.indexOf('NOOLDBOOKINGLOCK')!=-1) {
					pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
				} else {
					pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
				}
			});
			app.requestFactory.performRequest('cancel_booking_prelim', params);
		},

		isCarSwipePossible: function(model,callback) {
			var userParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
            userParams.parameters = {
				"orgNr":model.orgNr,
				"seqInOrg":model.seqInOrg
            };
			this.listenToOnce(app.requestFactory, 'get_virtual_card_swipe_possibility:success', function (response) {
				if(response.result && response.result.possibilities && response.result.possibilities.length > 0) {
					if(_.contains(response.result.possibilities,"DOOR_OPEN")&&_.contains(response.result.possibilities,"DOOR_CLOSE")) {
						callback(3);
					} else if(_.contains(response.result.possibilities,"DOOR_OPEN")) {
						callback(1);
					} else if(_.contains(response.result.possibilities,"DOOR_CLOSE")) {
						callback(2);
					} else {
						callback(0);
					}
				} else {
					callback(0);
				}
			});
			this.listenToOnce(app.requestFactory, 'get_virtual_card_swipe_possibility:error', function (response) {
                callback(false)
			});
            app.requestFactory.performRequest('get_virtual_card_swipe_possibility',userParams);
		},

		performCardSwipe: function(model,openCar) {
			console.log('CarSwipe: ' + (openCar?'öffnen':'schließen'));
			var userParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
            userParams.parameters = {
				"orgNr":model.orgNr,
				"seqInOrg":model.seqInOrg,
				"action":openCar?"DOOR_OPEN":"DOOR_CLOSE"
			};
			this.listenToOnce(app.requestFactory, 'execute_virtual_card_swipe:success', function (response) {
                pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
			});
			this.listenToOnce(app.requestFactory, 'execute_virtual_card_swipe:error', function (response) {
				pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
			});
			app.requestFactory.performRequest('execute_virtual_card_swipe',userParams);
			return true;
		}

	};
});
