/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'nicescroll'
], function($, _, Backbone, JST, pcorelib) {
    'use strict';
    return pcorelib.ApplicationView.extend({

        initialize: function() {
            pcorelib.ApplicationView.prototype.initialize.call(this, {
                headerView: this.buildHeaderView(),
                bottomView: this.buildBottomView(),
                loadingIndicatorImage: "./images/865707.gif"
            });
        },

        buildBottomView: function() {
            var menuItems = [
                { text: $.t("000-login:menu.bookings"), icon: 'glyphicon glyphicon-book', route: 'newBooking' },
                { text: $.t("000-login:menu.stationFinder"), icon: 'glyphicon glyphicon-map-marker', route: 'findStations' },
                { text: $.t("000-login:menu.settings"), icon: 'glyphicon glyphicon-cog', route: 'settings' }
            ];
            if (app.orgData && app.orgData.orgData) {
                if (app.orgData.orgData.imprintUrl && app.orgData.orgData.imprintUrl != "") {
                    menuItems.push({ text: $.t("000-login:menu.imprint"), icon: 'glyphicon glyphicon-info-sign', route: 'imprint' })
                }
                if (app.orgData.orgData.contactUrl && app.orgData.orgData.contactUrl != "") {
                    menuItems.push({ text: $.t("000-login:menu.service"), icon: 'glyphicon glyphicon-earphone', route: 'service' })
                }
            }
            menuItems.push({ text: $.t("000-login:menu.logout"), icon: 'glyphicon glyphicon-log-out', route: '' });
            return new pcorelib.BottomMenu({
                addClasses: 'app_bottom_height',
                addClassesSelector: '#appBottom',
                maxItems: 4,
                items: menuItems
            });
        },

        rebuildBottomView: function() {
            // todo bisschen dreckig. muss noch geklärt werden ob das probleme bereiten kann
            this.bottomView = this.buildBottomView();
            pcorelib.ApplicationView.prototype.render.call(this);
        },

        buildHeaderView: function() {
            return new (pcorelib.BaseView.extend({
                addClasses: 'app_header_height',
                hbsTemplate: 'HeadlineTemplateView',
                alternateTemplatePath: './scripts/templates/ui/',
                events: {
                    'touchstart #infoClick, touchstart .app_nav_btn': 'clickEventInfo',
                    'touchstart .backDiv, touchstart .backButton.app_nav_btn': 'clickEventBack'
                },
                clickEventInfo: function(ev) {
                    ev.stopPropagation();
                    pcorelib.EventChannel.trigger('app:router:navigate', { route: 'infoView' });
                },
                clickEventBack: function(ev) {
                    ev.stopPropagation();
                    pcorelib.EventChannel.trigger('app:router:back');
                }
            }))();
        },

        render: function() {
            var self = this;
            pcorelib.ApplicationView.prototype.render.call(this);
            
            if (app.embeddedMode) {
                $('#appBottom').css('height', '0px');
                $('#appBottom').css('display', 'none');
                $('#appHeader').css('height', '0px');
                $('#appHeader').css('display', 'none');
                $('#appContent').css('bottom', '0px');
                $('#appContent').css('top', '0px');
                $('.app_content_padding').css('padding', '0px');
            }
            
            if (typeof cordova !== 'undefined' && cordova !== null) {
                if (cordova.platformId == 'ios') {
                    if (!app.embeddedMode) {
                        $('#appHeader').css('padding-top', '15px');
                        $('#appContent').css('margin-top', '15px');
                    }
                }
            }

            $('#appContent').off('scroll').on('scroll', function() {
                if ($(this).scrollTop() + $(this).innerHeight() >= $(this)[0].scrollHeight) {
                    pcorelib.EventChannel.trigger('app:content:scroll:endpoint');
                }
            })
            
            return this;
        }

    });
});
