/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/collapsePanel',
    'components/InfoPanel'
], function ($, _, Backbone, JST, pcorelib, CollapsePanel, InfoPanel) {
    'use strict';
    return pcorelib.BaseView.extend({

        hbsTemplate: 'BookeeDetailView',
        alternateTemplatePath: './scripts/templates/bookees/',

        buttonClicked: function(event,buttonId) {
            event.preventDefault();
            var self = this;
            if(buttonId == 'embeddedBackButton') {
                window.history.back();
            }
        },

        build: function () {
            var self = this;

            if(!app.embeddedMode) {
                $('#embeddedBack').css('display','none');
            }

            this.model = {
                selectedBooke: app.selectedBookee
            }
            this.listenTo(app.requestFactory, 'get_bookee_details:success', function (response) {
                self.model.bookeDetails = response.result.bookee;
                this.trigger('view:initialized');
            });
            if(this.model.selectedBooke.get && this.model.selectedBooke.get('bookee')){
                app.requestFactory.performRequest('get_bookee_details', {"bookeeNr": this.model.selectedBooke.get('bookee').nr}, new Backbone.Collection());
            }
            else if(this.model.selectedBooke.bookee){
                app.requestFactory.performRequest('get_bookee_details', {"bookeeNr": this.model.selectedBooke.bookee.nr}, new Backbone.Collection());
            }
            else{
                console.log("Fehler mit dem ausgewählten Bookee: "+ this.model.selectedBooke);
            }
        },

        render: function () {
            pcorelib.BaseView.prototype.render.call(this);
            var self = this;
            /*
             * build & render vehicle-infos
             */
            var vehicleInfos = new InfoPanel({
                model: {
                    panelTitle: $.t('320-booking-info:lbl.vinfo'),
                    lines: [
                        {text: $.t('320-booking-info:txt.desc'), classes:'bookee_item_name bookee_line_info'},
                        {text: self.model.bookeDetails.displayName, classes:'bookee_item_info bookee_line_info'},
                        {text: self.model.bookeDetails.bookeeType.name, classes:'bookee_item_info bookee_line_info'}
                        //,{text: self.model.bookeDetails.isPool?'isPool':'noPool', classes:'bookee_item_info bookee_line_info'}
                    ]
                }
            });
            
            if(self.model.bookeDetails&&self.model.bookeDetails.fuelTypeName&&self.model.bookeDetails.fuelTypeName!="[]") {
                vehicleInfos.model.lines.push({text: self.model.bookeDetails.fuelTypeName, classes:'bookee_item_info bookee_line_info'});
            }
            if(!self.model.bookeDetails.isPool) {
                vehicleInfos.model.lines.push({text: self.model.bookeDetails.realBookeeColor, classes:'bookee_item_info bookee_line_info'});
                vehicleInfos.model.lines.push({text: self.model.bookeDetails.realBookeeName, classes:'bookee_item_info bookee_line_info'});
            } else {
                vehicleInfos.model.lines.push({text: $.t('320-booking-info:txt.pool'), classes:'bookee_item_info bookee_line_info'});
            } // Poolfahrzeug! Angaben können sich bis Fahrtantritt ändern

            var ExtendedVehicleInfos = vehicleInfos.ContentViewClass.extend({
                render: function () {
                    var localSelf = this;
                    var even = false;
                    localSelf.setTemplateFacoryIfNull(app.applicationView.templateFactory);
                    vehicleInfos.ContentViewClass.prototype.render.call(localSelf);
                    var remarks = self.model.bookeDetails.remarks;
                    for(var i = 0; i < remarks.length; i++){

                        var remark = remarks[i];

                        if(remark.contentType=='text/plain'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
                            localSelf.$el.append('<div class="bookee_item_info bookee_line_info">'+remark.content.replace(/(\r?\n)+/g, '<br>')+'</div>');
                        }
                        if(remark.contentType=='text/html'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
                            localSelf.$el.append("<div class='bookee_item_info bookee_line_info'>"+remark.content.replace(/(\r?\n)+/g, '<br>')+"</div>");
                        }
                        if( (remark.contentType=='image/jpeg'||remark.contentType=='image/png'||remark.contentType=='image/gif')
                            &&$.inArray('ADMINONLY', remark.infoType)==-1) {
										
										var image = "<img class='bookeeDetailImage' src='data:"+remark.contentType+";base64,"+remark.content+ "' style='max-width: 100%; display: block; margin-bottom: 1em;' />";
										 
										 localSelf.$el.append(image);
                        }
                    };
                    return this;
                }
            });
            vehicleInfos.contentView = new ExtendedVehicleInfos();
            vehicleInfos.renderTo(self.$el.find('#bookee_infos'));

            /*
             * build & render location-infos
             */
            var locationInfos = new InfoPanel({
                model: {
                    panelTitle: $.t('320-booking-info:lbl.pinfo'),
                    lines: [
                        {text: $.t('320-booking-info:txt.desc'), classes:'bookee_item_name bookee_line_info'}
                    ]
                }
            });
            var ExtendedLocationInfos = locationInfos.ContentViewClass.extend({
                events: {
                    'click img' : 'imgClicked'
                },
                render: function () {
                    var localSelf = this;
                    localSelf.setTemplateFacoryIfNull(app.applicationView.templateFactory);
                    locationInfos.ContentViewClass.prototype.render.call(localSelf);
                    var index = 0;
                    try {
	                    _.each(self.model.bookeDetails.place.remarks, function(remark){
	                        if(remark.contentType=='text/plain'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
	                            localSelf.$el.append("<div id='remark_"+remark.name+"' value='"+index+"' class='bookee_item_info bookee_line_info'>"+remark.content+"</div>");
	                        }
                            if(remark.contentType=='text/html'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
                                localSelf.$el.append("<div id='remark_"+remark.name+"' value='"+index+"' class='bookee_item_info bookee_line_info'>"+remark.content+"</div>");
                            }
	                        if(remark.contentType=='image/jpeg'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
	                            var image = "<img id='remark_"+remark.name+"' value='"+index+"' style='padding-top: 20px; height: auto; width: auto; max-width: 150px;' src='data:image/jpg;base64,"+remark.content+ "'/>";
	                            localSelf.$el.append(image);
	                        }
                            if(remark.contentType=='image/png'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
	                            var image = "<img id='remark_"+remark.name+"' value='"+index+"' style='padding-top: 20px; height: auto; width: auto; max-width: 150px;' src='data:image/png;base64,"+remark.content+ "'/>";
	                            localSelf.$el.append(image);
	                        }
                            if(remark.contentType=='image/gif'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
	                            var image = "<img id='remark_"+remark.name+"' value='"+index+"' style='padding-top: 20px; height: auto; width: auto; max-width: 150px;' src='data:image/gif;base64,"+remark.content+ "'/>";
	                            localSelf.$el.append(image);
	                        }
	                    });
                    } catch (err) {}
                    return this;
                },
                imgClicked: function(ev) {
                    var currentTimespamt = 'img_'+$.now();
                    app.imageCache[currentTimespamt] = {image:$(ev.target).context.src,title:''};
                    pcorelib.EventChannel.trigger('app:router:navigate',{route:'showImage/'+currentTimespamt});
                }
            });
            locationInfos.contentView = new ExtendedLocationInfos();
            try {
	            if(self.model.bookeDetails.place && self.model.bookeDetails.place.remarks.length>0) {
	                locationInfos.renderTo(self.$el.find('#bookee_infos'));
	            }
            } catch (err) {}
            if(app.embeddedMode){
	            setTimeout(function() {
		            var scrollHeight = document.getElementById('bookee_infos').scrollHeight;
		            $('#bookee_infos').css('height',(scrollHeight+50)+'px');
		            parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+50), '*');
		            pcorelib.EventChannel.trigger('app:busy:false');
	            }, 500);
			}
        }

    });
});
