/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/InfoPanel',
    'moment'
], function ($, _, Backbone, JST, pcorelib, InfoPanel, moment) {
    'use strict';
    return pcorelib.BaseView.extend({

        hbsTemplate: 'BookeeListView',
        alternateTemplatePath: './scripts/templates/bookees/',
        id: 'BookeeListView',
        sortedBookeeSuggestions: [],
        moreAvailable: false,
        requestDone: false,
        showBookeeType: true,

        build: function() {
            var self = this;

            if(!app.embeddedMode) {
                $('#embeddedBack').css('display','none');
            }

            this.contraints = [];
            this.model = {
                langGerman: (app.data.langParameters.selectedLang == "de")
            };
            app.bookeeLimit = 100;
            self.trigger('view:initialized');

            pcorelib.EventChannel.on('view:bookee:renderList', function() {

                if(!self.constraintRender) {
                    pcorelib.EventChannel.trigger('app:busy:true');
                    self.makeRequest(function(success) {
                        if(success) {
                            self.$el.find('#time_text').html('' + app.service.booking.getDate('start',true,'ger') + ' '+ $.t("220-new-b-vehicles:txt.until")+' ' + app.service.booking.getDate('end',true,'ger'));
                            self.renderList();
                        }
                    });
                }
                if(!app.summarizedBookingSuggestions) {
                    pcorelib.EventChannel.trigger('app:busy:false');
                }

            });

        },

        renderGroup: function(groupIndex, group) {

            var self = this;
            //app.applicationView.hideLoadingIndicator();
            var groupNr = 0;
            var groupName = 'Stadtflitzer';
            if(group[0].get('bookee').place) {
                var groupNr = group[0].get('bookee').place.nr;
                var groupName = group[0].get('bookee').place.name;
            }
            var distance = group[0].get('bookee').place.currentDistance.toFixed(2);
            var distanceUnit = app.constants.distanceUnit[app.orgSettings.settings.distanceUnit];
            var groupTitleWithDistance = '<div class ="panel_title_name">'+ groupName +'</div><div class ="panel_title_distance">'+ distance +' '+ distanceUnit +'</div>';

            var lines = [];
            var lineIndex = 0;
            _.each(group, function (model) {
                var iconColor = ewaIconColor;

                if (app.orgSettings.settings.showRemarks) {

                        var infoType = null;
                        if(model.get('bookee') && model.get('bookee').remarkInfos) {

                            for(var attr in model.get('bookee').remarkInfos) {

                                if(model.get('bookee').remarkInfos[attr].infoType.length>0) {
                                    if($.inArray("ALERT",model.get('bookee').remarkInfos[attr].infoType)||$.inArray("WARNING",model.get('bookee').remarkInfos[attr].infoType)){
                                        infoTypePlace = model.get('bookee').remarkInfos[attr].infoType;
                                    }
                                }
                            }

                        }

                        // infoTypePlace ist das kriterium zum einfärben des I's, je nach stationsbemerkung
                        var infoTypePlace;
                        if(model.get('bookee').place) {
                            for(var attr in model.get('bookee').place.remarkInfos) {

                                if(model.get('bookee').place.remarkInfos[attr].infoType.length>0) {
                                    if($.inArray("ALERT",model.get('bookee').place.remarkInfos[attr].infoType)||$.inArray("WARNING",model.get('bookee').place.remarkInfos[attr].infoType)){
                                        infoTypePlace = model.get('bookee').place.remarkInfos[attr].infoType;
                                    }
                                }
                            }
                        }
                        if (infoType == 'WARNING' || infoTypePlace == 'WARNING') {
                            iconColor = 'yellow';
                        }
                        if (infoType == 'ALERT' || infoTypePlace == 'ALERT') {
                            iconColor = 'red';
                        }

                }
                lines.push({
                    rIconColor: iconColor,
                    rLineClass: 'glyphicon glyphicon-info-sign',
                    lineId: 'lineName_'+lineIndex,
                    text: model.get('bookee').displayName,
                    classes: 'bookee_item_name bookee_line_info'
                });

                var bookeeType = self.showBookeeType?(model.get('bookee').bookeeType?(model.get('bookee').bookeeType.name):undefined):undefined;
                var bookeePool = (model.get('bookee').isPool?undefined:model.get('bookee').realBookeeName);
                var modelLine = (bookeePool?(bookeePool+" "+(bookeeType?("("+bookeeType+")"):"")):(bookeeType?(""+bookeeType+""):""));

                lines.push({
                    lineId: 'lineInfo_'+lineIndex,
                    text: modelLine,
                    classes: 'bookee_item_info bookee_line_info'
                });
                lineIndex++;
            });
            var panel = new InfoPanel({
                model: {
                    index: groupIndex,
                    group: groupNr,
                    panelTitle: groupName,
                    lines: lines,
                    groupTitleWithDistance: true
                },
                itemClicked: function (ev, value, group) {
                    if($(ev.target).hasClass('line_info_icon')) {
                        var teilId = ((ev.target.id).substring(9));
                        var bookeIndex = teilId.substring(0,teilId.length-5);
                        app.selectedBookee = ((app.summarizedBookingSuggestions)[''+group])[bookeIndex];
                        pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookeeDetails'});
                    } else {
                        var bookeIndex = (ev.target.id).substring(9);
                        app.selectedBookee = ((app.summarizedBookingSuggestions)[''+group])[bookeIndex];

                        if(!(app.selectedBookee)) {
                            pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.1'));
                        } else {
                            app.service.booking.prepareBooking(app.selectedBookee,function(model, isSuccess){
                                if(isSuccess) {
                                    app.prelimModel = model;
                                    app.prelimActive = true;
                                    pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookeeSelected'});
                                } else {
                                    if(model&&model.responseJSON&&model.responseJSON.errorCode==14) {
                                        self.handleBookingConstraint(true,model);
                                    } else if (model&&model.responseJSON&&model.responseJSON.errorCode) {
													pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+model.responseJSON.errorCode));
												} else {
                                        pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.1'));
                                        self.makeRequest(function(success) {
                                            self.renderList();
                                        });
                                    }
                                }
                            });
                        }
                    }
                }
            }).renderTo(self.$el.find('#item_list'));
            $(self.$el.find('.alternative_title')[groupIndex]).append(groupTitleWithDistance);
        },

        showErrorMessage: function (errorText) {
            if ($('#errorMessage').hasClass('hideErrorMessage')) {
                $('#errorMessage').removeClass('hideErrorMessage').addClass('showErrorMessage');
            }
            $('#errorMessage').find('.errorText').html(errorText);
            $('#errorMessage').find('button').html(Handlebars.compile('{{t "236-new-b-directions:btn.close"}}'));
        },

        hideErrorMessage: function () {
            if ($('#errorMessage').hasClass('showErrorMessage')) {
                $('#errorMessage').removeClass('showErrorMessage').addClass('hideErrorMessage');
            }
        },

        handleBookingConstraint: function(isFirst,constraintModel) {
            var self = this;
            self.constraintRender = true;
            if(isFirst){
                this.contraints = {};
            }
            if(constraintModel.responseJSON.hasOwnProperty("constraints")){
                console.log(constraintModel.responseJSON.constraints);
                if(!this.contraints.batteryCharge&&constraintModel.responseJSON.constraints.hasOwnProperty("batteryCharge")){
                    var batteryConstraintValue = constraintModel.responseJSON.constraints.batteryCharge;
                    this.contraints.batteryCharge = batteryConstraintValue;
                    this.showConstraintsModal('Geringe Akkuladung','Das Fahrzeug hat derzeit nur noch eine geringe Akkuladung. Möchten sie trotzdem fortfahren?');
                }
                if(!this.contraints.overlappingBookings&&constraintModel.responseJSON.constraints.hasOwnProperty("overlappingBookings")){
                    var boverlappingBookingsConstraintValue = constraintModel.responseJSON.constraints.overlappingBookings;
                    this.contraints.overlappingBookings = boverlappingBookingsConstraintValue;
                    this.showConstraintsModal('Achtung','Es existiert bereits eine Buchung für diese Kundennummer in dieser Buchungszeit. Trotzdem Buchen?');
                }
            }
        },

        showConstraintsModal:function(title,message){
            var self = this;
            self.model.constraintTitle = title;
            self.model.constraintMessage = message;
            self.render();
            setTimeout(_.bind(function () {
                $('#confirmConstraintsModal').modal('show');
                $('#modalConstraintsConfirm').off('click');
                $('#modalConstraintsConfirm').on('click', function () {
                    app.service.booking.prepareBooking(app.selectedBookee, function (model, isSuccess) {
                        self.constraintRender = false;
                        if (isSuccess) {
                            app.prelimModel = model;
                            pcorelib.EventChannel.trigger('app:router:navigate', { route: 'bookeeSelected' });
                        } else {
                            if (model && model.responseJSON && model.responseJSON.errorCode == 14) {
                                self.handleBookingConstraint(false, model);
                            } else {
                                pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.1'));
                                self.makeRequest(function (success) {
                                    self.render();
                                });
                            }
                        }

                    }, self.contraints);
                });
                $('#modalConstraintsCancel').on('click', function () {
                    self.constraintRender = false;
                    self.makeRequest(function (success) {
                        self.render();
                    });
                });
            }, this), 500);
        },

        render: function () {
            var self = this;
			if(app.embeddedMode) {
				$('#appContent').css('visibility','inherit');
			}
            pcorelib.BaseView.prototype.render.call(this);
            this.$el.find('#noCarText').css('visibility', 'hidden');
            this.$el.find('#bookeelist_time_info').css('visibility', 'hidden');
            this.$el.find('#item_list').css('visibility', 'hidden');
            pcorelib.EventChannel.trigger('app:busy:true');
            if(!app.prelimActive){
                if (!self.constraintRender) {
                    pcorelib.EventChannel.trigger('app:busy:true');
                    self.makeRequest(function (success) {
                        if (success) {
                            self.$el.find('#time_text').html('' + app.service.booking.getDate('start', true, 'ger') + ' ' + $.t("220-new-b-vehicles:txt.until") + ' ' + app.service.booking.getDate('end', true, 'ger'));
                            self.renderList();
                        }
                    });
                } else {
                    pcorelib.EventChannel.trigger('app:busy:false');
                }
                if(!app.summarizedBookingSuggestions) {
                    pcorelib.EventChannel.trigger('app:busy:false');
                }
            }
        },

        renderList: function(){
            pcorelib.EventChannel.trigger('app:busy:true');
            var self = this;
            var groupIndex = 0;
            this.$el.find('#item_list').empty('');
            _.each(self.sortedBookeeSuggestions, function (group) {
                var placeNr = group[0].attributes.bookee.place.nr;
                self.renderGroup(groupIndex,app.summarizedBookingSuggestions[placeNr]);
                groupIndex++;
            });
            if(groupIndex != 0) {
                this.$el.find('#noCarText').css('display', 'none');
            }
            if(app.embeddedMode){
                var scrollHeight = document.getElementById('innerContainer').scrollHeight;
                $('#innerContainer').css('height',(scrollHeight+50)+'px');
                parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+50), '*');
            }
            pcorelib.EventChannel.trigger('app:busy:false');
        },

        makeRequest: function(callback){
            var self = this;
            self.requestDone = false;
            function compareByDistance(a,b) {
                a = a[0].attributes.bookee.place.currentDistance;
                b = b[0].attributes.bookee.place.currentDistance;
                return  a > b ?  1
                    : a < b ? -1
                    :          0;
            }

            this.listenToOnce(app.requestFactory,'list_available_bookees:success',function(response){
                app.bookingSuggestions = new Backbone.Collection(response.result.bookingSuggestions);
                app.buildSummarizedBookees();

                self.moreAvailable = response.result.moreAvailable;

                self.sortedBookeeSuggestions = [];
                _.each(app.summarizedBookingSuggestions, function (group) {
                    self.sortedBookeeSuggestions.push(group);
                });
                self.sortedBookeeSuggestions.sort(compareByDistance);
                self.requestDone = true;
                this.$el.find('#noCarText').css('visibility', 'visible');
                this.$el.find('#bookeelist_time_info').css('visibility', 'visible');
                this.$el.find('#item_list').css('visibility', 'visible');
                callback(true);
            });

            this.listenTo(app.requestFactory,'list_available_bookees:error',function(response){
                self.requestDone = true;
                this.$el.find('#noCarText').css('visibility', 'visible');
                this.$el.find('#bookeelist_time_info').css('visibility', 'visible');
                this.$el.find('#item_list').css('visibility', 'visible');
                callback(false);
            });

            var noCache = false;
            if(app.applicationRouter.currentRouteParams) {
                noCache = app.applicationRouter.currentRouteParams.noCache;
            }
            //app.requestFactory.overrideParams['list_available_bookees'] = null;
            app.requestFactory.performRequest('list_available_bookees',null,new Backbone.Collection(),noCache);
        },

        buttonClicked: function(event,buttonId) {
            event.preventDefault();
            var self = this;
            if(buttonId == 'embeddedBackButton') {
                pcorelib.EventChannel.trigger('app:router:navigate',{route:'newBooking'});
            } else {
                app.bookeeLimit += 100;
                this.makeRequest(function(success) {
                    if(success) {
                        self.renderList();
                    }
                });
            }
        }

    });
});
