/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/collapsePanel',
    'components/InfoPanel',
    './BookeeCostPanelView',
    'moment'
], function ($, _, Backbone, JST, pcorelib, CollapsePanel, InfoPanel, BookeeCostPanelView, moment) {
    'use strict';
    return pcorelib.GoogleMapView.extend({

		hbsTemplate: 'BookeeSuccessView',
		alternateTemplatePath: './scripts/templates/bookees/',
		showBookeeType: true,

		events: {
			'click' : 'clickEvent',
			"click .panel-heading": "panelClicked"
		},

		panelClicked: function(ev) {
			$(ev.currentTarget).parent().find('.panel-body').toggle();
			this.calcRoute(
				new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude),
				new google.maps.LatLng(this.model.selectedBooke.get('bookee').place.latitude, this.model.selectedBooke.get('bookee').place.longitude),
				google.maps.TravelMode.WALKING,
				true
			);
			google.maps.event.trigger(this.map, "resize");
		},

		build: function() {
			var self = this;
			app.service.booking.getBookingDetails(app.selectedBookee,function(model){
				self.model = model;
				self.height = app.applicationView.getContentHeight()-65+'px';
				self.mapAttributes = {
					zoom: 15,
					mapTypeId: google.maps.MapTypeId.ROADMAP,
					center: new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude)
				};
				self.trigger('view:initialized');
			});
		},

		loadData: function() {
			var self = this;
			pcorelib.EventChannel.trigger('app:busy:true');
			app.accessLocationService().updateLocation(function(location){
				self.centerMapTo(new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude));
				self.updateMap();
				pcorelib.EventChannel.trigger('app:busy:false');
			});
		},

		buttonClicked: function(event,buttonId) {
			if(buttonId=='myBookingsBtn') {
                pcorelib.EventChannel.trigger('app:router:navigate',{route:'myBookingsView'});
			}
		},

        render: function () {
			var self = this;
            pcorelib.GoogleMapView.prototype.render.call(this);
			this.$el.find('.panel-body').hide();
			
			
			console.log(JSON.stringify(this.model.selectedBooke, null, "\t"));
				var realBookeeNameOrPoolNote = this.model.selectedBooke.bookeDetails.isPool ? $.t('320-booking-info:txt.pool') : "";
			    var bookeeType = self.showBookeeType?(this.model.selectedBooke.bookeDetails.bookeeType?(this.model.selectedBooke.bookeDetails.bookeeType.name):undefined):undefined;
                var bookeePool = this.model.selectedBooke.bookeDetails.realBookeeName;
                var modelLine = (bookeePool?(bookeePool+" "+(bookeeType?("("+bookeeType+")"):"")):(bookeeType?(""+bookeeType+""):""));
            var panel = new InfoPanel({
                model: {
                    panelTitle: this.model.selectedBooke.attributes.bookee.place.name,
                    lines: [
                        {text: '' + app.service.booking.getDate('start',true,'ger') + ' '+$.t("235-new-b-confirm:txt.until")+' ' + app.service.booking.getDate('end',true,'ger'), classes:'bookee_item_date bookee_line_info'},
                        {text: this.model.selectedBooke.bookeDetails.displayName, classes:'bookee_item_name bookee_line_info'},
                        {text: realBookeeNameOrPoolNote , classes:'bookee_item_info bookee_line_info'},
						{text: modelLine , classes:'bookee_item_info bookee_line_info'}
                    ]
                }
            }).renderTo(this.$el.find('#data_panels'));

			/*
			 * build & render location-infos
			 */
			var locationInfos = new InfoPanel({
				model: {
					toggleMode: true,
					toggleClosed: true,
					panelTitle: $.t('320-booking-info:lbl.pinfo'),
					lines: [
						{text: $.t('320-booking-info:txt.desc'), classes:'bookee_item_name bookee_line_info'}
					]
				}
			});
			var ExtendedLocationInfos = locationInfos.ContentViewClass.extend({
				events: {
					'click img' : 'imgClicked'
				},
				render: function () {
					var localSelf = this;
					localSelf.setTemplateFacoryIfNull(app.applicationView.templateFactory);
					locationInfos.ContentViewClass.prototype.render.call(localSelf);
					var index = 0;
					_.each(self.model.bookeDetails.place.remarks, function(remark){
						if(remark.contentType=='text/plain'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
							localSelf.$el.append("<div id='remark_"+remark.name+"' value='"+index+"' class='bookee_item_info bookee_line_info'>"+remark.content+"</div>");
						}
						if(remark.contentType=='text/html'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
							localSelf.$el.append("<div id='remark_"+remark.name+"' value='"+index+"' class='bookee_item_info bookee_line_info'>"+remark.content+"</div>");
						}
						if(remark.contentType=='image/jpeg'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
							var image = "<img id='remark_"+remark.name+"' value='"+index+"' style='padding-top: 20px; height: auto; width: auto; max-width: 150px;' src='data:image/jpg;base64,"+remark.content+ "'/>";
							localSelf.$el.append(image);
						}
                        if(remark.contentType=='image/png'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
							var image = "<img id='remark_"+remark.name+"' value='"+index+"' style='padding-top: 20px; height: auto; width: auto; max-width: 150px;' src='data:image/png;base64,"+remark.content+ "'/>";
							localSelf.$el.append(image);
						}
                        if(remark.contentType=='image/gif'&&$.inArray('ADMINONLY', remark.infoType)==-1) {
							var image = "<img id='remark_"+remark.name+"' value='"+index+"' style='padding-top: 20px; height: auto; width: auto; max-width: 150px;' src='data:image/gif;base64,"+remark.content+ "'/>";
							localSelf.$el.append(image);
						}
					});
					return this;
				},
				imgClicked: function(ev) {
					var currentTimespamt = 'img_'+$.now();
					app.imageCache[currentTimespamt] = {image:ev.currentTarget.currentSrc,title:''};
					pcorelib.EventChannel.trigger('app:router:navigate',{route:'showImage/'+currentTimespamt});
				}
			});
			locationInfos.contentView = new ExtendedLocationInfos();
			if(self.model.bookeDetails.place.remarks.length>0) {
				locationInfos.renderTo(self.$el.find('#bookee_infos'));
			}
			
            if(app.embeddedMode){
				setTimeout(function() {
					var scrollHeight = document.getElementById('bookeeSuccessForm').scrollHeight;
					$('#bookeeSuccessForm').css('height',(scrollHeight+50)+'px');
					parent.postMessage('CANTAMEN-FRAME-LOADED:'+app.getProvidedHeight(), '*');
					pcorelib.EventChannel.trigger('app:busy:false');
	            }, 500);
	        }

			return this;
        }

    });
});
