/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/collapsePanel',
    'components/InfoPanel',
    './BookeeCostPanelView',
    'moment',
    'helper'
], function ($, _, Backbone, JST, pcorelib, CollapsePanel, InfoPanel, BookeeCostPanelView, moment, Helper) {
    'use strict';
    return pcorelib.BaseView.extend({

        hbsTemplate: 'BookeeView',
        alternateTemplatePath: './scripts/templates/bookees/',
        bookingPerformed:false,
        batteryConstraint: null,
        rebuild: false,
        debug: false,
        showBookeeType: true,

        events: {
            "keypress #remarkPanel" : "keyPressed",
            "keypress #promotionPanel" : "keyPressed",
            'click' : 'clickEvent' // necessary to reregister event of parante view, because self.delegateEvents would otherwise only delegate the local events,
                                   // not the events of the parent view
        },

        build: function () {
            var self = this;
            // debugging code for simulating the error
            // if(self.debug){
            //     isSuccess = false;
            //     model.responseJSON = {
            //         errorCode: 14,
            //         constraints:{
            //             batteryCharge: 1200
            //         }
            //     };
            //     model.batteryWarning= true;
            //     self.debug = false;

            //     if(model.hasOwnProperty("prelim")) {
            //         app.service.booking.retractBooking(model.prelim.hash, function (response, success) {
            //         });
            //     }
            // }
            // debugging code end
            self.model = app.prelimModel;
				
				
				//Forced Booking Addprops
				self.listenTo(app.requestFactory, 'list_booking_additional_data:success', function (response) {
					self.model.additionalBookeeData = response.result.types;
					
					if (_.isEmpty(self.model.additionalBookeeData)) {
						return;
					}
					
					var addPropView = pcorelib.BaseView.extend({
						hbsTemplate: 'panelComment',
						alternateTemplatePath: './scripts/components/'
					});
					
					var propViews = [];
					_.each(self.model.additionalBookeeData, function(addProp) {
						propViews.push(new addPropView({model: {title: addProp.label, required: true, nrinorg: addProp.nrinorg}}));
					});
					
					self.renderPanelWithModel({
						panelTitle: $.t("230-new-b-review:lbl.addProp"),
						panelViews: propViews,
						panelId: 'addProp',
						panelOpen: true
					});
				});
				
				var userParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
				if(self.model.selectedBooke.get && self.model.selectedBooke.get('bookee')){
					userParams.parameters = {"bookeeNr": self.model.selectedBooke.get('bookee').nr};
					app.requestFactory.performRequest('list_booking_additional_data', userParams, new Backbone.Collection());
				}
				else if(self.model.selectedBooke.bookee){
					userParams.parameters = {"bookeeNr": self.model.selectedBooke.bookee.nr};
					app.requestFactory.performRequest('list_booking_additional_data', userParams, new Backbone.Collection());
				}
				else{
					 console.log("Fehler mit dem ausgewählten Bookee: "+ self.model.selectedBooke);
				}
				
				
            if(self.model.prelimError == false && !self.model.batteryWarning){
                self.trigger('view:initialized');

                var selectedStart = app.service.booking.bookingData.startDate;
                var selectedEnd = app.service.booking.bookingData.endDate;

                var serverStart = self.model.selectedBooke.attributes.start;
                var serverEnd = self.model.selectedBooke.attributes.end;

                if(serverStart != selectedStart || serverEnd != selectedEnd) {

                    var serverStartForm = Helper._formatDateString(new Date(serverStart),'ger');
                    var serverEndForm = Helper._formatDateString(new Date(serverEnd), 'ger');

                    $('.addPanelContent').html('<p style="color:orange"></p>');
                    $('.addPanelContent').find('p').html(Handlebars.compile('{{t "230-new-b-review:txt.warn.time"}}'));
                    $('#bookeeDataView').find('.bookee_item_date').css("text-decoration", "line-through");
                    $('#bookeeDataView').find('.bookee_item_date').append('</br> <p style="text-decoration:none; display:inline-block">  </p>');
                    $('#bookeeDataView').find('.bookee_item_date').find('p').html( serverStartForm + ' bis ' +  serverEndForm);
                }
                
            } else if(self.model.batteryWarning == true) {
                if(self.model.responseJSON.errorCode == 14){
                    if(model.responseJSON.hasOwnProperty("constraints")){
                        self.batteryConstraint = model.responseJSON.constraints.batteryCharge;
                        self.model.batteryConstraint = self.batteryConstraint / 100;

                    }
                }
                self.trigger('view:initialized');
            } else {
                pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.1'));
            }
        },

		buttonClicked: function(event,buttonId) {
			var self = this;
			if(buttonId=='bookBtn') {
                self.performBooking();
			}
		},

        performBooking: function(){
            var self = this;
            self.model = app.prelimModel;
            app.service.booking.performBooking(self.model.prelim.hash,"BookeeView",function(response,success){
                if(success) {
                    self.bookingPerformed = true;
                    pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookeeSuccessView'});
                } else {
                    pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
                }
                app.prelimActive = false;
            });
        },

		beforeClose: function() {
			var self = this;
            self.model = app.prelimModel;
			if(!self.model.prelimError&&!this.bookingPerformed && !self.rebuild == true) {
				app.service.booking.retractBooking(self.model.prelim.hash, function (response, success) {
                    app.prelimActive = false;
					if (success) {
                        self.trigger('view:readytoclose');
                        pcorelib.EventChannel.trigger('view:bookee:renderList');
                    }
				});
			} else {
				this.trigger('view:readytoclose');
			}
		},

        keyPressed: function(ev){
            if(ev.which == 13){
                ev.preventDefault();
                this.performBooking();
            }
        },

        /*
         * Data-Panels
         */

        renderPanelWithModel: function(panelModel) {
            var panel = new CollapsePanel({model:panelModel});
            this.$el.find('#bookee_panels').append(panel.$el);
            panel.render();
        },

        render: function () {
			var self = this;
            self.model = app.prelimModel;
            pcorelib.BaseView.prototype.render.call(this);

            // if a constraint as been sent from the server, show the modal dialog
            if(self.model.batteryWarning == true){
                self.showConstraintsModal();
            } else {
                self.rebuild = false;
                var sName = '';
                try {
                    sName = this.model.bookeDetails.place.name;
                } catch (err) {
                }
            
                var bookeeType = self.showBookeeType?(this.model.bookeDetails.bookeeType?(this.model.bookeDetails.bookeeType.name):undefined):undefined;
                var bookeePool = (this.model.bookeDetails.isPool?undefined:this.model.bookeDetails.realBookeeName);
                var modelLine = (bookeePool?(bookeePool+" "+(bookeeType?("("+bookeeType+")"):"")):(bookeeType?(""+bookeeType+""):""));
                console.log(JSON.stringify(this.model.bookeDetails,null, "\t"));

                new InfoPanel({
                    model: {
                        panelTitle: sName,
                        lines: [
                            {
                                text: '' + app.service.booking.getDate('start', true, 'ger') + ' bis ' + app.service.booking.getDate('end', true, 'ger'),
                                classes: 'bookee_item_date bookee_line_info'
                            },
                            {
                                text: this.model.bookeDetails.displayName,
                                classes: 'bookee_item_name bookee_line_info'
                            },
                            {
                                text: modelLine,
                                classes: 'bookee_item_info bookee_line_info'
                            }
                        ]
                    }
                }).renderTo(this.$el.find('#data_panels'));
                var TestView = pcorelib.BaseView.extend({
                    hbsTemplate: 'panelComment',
                    alternateTemplatePath: './scripts/components/'
                });
                if (app.orgSettings && app.orgSettings.settings.promotionCodesEnabled) {
                    this.renderPanelWithModel({
                        panelTitle: $.t("230-new-b-review:lbl.promotion"),
                        panelView: new TestView({model: {title: $.t("230-new-b-review:lbl.promotion")}}),
                        panelId: "promotionPanel"
                    });
                }
                this.renderPanelWithModel({
                    panelTitle: $.t("230-new-b-review:btn.remark"),
                    panelView: new TestView({model: {title: $.t("230-new-b-review:lbl.remark")}}),
                    panelId: "remarkPanel"
                });
                if (self.model.prelim && self.model.prelim.hash) {
                    this.renderPanelWithModel({
                        panelTitle: $.t("230-new-b-review:btn.cost"), panelView: new BookeeCostPanelView({
                            model: {
                                bookPrelimHash: self.model.prelim.hash,
                                title: $.t("230-new-b-review:lbl.dist"),
                                distUnit: app.constants.distanceUnit[app.orgSettings.settings.distanceUnit],
                                estimatedCost: ""
                            }
                        }), panelId: "costPanel"
                    });
                }
                if (self.model.prelimError) {
                    $('#bookBtn').prop('disabled', true);
                } else {
                    $('#bookBtn').prop('disabled', false);
                }

                if(app.embeddedMode){
                    setTimeout(function() {
                        var scrollHeight = document.getElementById('bookeeDataView').scrollHeight;
                        $('#bookeeDataView').css('height',(scrollHeight+50)+'px');
                        parent.postMessage('CANTAMEN-FRAME-LOADED:'+app.getProvidedHeight(), '*');
                        pcorelib.EventChannel.trigger('app:busy:false');
                    }, 500);
                }

            }
            // delegating events to recover the lost events of the two buttons after the battery dialogue
            self.delegateEvents();
	
        },

        showConstraintsModal:function(){
            var self=this;
            $('#confirmConstraintsModal').modal('show');
            $('#modalConstraintsConfirm').on('click',function(){
                // Save selected constraint in Bookee settings, to be transmitted in next call to "prepareBooking"
                app.selectedBookee.batteryConstraint = self.batteryConstraint;
                self.rebuild = true;
                // repeat build, to re-call"prepareBooking" and initialize other fields
                self.build();
            });
            $('#modalConstraintsCancel').on('click',function(){
                pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookies',params:{noCache:true}});
            });
        }

    });
});
