/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/collapsePanel',
    'components/InfoPanel',
    'moment',
    'helper',
    "handlebars",
    "./BookingRemarkPanelView"
], function ($, _, Backbone, JST, pcorelib, CollapsePanel, InfoPanel, Moment, Helper, Handlebars, BookingRemarkPanelView) {
    'use strict';
    return pcorelib.GoogleMapView.extend({

        hbsTemplate: 'BookingSelectedView',
        alternateTemplatePath: './scripts/templates/bookings/',

        events: {
            'click' : 'clickEvent',
            "click .panel-heading": "panelClicked",
            "dp.change #inputBookingStartContainer": "onDateChanged",
            "dp.change #inputBookingEndContainer": "onDateChanged"
        },

        updateMapContent: function() {
            this.calcRoute(
                new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude),
                new google.maps.LatLng(this.model.selectedBooking.bookee.place.latitude, this.model.selectedBooking.bookee.place.longitude),
                google.maps.TravelMode.WALKING,
                true
            );
            google.maps.event.trigger(this.map, "resize");
        },

        panelClicked: function(ev) {
            $(ev.currentTarget).parent().find('.panel-body').toggle();
            this.updateMapContent();
        },

        build: function () {
            var self = this;
            self.model = {
                selectedBooking: app.selectedBooking
            }

            if(!app.embeddedMode) {
                $('#embeddedBack').css('display','none');
            }

            //app.requestFactory.performRequest('get_booking_change_data',obj,new Backbone.Collection());

            this.listenToOnce(app.requestFactory,'get_booking_change_data:success',function(response){
                this.height = app.applicationView.getContentHeight()-65+'px',
                    this.mapAttributes = {
                        zoom: 15,
                        mapTypeId: google.maps.MapTypeId.ROADMAP,
                        center: new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude)
                    };

                this.model.bookingChangeData = response.result.changeData;
                this.model.allowedStartDate = Moment(this.model.bookingChangeData.minStart).toDate();
                this.model.allowedEndDate = Moment(this.model.bookingChangeData.maxEnd).toDate();
                self.trigger('view:initialized');
            });

            if(self.model && self.model.selectedBooking) {
                var userParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
                userParams.parameters = {"orgNr":self.model.selectedBooking.orgNr,"seqInOrg":self.model.selectedBooking.seqInOrg};
                app.requestFactory.performRequest('get_booking_change_data',userParams,new Backbone.Collection());
            }

        },

        loadData: function() {
            var self = this;
            pcorelib.EventChannel.trigger('app:busy:true');
            app.accessLocationService().updateLocation(function(location){
                self.centerMapTo(new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude));
                self.updateMap();
                pcorelib.EventChannel.trigger('app:busy:false');
            });
        },

        updateMapOverlay: function() {
            this.addCenterMarker('http://www.daftlogic.com/images/cross-hairs.gif',{coords:[0,0,0,0],type:'rect'});
        },

		buttonClicked: function(event,buttonId) {
            var self=this;
            event.preventDefault();
            if(buttonId=='embeddedBackButton') {
                pcorelib.EventChannel.trigger('app:router:navigate',{route:'newBooking'});
			}
			if(buttonId=='infoBtn') {
                app.selectedBookee = this.model.selectedBooking;
                pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookeeDetails'});
			}
            if(buttonId=='chgBookingBtn') {
                //pcorelib.EventChannel.trigger('app:router:navigate',{route:'myBookingsView'});
            }
            if(buttonId=='addRemarkBtn') {
                //pcorelib.EventChannel.trigger('app:router:navigate',{route:'myBookingsView'});
            }
            if(buttonId=='stornoBookingBtn') {


                app.service.booking.cancelBooking(this.model.selectedBooking.orgNr,this.model.selectedBooking.seqInOrg,
                    function(model){
                        self.model.stornoHash=model.hash;
                        self.showStornoModal();
                    }
                );
            }
		},


        /*
         * Data-Panels
         */

        renderPanelWithModel: function(panelModel) {
            var panel = new CollapsePanel({model:panelModel});
            this.$el.find('#bookee_panels').append(panel.$el);
            panel.render();
        },

        render: function () {
			var self = this;
            pcorelib.GoogleMapView.prototype.render.call(this);
            this.$el.find('.panel-body').hide();
            var dateContent='';

            this.renderBookingPanel();


            var curBookingRemark = (self.model.selectedBooking.remarks && self.findBookingRemark()!=null)?self.findBookingRemark():"";
            var remarkExists;
            if(curBookingRemark && curBookingRemark!=""){
                remarkExists=true;
            }
            else{
                remarkExists=false;
            }
            this.renderPanelWithModel({panelTitle: $.t("310-booking-status:txt.rem"), panelView: new BookingRemarkPanelView({
                model:{
                    title:$.t("310-booking_status:txt.rem"),
                    bookingRemark:curBookingRemark,
                    orgNr:self.model.selectedBooking.orgNr,
                    seqInOrg:self.model.selectedBooking.seqInOrg,
                    remarkExists:remarkExists
                }
            }),
                panelId:"remarkPanel"
            });

            if(app.embeddedMode){
				var scrollHeight = document.getElementById('divBookingSelectedView').scrollHeight;
	            $('#divBookingSelectedView').css('height',(scrollHeight+50)+'px');
	            parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+125+50), '*');
	            pcorelib.EventChannel.trigger('app:busy:false');
			}


            return this;
        },

        renderBookingPanel:function(){
            var self=this;
            var iconColor = ewaIconColor;
            if (app.orgSettings.settings.showRemarks) {
                if(this.model.selectedBooking.bookee && this.model.selectedBooking.bookee.remarkInfos&&this.model.selectedBooking.bookee.remarkInfos[0] && this.model.selectedBooking.bookee.remarkInfos[0].infoType) {
                    var infoType = this.model.selectedBooking.bookee.remarkInfos[0].infoType[0];
                    if (infoType == 'WARNING') {
                        iconColor = 'yellow';
                    }
                    if (infoType == 'ALERT') {
                        iconColor = 'red';
                    }
                }
            }
            var now = new Date();
            this.model.isBookingStarted = (now > Moment(self.model.selectedBooking.started).toDate());
            this.model.isBookingEnded = (now > Moment(self.model.selectedBooking.ended).toDate())
            var lines =[{text: '' + Helper._formatDateString(Moment(self.model.selectedBooking.started).toDate(),"ger") + ' '+$.t("310-booking-status:txt.until")+' ' + Helper._formatDateString(Moment(self.model.selectedBooking.ended).toDate(),"ger"), classes:'highlightText bookee_item_date bookee_line_info'}];
            if(!self.model.isBookingEnded && !self.model.selectedBooking.bookee.place.isPosition) { //Freefloater (isPosition = true) können nicht verlängert werden)
                lines.push(
                    {text: "<div class='hidden' id='changeDateContainer'></div>"},
                    {text: "<button type='button' id='changeDateBtn' class='noHighlightBtn btn btn-default'>"+ $.t("310-booking-status:btn.chg")+"</button>"},
                    {text: $.t("310-booking-status:txt.maxrange")+':', classes:'commentText bookee_item_info bookee_line_info'},
                    {text: '' + Helper._formatDateString(this.model.allowedStartDate,"ger") + ' '+$.t("310-booking-status:txt.until")+' ' + Helper._formatDateString(this.model.allowedEndDate,"ger"), classes:'commentText bookee_item_info bookee_line_info'},
                    {text: '<hr/>'}
                )
            }
            var realBookeeNameOrPoolNote = self.model.selectedBooking.bookee.isPool ? $.t('320-booking-info:txt.pool') : undefined;
            if(self.model.selectedBooking.bookee.place) {
                lines.push(
                    {
                        text: self.model.selectedBooking.bookee.place.name,
                        classes: '.bookee_item_name bookee_line_info',
                        rIconColor: iconColor,
                        rLineClass: 'glyphicon glyphicon-info-sign',
                        lineId: 'lineName_' + 0
                    },
                    {
                        text: self.model.selectedBooking.bookee.bookeeType.name,
                        classes: 'bookee_item_info bookee_line_info'
                    }
                );
                if(realBookeeNameOrPoolNote) {
                    lines.push(
                        {
                            text: realBookeeNameOrPoolNote,
                            classes: 'bookee_item_info bookee_line_info'
                        }
                    );
                }
                lines.push(
                    {
                        text: self.model.selectedBooking.bookee.realBookeeName,
                        classes: 'bookee_item_info bookee_line_info'
                    }
                );
            }
            else{
                lines.push(
                    {
                        text: self.model.selectedBooking.bookee.displayName,
                        classes: '.bookee_item_name bookee_line_info',
                        rIconColor: iconColor,
                        rLineClass: 'glyphicon glyphicon-info-sign',
                        lineId: 'lineName_' + 0
                    },
                    {
                        text: self.model.selectedBooking.bookee.bookeeType.name,
                        classes: 'bookee_item_info bookee_line_info'
                    },
                    {
                        text: self.model.selectedBooking.bookee.realBookeeName,
                        classes: 'bookee_item_info bookee_line_info'
                    }
                );
            }

            var bookingPanel = new InfoPanel({
                model: {
                    panelTitle: self.model.selectedBooking.bookee.displayName,
                    lines: lines
                },
                itemClicked: function (ev, value, group) {
                    if(ev.target.id=="changeDateBtn"){
                        if(self.$el.find('#changeDateContainer').hasClass('hidden')){
                            self.$el.find('#changeDateContainer').removeClass("hidden");
                            self.renderClocks();
                            self.$el.find('#changeDateBtn').addClass("hidden");
                        }
                    }
                    if(ev.target.id=="confirmChangeBtn"){
                        self.confirmChangeDate(true);
                    }
                    if(ev.target.id=="cancelChangeBtn"){
                        self.closeConfirmDate();
                    }
                    if($(ev.target).hasClass('line_info_icon')){
                        app.selectedBookee = self.model.selectedBooking;
                        pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookeeDetails'});
                    }

                }
            });
            var ExtendedBookingInfos = bookingPanel.ContentViewClass.extend({
                //hbsTemplate: 'BookingChangeDate',
                //alternateTemplatePath: './scripts/templates/components/',
                render: function () {
                    var localSelf = this;
                    localSelf.setTemplateFacoryIfNull(app.applicationView.templateFactory);
                    var template= Handlebars.compile(localSelf._templateFactory.retrieveTemplate('BookingChangeDate','./scripts/templates/bookings/'));
                    bookingPanel.ContentViewClass.prototype.render.call(localSelf);
                    localSelf.$el.find('#changeDateContainer').html(template);
                    localSelf.$el.find();
                    return this;
                }
            });
            bookingPanel.contentView = new ExtendedBookingInfos();
            //TODO herausfinden, ob id übergebbar
            self.$el.find('#data_panels li').empty();
            bookingPanel.renderTo(self.$el.find('#data_panels'));


            if(app.orgSettings && app.orgSettings.settings && app.orgSettings.settings.virtualCardSwipeAllowed){
                var cardSwipeView = new (pcorelib.BaseView.extend({
                    hbsTemplate: 'SimpleSelectBox',
                    alternateTemplatePath: './scripts/templates/components/',
                    _templateFactory: app.templateFactory,
                    buttonClicked: function(event,buttonId) {
                        if(buttonId == 'valABtn') {
                            this.$el.find("#"+buttonId).addClass('active').siblings().removeClass('active');
                            app.service.booking.performCardSwipe(self.model.selectedBooking,true);
                        }
                        if(buttonId == "valBBtn") {
                            this.$el.find("#"+buttonId).addClass('active').siblings().removeClass('active');
                            app.service.booking.performCardSwipe(self.model.selectedBooking,false);
                        }
                    }
                }))({model:{
                    displaySpinner: true,
                    title: '310-booking-car-swipe:txt.title',
                    valueA: '310-booking-car-swipe:txt.auf',
                    valueB: '310-booking-car-swipe:txt.zu',
                    displayValueA: false,
                    displayValueB: false,
                    hide: false
                }});
                self.$el.find('.addPanelContentA').html(cardSwipeView.render().$el);
                app.service.booking.isCarSwipePossible(this.model.selectedBooking,function(swipeMode){
                    if(swipeMode==3) {
                        cardSwipeView.model.displaySpinner = false;
                        cardSwipeView.model.displayValueA = true;
                        cardSwipeView.model.displayValueB = true;
                        cardSwipeView.render();
                    } else if(swipeMode==1) {
                        cardSwipeView.model.displaySpinner = false;
                        cardSwipeView.model.displayValueA = true;
                        cardSwipeView.model.displayValueB = false;
                        cardSwipeView.render();
                    } else if(swipeMode==2) {
                        cardSwipeView.model.displaySpinner = false;
                        cardSwipeView.model.displayValueA = false;
                        cardSwipeView.model.displayValueB = true;
                        cardSwipeView.render();
                    } else {
                        cardSwipeView.model.displaySpinner = false;
                        cardSwipeView.model.hide = false;
                        cardSwipeView.render();
                    }
                });
            }

        },

        confirmChangeDate: function (isFirst,constraints) {
            var self = this;
            this.listenToOnce(app.requestFactory, 'change_booking_prelim:success', function (response) {
                self.model.dateHash = response.result.booking.hash;
                self.renderConfirmDate();
            });
            this.listenToOnce(app.requestFactory, 'change_booking_prelim:error', function (response) {
                console.log('ErrorCode: ' + response.responseJSON.errorCode);
                if (response && response.responseJSON && response.responseJSON.errorCode == 14) {
                    self.handleBookingConstraint(isFirst, response);
                } else {
                    pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.' + response.responseJSON.errorCode));
                }
                //self.showConstraintsModal('Geringe Akkuladung','Das Fahrzeug hat derzeit nur noch eine geringe Akkuladung. Möchten sie trotzdem fortfahren?');
                //pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
            });
            var userParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
            userParams.parameters = {
                "orgNr": self.model.selectedBooking.orgNr,
                "seqInOrg": self.model.selectedBooking.seqInOrg,
                "changeData": {
                    "end": Helper._formatDateString(Moment(self.model.bookingEnd, "DD.MM.YYYY HH:mm").toDate(), "iso8601")
                }
            }
            userParams.parameters.changeData.start = Helper._formatDateString(Moment(self.model.bookingStart, "DD.MM.YYYY HH:mm").toDate(), "iso8601");
            if(constraints){
                userParams.parameters.changeData.constraints = constraints;
            }
            app.requestFactory.performRequest('change_booking_prelim', userParams, new Backbone.Collection());
        },

        handleBookingConstraint: function(isFirst,constraintModel) {
            var self = this;
            self.constraintRender = true;
            if(isFirst){
                this.contraints = {};
            }
            if(constraintModel.responseJSON.hasOwnProperty("constraints")){
                console.log(constraintModel.responseJSON.constraints);
                if(!this.contraints.batteryCharge&&constraintModel.responseJSON.constraints.hasOwnProperty("batteryCharge")){
                    var batteryConstraintValue = constraintModel.responseJSON.constraints.batteryCharge;
                    this.contraints.batteryCharge = batteryConstraintValue;
                    this.showConstraintsModal('Geringe Akkuladung','Das Fahrzeug hat derzeit nur noch eine geringe Akkuladung. Möchten sie trotzdem fortfahren?');
                }
                if(!this.contraints.overlappingBookings&&constraintModel.responseJSON.constraints.hasOwnProperty("overlappingBookings")){
                    var boverlappingBookingsConstraintValue = constraintModel.responseJSON.constraints.overlappingBookings;
                    this.contraints.overlappingBookings = boverlappingBookingsConstraintValue;
                    this.showConstraintsModal('Achtung','Es existiert bereits eine Buchung für diese Kundennummer in dieser Buchungszeit. Trotzdem Buchen?');
                }
            }
        },

        showConstraintsModal:function(title,message){
            var self = this;
            self.model.constraintTitle = title;
            self.model.constraintMessage = message;
            self.render();
            setTimeout(_.bind(function () {
                $('#confirmConstraintsModal').modal('show');
                $('#modalConstraintsConfirm').off('click');
                $('#modalConstraintsConfirm').on('click', function () {
                    self.confirmChangeDate(false,self.contraints);
                });
                $('#modalConstraintsCancel').on('click', function () {
                    self.constraintRender = false;
                });
            }, this), 500);
        },

        findBookingRemark:function(){
            var bookingRemark=null;
            if(this.model.selectedBooking.remarks){
                for(var i=0;i<this.model.selectedBooking.remarks.length;i++){
                    if(this.model.selectedBooking.remarks[i].types && this.model.selectedBooking.remarks[i].types.indexOf("SELFBILL")>-1){
                        bookingRemark = this.model.selectedBooking.remarks[i].remark;
                        return bookingRemark;
                    }
                }
            }
            return bookingRemark;
        },

        showStornoModal:function(){
            var self=this;
            $('#confirmStornoModal').modal('show');
            $('#modalStornoConfirm').on('click',function(){
                self.listenToOnce(app.requestFactory, 'confirm_booking:success', function (response) {
                    pcorelib.EventChannel.trigger('app:router:navigate',{route:'myBookingsView'});
                });
                self.listenToOnce(app.requestFactory, 'confirm_booking:error', function (response) {
                    pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
                });
                app.requestFactory.performRequest('confirm_booking', {
                    "hash": self.model.stornoHash
                });
            });
            $('#modalStornoCancel').on('click',function(){
                app.service.booking.retractBooking(self.model.stornoHash, function (response, success) {
                    if (success) {
                    }
                });
            });
        },

        renderConfirmDate:function(){
            var self=this;
            this.model.oldDate = Helper._formatDateString(Moment(self.model.selectedBooking.started).toDate(),"ger") + ' '+$.t("310-booking-status:txt.until")+' ' + Helper._formatDateString(Moment(self.model.selectedBooking.ended).toDate(),"ger");
            this.model.newDate = Helper._formatDateString(Moment(self.model.bookingStart,"DD.MM.YYYY HH:mm").toDate(),"ger") + ' '+$.t("310-booking-status:txt.until")+' ' + Helper._formatDateString(Moment(self.model.bookingEnd,"DD.MM.YYYY HH:mm").toDate(),"ger");
            $("#oldDate").text(this.model.oldDate);
            $("#newDate").text(this.model.newDate);
            $('#confirmChangeModal').modal('show');
            $('#modalDateConfirm').on('click',function(){
                app.service.booking.performBooking(self.model.dateHash,"BookingSelectedView",function(response,success){
                    if(success) {
                        self.model.selectedBooking.started = Helper._formatDateString(Moment(self.model.bookingStart,"DD.MM.YYYY HH:mm").toDate(),"iso8601");
                        self.model.selectedBooking.ended = Helper._formatDateString(Moment(self.model.bookingEnd,"DD.MM.YYYY HH:mm").toDate(),"iso8601");
                        self.renderBookingPanel();
                    }
                });
            });
            $('#modalDateCancel').on('click',function(){
                app.service.booking.retractBooking(self.model.dateHash, function (response, success) {
                    if (success) {
                        self.closeConfirmDate();
                    }
                });
            });
        },

        closeConfirmDate:function(){
            if(!this.$el.find('#changeDateContainer').hasClass('hidden')){
                this.$el.find('#changeDateContainer').addClass("hidden");
                this.$el.find('#changeDateBtn').removeClass("hidden");
            }
        },

        renderClocks_old:function(){
            var self = this;
            this.clock1 = $('#inputBookingStartContainerT').clockpicker({
                donetext: 'Done',
                autoclose: true,
                placement:'bottom',
                afterDone: function() {
                    if(!self.model.isBookingStarted){
                        self.validateClocks();
                    }
                },
                beforeShow: function() {
                    if(self.model.isBookingStarted){
                    }
                    else{
                        self.validateClocks();
                    }
                }
            });
            this.clock2 = $('#inputBookingEndContainerT').clockpicker({
                donetext: 'Done',
                autoclose: true,
                placement:'bottom',
                afterDone: function() {
                    self.validateClocks();
                },
                beforeShow: function() {
                    self.validateClocks();
                }
            });
            var enableStartDate = !self.model.isBookingStarted;


            var startDate= new Date(this.model.selectedBooking.started);//.substring(this.model.selectedBooking.started.length-5);
            var endDate = new Date(this.model.selectedBooking.ended);
            var timeSTr = Helper._formatDateString(startDate,"min");
            this.model.bookingStart = timeSTr;
            var timeSTrEnd = Helper._formatDateString(endDate,"min");
            this.model.bookingEnd = timeSTrEnd;
            var currentTime = Helper._formatDateString(startDate,"short");//this.model.selectedBooking.started.substring(10,this.model.selectedBooking.started.length-10);
            $('#inputBookingStartT').val(currentTime);
            var currentTime2 = Helper._formatDateString(endDate, "short");//this.model.selectedBooking.ended.substring(10,this.model.selectedBooking.ended.length-10);
            $('#inputBookingEndT').val(currentTime2);

            if(self.model.isBookingStarted){
                $( "#inputBookingStart" ).prop( "disabled", true );
                //$( "#inputBookingStartT" ).prop( "disabled", true );
                this.clock1.hide();
                $("#textTime").val(currentTime);
                if($("#textTime").hasClass("hidden")){
                    $("#textTime").removeClass("hidden");
                }
            }
            else{
                $( "#inputBookingStart" ).prop( "disabled", false );
                //$( "#inputBookingStart" ).prop( "disabled", false );
                //$( "#inputBookingStartT" ).prop( "disabled", false );
                if(!$("#textTime").hasClass("hidden")){
                    $("#textTime").addClass("hidden");
                }
            }

            var allowedStartDate = new Date(this.model.allowedStartDate.toString());
            var allowedEndDate = new Date(this.model.allowedEndDate.toString());
            allowedStartDate.setHours(0);
            allowedStartDate.setMinutes(0);

            allowedEndDate.setHours(0);
            allowedEndDate.setMinutes(0);
            this.dtp1 = $('#inputBookingStartContainer').datetimepicker({
                locale: 'de',
                //minDate: Moment(self.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate(),
                minDate: allowedStartDate,
                maxDate: allowedEndDate,
                format: "DD.MM.YYYY",//format: 'L',
                ignoreReadonly: enableStartDate,
                allowInputToggle: true,
                widgetPositioning:{horizontal: 'left'}
            });
            this.dtp2 = $('#inputBookingEndContainer').datetimepicker({
                locale: 'de',
                //minDate: Moment(self.model.bookingEnd, "DD.MM.YYYY HH:mm").utc().toDate(),
                minDate: allowedStartDate,
                maxDate: allowedEndDate,
                ignoreReadonly: true,
                allowInputToggle: true,
                format: "DD.MM.YYYY",//'L',
                widgetPositioning:{horizontal: 'left'}
            });

            $('#inputBookingStartContainer').data("DateTimePicker").date(startDate);
            $('#inputBookingEndContainer').data("DateTimePicker").date(endDate);


        },


        renderClocks: function() {
            var self = this;
            this.clock1 = $('#inputBookingStartContainerT').clockpicker({
                donetext: 'Done',
                autoclose: true,
                placement:'bottom',
                afterDone: function() {
                    self.validateClocks(true);
                },
                beforeShow: function() {
                    self.validateClocks();
                }
            });
            this.clock2 = $('#inputBookingEndContainerT').clockpicker({
                donetext: 'Done',
                autoclose: true,
                placement:'bottom',
                afterDone: function() {
                    self.validateClocks(true);
                },
                beforeShow: function() {
                    self.validateClocks();
                }
            });


            var startDate= new Date(this.model.selectedBooking.started);//.substring(this.model.selectedBooking.started.length-5);
            var endDate = new Date(this.model.selectedBooking.ended);
            var timeSTr = Helper._formatDateString(startDate,"min");
            this.model.bookingStart = timeSTr;
            var timeSTrEnd = Helper._formatDateString(endDate,"min");
            this.model.bookingEnd = timeSTrEnd;
            var currentTime = Helper._formatDateString(startDate,"short");//this.model.selectedBooking.started.substring(10,this.model.selectedBooking.started.length-10);
            $('#inputBookingStartT').val(currentTime);
            var currentTime2 = Helper._formatDateString(endDate, "short");//this.model.selectedBooking.ended.substring(10,this.model.selectedBooking.ended.length-10);
            $('#inputBookingEndT').val(currentTime2);

            //var startDate= new Date(this.model.selectedBooking.started);//.substring(this.model.selectedBooking.started.length-5);
            //var endDate = new Date(this.model.selectedBooking.ended);

            if(this.model.bookingStart) {
                var currentTime = this.model.bookingStart.substring(this.model.bookingStart.length-5);
                $('#inputBookingStartT').val(currentTime);
                var currentTime2 = this.model.bookingEnd.substring(this.model.bookingEnd.length-5);
                $('#inputBookingEndT').val(currentTime2);
            }
            this.dtp1 = $('#inputBookingStartContainer').datetimepicker({
                locale: 'de',
                //minDate: Moment(self.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate(),
                // bugfix: Date has to be set to beginning of the current Day
                // otherwise the plugin thinks, the clickable current day is "before" the minDate
                // because it checks in seconds, not days
                minDate: new Date().setHours(0,0,0,0),
                maxDate: app.service.booking.getBookingModel().max.inDifferenceFromNow(),
                format: 'L',
                ignoreReadonly: true,
                allowInputToggle: true,
                widgetPositioning:{horizontal: 'left'}
            });
            this.dtp2 = $('#inputBookingEndContainer').datetimepicker({
                locale: 'de',
                //minDate: Moment(self.model.bookingEnd, "DD.MM.YYYY HH:mm").utc().toDate(),
                minDate: new Date().setHours(0,0,0,0),
                maxDate: app.service.booking.getBookingModel().max.inDifferenceFromNow(),
                ignoreReadonly: true,
                allowInputToggle: true,
                format: 'L', widgetPositioning:{horizontal: 'left'}
            });

            $('#inputBookingStartContainer').data("DateTimePicker").date(startDate);
            $('#inputBookingEndContainer').data("DateTimePicker").date(endDate);

				// Startzeit Änderung ausblenden, da nicht möglich
				if (self.model.isBookingStarted) {
					self.$el.find('label[for=inputBookingStart]').parent().hide();
				}
				
        },

		isSameDay: function(date1,date2) {
			return (date1.getDate()==date2.getDate())&&
				(date1.getMonth()==date2.getMonth())&&
				(date1.getYear()==date2.getYear());
		},
		utcStartTime: function() {
			var self = this;
			var newStartTime = self.model.bookingStart.substring(0, self.model.bookingStart.length-5) + $('#inputBookingStartT').val();
			var newUtcTimeStart = (!self.isBookingStarted)? Moment(newStartTime, "DD.MM.YYYY HH:mm").utc().toDate():Moment(self.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate();
			return newUtcTimeStart;
		},

		utcEndTime: function() {
			var self = this;
			var newStartTime = self.model.bookingEnd.substring(0, self.model.bookingEnd.length-5) + $('#inputBookingEndT').val();
			var newUtcTimeStart = Moment(newStartTime, "DD.MM.YYYY HH:mm").utc().toDate();
			return newUtcTimeStart;
		},

        validateClocks: function() {
            var self = this;
            // Build new Current-Value
            var utcCurrent = Moment().utc().toDate();
            // Build new Start-Value
            var newUtcTimeStart = self.utcStartTime();
            var newUtcTimeEnd = self.utcEndTime();
            var isToday = self.isSameDay(utcCurrent,newUtcTimeStart);
            var standAndEndSameDay = self.isSameDay(newUtcTimeEnd,newUtcTimeStart);
            // Start-Time in past
            if((isToday||standAndEndSameDay)&&newUtcTimeStart.getTime()<utcCurrent.getTime() && !self.model.isBookingStarted) {
                // Update to min Start-Time
                newUtcTimeStart = utcCurrent;
            }
            // End-Time smaller than Start-Time
            if((isToday||standAndEndSameDay)&&newUtcTimeEnd.getTime()<=newUtcTimeStart.getTime()) {
                // Update to min Start-Time
                newUtcTimeEnd = newUtcTimeStart;
            }
            self.model.bookingStart = app.service.booking.getDate('start',true,'min',newUtcTimeStart, true);//Helper._formatDateString(newUtcTimeStart,"min");
            self.model.bookingEnd = app.service.booking.getDate('end',true,'min',newUtcTimeEnd);//Helper._formatDateString(newUtcTimeEnd,"min");
            $('#inputBookingStartT').val(self.model.bookingStart.substring(self.model.bookingStart.length-5));
            $('#inputBookingEndT').val(self.model.bookingEnd.substring(self.model.bookingEnd.length-5));
            self.updateAllowedHoursAndMinutes();
        },
        updateAllowedHoursAndMinutes: function() {
            var self = this;
            var interval = app.service.booking.getBookingModel().interval.inMinutes();
            var avMinutes = [];
            var startValue = 0;
            for (var i = 0; i < (60/interval); i++) {
                avMinutes.push(startValue+(i*interval));
            }
            var utcTimeStart = Moment(self.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate();
            var utcTimeEnd = Moment(self.model.bookingEnd, "DD.MM.YYYY HH:mm").utc().toDate();
            var utcCurrent = Moment().utc().toDate();
            var utcCurrentHours = utcCurrent.getHours();
            var startIsToday = self.isSameDay(utcCurrent,utcTimeStart);
            var endIsToday = self.isSameDay(utcCurrent,utcTimeEnd);
            var startEndSameDay = self.isSameDay(utcTimeStart,utcTimeEnd);
            var avHoursStart = null;
            if(startIsToday) {
                avHoursStart = [];
                for (var i = 0; i < 24; i++) {
                    if(i>=utcCurrentHours) {
                        avHoursStart.push(i);
                    }
                }
            }
            var avHoursEnd = null;
            if(startEndSameDay) {
                avHoursEnd = [];
                for (var i = 0; i < 24; i++) {
                    if(i>=utcTimeStart.getHours()) {
                        avHoursEnd.push(i);
                    }
                }
            }
            if(this.clock1!=null&& !this.model.isBookingStarted && startIsToday) {
                this.clock1.clockpicker('setAllowedHoursAndMinutes',avHoursStart,avMinutes);
            } else {
                this.clock1.clockpicker('setAllowedHoursAndMinutes',null,avMinutes);
            }
            if(this.clock2!=null) {
                if(endIsToday||startEndSameDay) {
                    this.clock2.clockpicker('setAllowedHoursAndMinutes',avHoursEnd,avMinutes);
                } else {
                    this.clock2.clockpicker('setAllowedHoursAndMinutes',null,avMinutes);
                }
            }
        },
        onDateChanged: function(ev) {
            if(this.initial) {
                return;
            }
            var self = this;
            if(ev.currentTarget.id=='inputBookingStartContainer') {
                if($('#inputBookingEndContainer').data("DateTimePicker")){
                    $('#inputBookingEndContainer').data("DateTimePicker").minDate(ev.date);
                }
                var timeSTr = Helper._formatDateString(new Date(ev.date.toDate()),"min");
                var newTime = timeSTr.substring(0,timeSTr.length-5) + $('#inputBookingStartT').val();
                this.model.bookingStart = newTime;
            }
            if(ev.currentTarget.id=='inputBookingEndContainer') {
                var timeSTr = Helper._formatDateString(new Date(ev.date.toDate()),"min");
                var newTime = timeSTr.substring(0,timeSTr.length-5) + $('#inputBookingEndT').val();
                this.model.bookingEnd = newTime;

            }
            if(this.initial==false) {
                self.validateClocks();
            }
        }
    });
});
