/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/InfoPanel',
    'moment',
	'helper',
], function ($, _, Backbone, JST, pcorelib, InfoPanel, Moment, Helper) {
    'use strict';
    return pcorelib.BaseView.extend({
        hbsTemplate: 'MyBookingsListView',
        alternateTemplatePath: './scripts/templates/bookings/',
        myBookings: null,
        selectedDate:null,
        shouldShowAboBookingsOption: false,
        shouldShowSubMembersOption: false,
        withAboBookings:false,
        withSubMembers:false,
        events:{
            'click' : 'clickEvent',
            'click #aboBookings':'aboBookingsClicked',
            'click #aboBookingsLabel':'aboBookingsClicked',
            'click #subMembers':'subMembersClicked',
            'click #subMembersLabel':'subMembersClicked',
            "dp.change #inputBookingsFromContainer": "onDateChanged"
        },
        build: function() {
            var self = this;
            self.selectedDate =new Date();

            // getting flags from the organization and member settings
            // dictating whether the abobookings and submembers options should be displayed
            if(app.orgSettings.settings.hasOwnProperty("showAboBookings")){
                self.shouldShowAboBookingsOption = app.orgSettings.settings.showAboBookings;
            };
            if(app.userData.member.hasOwnProperty("showAllMemberBookings")){
                self.shouldShowSubMembersOption = app.userData.member.showAllMemberBookings;
            };
            // forwarding thos flags to the template model
            self.model.shouldShowAboBookingsOption= self.shouldShowAboBookingsOption;
            self.model.shouldShowSubMembersOption = self.shouldShowSubMembersOption;

            self.sendRequest(true);
            this.listenTo(pcorelib.EventChannel,'app:content:scroll:endpoint',function(){
            	self.performNextPaging();
			});
        },

        autoPaging: true,
        pagingActive: true,
        currentPageIndex: 0,
        currentPageCount: 10,
        
        performNextPaging: function() {
            if(this.myBookings.bookings.length>=this.currentPageCount) {
                this.currentPageIndex = this.currentPageIndex + this.currentPageCount;
                this.requestItemPaging(false);
            }
        },

        renderPagingButton: function(selector,options) {
            var self = this;
            this.$el.find(selector?selector:'#additionalContent').append(
                (new (pcorelib.BaseView.extend({
                    hbsStringTemplate: '<button type="button" id="pageBtn" class="highlightBtn btn btn-default">Mehr</button>',
                    buttonClicked:function(event, buttonId){
                        self.performNextPaging();
                    },
                }))()).render().$el
            );
        },
        
        isExistingBookee: function(newItem) {
            var self = this;
            _.each(self.myBookings.bookings, function (item) {
                if(newItem.bookee.nr == item.bookee.nr) {
                    return true;
                }
            });
            return false;
        },

        requestItemPaging: function(initialize) {
            var self = this;
            this.listenToOnce(app.requestFactory, 'list_bookings:success', function (response) {
                if(self.pagingActive) {
                    if(self.myBookings) {
                        var localIndex = self.myBookings.bookings.length;
                        _.each(response.result.bookings, function (newItem) {
                            if(!self.isExistingBookee(newItem)) {
                                self.myBookings.bookings[localIndex] = newItem;
                                localIndex++;   
                            }

                        });

                    } else {
                        self.myBookings = response.result;
                    }
                } else {
                    self.myBookings = response.result;
                }
                if (initialize) {
                    this.trigger('view:initialized');
                }
                else {
                    this.updateView();
                }
            });
            this.listenToOnce(app.requestFactory, 'list_bookings:error', function (response) {
                pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
                self.myBookings = [];
                if (initialize) {
                    this.trigger('view:initialized');
                }
                else {
                    this.updateView();
                }
            });
            var localDate = this.selectedDate;
            localDate.setHours(0);
            localDate.setMinutes(0);
            localDate.setSeconds(0);
            localDate.setMilliseconds(0);
            var nowString = Moment(localDate).format("YYYY-MM-DDTHH:mm:ss.SSSZ");
            var obj={
                "startTime": nowString,
                // flag for submembers for server request
                "withSubMembers": this.withSubMembers
            };
            // flags for abobookings for server request
            if(this.withAboBookings) obj.bookingStates =["NORMAL", "ABO"];

            if(self.pagingActive) {
                obj.startList = self.currentPageIndex;
                obj.number = self.currentPageCount;
            };

            app.requestFactory.performRequest('list_bookings',obj,new Backbone.Collection());
        },

        sendRequest:function(initialize){
            this.requestItemPaging(initialize,-1);
        },

        buttonClicked:function(event, buttonId){
            if(buttonId=='newBookingBtn'){
                pcorelib.EventChannel.trigger('app:router:navigate',{route:'newBooking'});
					 this.$el.find('#showMyBookingsBtn').removeClass('active');
            }
        },

        aboBookingsClicked:function(event){
            this.withAboBookings = !this.withAboBookings;
            $("#aboBookings").prop("checked",this.withAboBookings);
            this.myBookings = null;
            this.currentPageIndex = 0;
            this.sendRequest(false);
        },

        subMembersClicked:function(event){
            this.withSubMembers = !this.withSubMembers;
            $("#subMembers").prop("checked",this.withSubMembers);
            this.myBookings = null;
            this.currentPageIndex = 0;
            this.sendRequest(false);
        },

        render: function () {
            pcorelib.BaseView.prototype.render.call(this);
            var self = this;
            pcorelib.EventChannel.trigger('app:busy:true');
            this.dtp1 = $('#inputBookingsFromContainer').datetimepicker({
                locale: 'de',
                /*minDate: new Date(),
                maxDate: app.service.booking.getBookingModel().max.inDifferenceFromNow(),*/
                ignoreReadonly: true,
                allowInputToggle: true,
                format: 'L',
                widgetPositioning:{horizontal: 'left'}
            });
            $('#inputBookingsFromContainer').data("DateTimePicker").date(this.selectedDate);

            this.showList();

            if(app.embeddedMode){
                var scrollHeight = document.getElementById('myBookingsListView').scrollHeight;
                $('#myBookingsListView').css('height',(scrollHeight+50)+'px');
                parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+125+50), '*');
 	            pcorelib.EventChannel.trigger('app:busy:false');
           }
            if(self.pagingActive&&!self.autoPaging) this.renderPagingButton();
            pcorelib.EventChannel.trigger('app:busy:false');
            return this;
        },

        showList:function(){
            var groupIndex = 0;
            var self=this;
            if(this.myBookings.bookings && this.myBookings.bookings.length>0){
                _.each(this.myBookings.bookings, function (item) {
                    var lines = [];
                    var start = Helper._formatDateString(Moment(item.started).toDate(),'ger');
                    var end = Helper._formatDateString(Moment(item.ended).toDate(),'ger');
                    var realBookeeNameOrPoolNote = item.bookee.isPool ? $.t('320-booking-info:txt.pool') : undefined;
                    try {
	                    lines.push({
	                        lineId: 'lineBooking_0',
	                        text: start +" "+ $.t("300-mybookings:txt.until")+" "+end,
	                        classes: 'bookee_item_date bookee_line_info'
	                    });
                    } catch (err) {}
                    // if(/* überprüfung ob kunde, der gebucht hat, unterkunde ist */) {
                    //     try {
                    //         lines.push({
                    //             lineId: 'lineBooking_1.1',
                    //             text: $.t("100-home:txt.subuser") + " " /* Kundennummer */,
                    //             classes: 'bookee_item_sub bookee_line_info'
                    //         });
                    //     } catch(err){}
                    // }
                    try {
	                    lines.push({
	                        lineId: 'lineBooking_1',
	                        text: item.bookee.place.name,
	                        classes: 'bookee_item_info bookee_line_info'
	                    });
                    } catch (err) {}
                    try {
                        if(realBookeeNameOrPoolNote) {
                            lines.push({
                                lineId: 'lineBooking_2',
                                text: realBookeeNameOrPoolNote,
                                classes: 'bookee_item_info bookee_line_info'
                            });
                        }
                    } catch (err) {}
                    try {
	                    lines.push({
	                        lineId: 'lineBooking_3',
	                        text: item.bookee.realBookeeName,
	                        classes: 'bookee_item_info bookee_line_info'
	                    });
                    } catch (err) {}
                    try {
	                    lines.push({
	                        lineId: 'lineBooking_4',
	                        text: item.bookee.bookeeType.name,
	                        classes: 'bookee_item_info bookee_line_info'
	                    });
                    } catch (err) {}

                    try {
                        if(item.bookee.isElectroMobile){
                            lines.push({
                                lineId: 'lineBooking_5',
                                text: self.showBatteryCharge(item.expectedBatteryCharge),
                                classes: 'bookee_item_comment bookee_line_info'
                            });
                        }
                    } catch (err) {}

                    try {
                        if(item.remarks && self.findBookingRemark(item)!=null){
                            lines.push({
                                lineId: 'lineBooking_6',
                                text: self.findBookingRemark(item),
                                classes: 'bookee_item_comment bookee_line_info'
                            });
                        }
                    } catch (err) {}

                    var panel = new InfoPanel({
                        model: {
                            index: 0,
                            group: groupIndex,
                            panelTitle: item.bookee.displayName,
                            lines: lines
                        },
                        itemClicked: function (ev, value, group) {
                            var bookingIndex = group;
                            app.selectedBooking = self.myBookings.bookings[bookingIndex];
                            pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookingSelected'});
                        }
                    }).renderTo(self.$el.find('#item_list'));
                    groupIndex++;
                });
                this.$el.find('#noBookingsText').css('display', 'none');
            } else {
                this.$el.find('#noBookingsText').css('display', 'block');
            }
        },

        findBookingRemark:function(item){
            var bookingRemark=null;
            if(item.remarks){
                for(var i=0;i<item.remarks.length;i++){
                    if(item.remarks[i].types && item.remarks[i].types.indexOf("SELFBILL")>-1){
                        bookingRemark = item.remarks[i].remark;
                        return bookingRemark;
                    }
                }
            }
            return bookingRemark;
        },

        updateView:function(){
            this.$el.find('#item_list').empty('');
            this.showList();
        },

        showBatteryCharge:function(text){
            var percent=parseInt(text)/100;
            var showText= "Voraussichtlicher Ladestand: "+percent+"%";
            return showText;
        },

        onDateChanged:function(event){
            if(event.currentTarget.id=='inputBookingsFromContainer') {
                if(event.oldDate !=null && event.date != event.oldDate){
                    var date =event.date.toDate();
                    if(date!=this.selectedDate){
                        this.selectedDate = date;
                        // Resetting list to get correct entries after date change
                        this.myBookings = null;
                        this.currentPageIndex = 0;
                        this.sendRequest(false);
                    }
                }
            }
        }
    });
});
