/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
	'moment'
], function ($, _, Backbone, JST, pcorelib, Moment) {
    'use strict';
    return pcorelib.BaseView.extend({

        hbsTemplate: 'OrganisationsView',
        alternateTemplatePath: './scripts/templates/login/',

		noConnectionScreen: false,

        build: function() {
            var self = this;

            window.onmessage = app.onMessage;

            app.deleteSessionData();
            this.listenToOnce(app.requestFactory,'list_organisations:success',function(response){
                this.stopListening();
                self.model = {
                    langGerman:(app.data.langParameters.selectedLang=="de"),
                    orgas:response.result,
                    hasOnlyOneOrgas: (response.result && response.result.orgas && response.result.orgas.length <= 1),
                    hasSubOrgas: false,
                    saveLogin: app.configuration.saveLoginData
                };
                if(app.configuration.saveLoginData) {
                    self.model.loginOrga = app.configuration.loginOrga;
                    self.model.loginName = app.configuration.loginName;
                    self.model.loginPassword = app.configuration.loginPassword;
                }
                self.trigger('view:initialized');

                $('#inputUsername').keypress(function(e) {
                    var code = (e.keyCode ? e.keyCode : e.which);
                    if (code == 13) $('#inputPassword').focus();
                });
                $('#inputPassword').keypress(function(e) {
                    var code = (e.keyCode ? e.keyCode : e.which);
                    if (code == 13) $('#saveDataSelect').focus();
                });
            });
			this.listenToOnce(app.requestFactory,'list_organisations:error',function(response){
				self.noConnectionScreen = true;
				self.trigger('view:initialized');
			});
            var adminParams = JSON.parse(JSON.stringify(app.auth.adminAuthentication));
            adminParams.parameters = {"usage":"MEMBER_LOGIN"};
            app.requestFactory.performRequest('list_organisations',adminParams,new Backbone.Collection());
        },

		render: function() {
			var self = this;
            pcorelib.BaseView.prototype.render.call(this);
            isHomeView = true;
            /*
            if(app.embeddedMode) {
                $('#appContent').css('visibility','visible');
                var scrollHeight = document.getElementById('noConnectionForm').scrollHeight;
                $('#noConnectionForm').css('height',(scrollHeight+50)+'px');
                parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+50), '*');
            }
            */
			if(this.noConnectionScreen) {
				//app.applicationRouter.navigate("noConnectionOrError", true);
                pcorelib.EventChannel.trigger('app:router:navigate',{route:'noConnectionOrError'});

			} else {
				this.$el.find('#isConnectedForm').css('display','block');
			}
			if(app.embeddedMode && app.startupPlace && !this.buttonAlreadyClicked) {
				self.buttonClicked(null,'loginBtn');
			}
            if(app.embeddedMode){
                try{
                    var scrollHeight = (document.getElementById('isConnectedForm') | document.getElementById('noConnectionForm')).scrollHeight;
                    $('#isConnectedForm').css('height',(scrollHeight+50)+'px');
                    $('#noConnectionForm').css('height',(scrollHeight+50)+'px');
                    parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+50), '*');
                }catch(e){};
            }

            self.showOrgaLoginText();

            $('#orgasSelect').off('change').on('change', function () {
                self.showOrgaLoginText();
            });

            $('#saveDataSelect').off('change').on('change', function () {
                if(this.value=='no'){
                    app.configuration.loginName = "";
                    app.configuration.loginPassword = "";
                    app.saveConfiguration();
                    self.model.saveLogin = false;
                    self.model.loginName = "";
                    self.model.loginPassword = "";
                    self.render();
                }
            });

            window.setTimeout(function () {
                pcorelib.EventChannel.trigger('splashscreen:hide');
            }, 500);

			$('#appContent').css('visibility','visible');
			if(app.embeddedMode) {
				this.showWarning('Bitte geben Sie Ihren Stadtmobil Benutzernamen und das zugehörige Passwort ein!');
			} else {
				this.showWarning('Bitte geben Sie Ihren Benutzernamen und das Passwort ein!');
			}

			return this;
		},

		showOrgaLoginText: function() {
			var self = this;
			var selectedOrgaNr = parseInt(self.$el.find('#orgasSelect').val());
			var selectedOrga = _.findWhere(self.model.orgas.orgas, {nr: selectedOrgaNr});
			self.$el.find('.marquee').remove();

			if (selectedOrga && typeof selectedOrga.appLoginText !== "undefined" && selectedOrga.appLoginText && selectedOrga.appLoginText.length > 1) {
				var text = selectedOrga.appLoginText.replace(/(<([^>]+)>)/ig,"");
				var duration = Math.max(text.length / 10, 5);
				var marqueeElement = $('<div class="marquee"><span class="first">'+text+'</span><span class="last">'+text+'</span></div>');
				marqueeElement.find('span.first').css('animation', 'marqueeIntro '+duration+'s linear, marquee '+duration+'s linear '+(duration * .75)+'s infinite');
				marqueeElement.find('span.last').css('animation', 'marquee '+duration+'s linear '+(duration * .25)+'s infinite');
				self.$el.prepend(marqueeElement);

			}
		},

        isLoginDataValid: function() {
        	try {
	            return app.data.userParameters.password.length>=3&&app.data.userParameters.user.length>=1;
        	} catch (err) {
        		return false;
        	}
        },

        showWarning: function(message) {
        	if(app.embeddedMode) {
				$('#appContent').css('visibility','visible');
	            var scrollHeight = (document.getElementById('isConnectedForm') | document.getElementById('noConnectionForm')).scrollHeight;
	            $('#isConnectedForm').css('height',(scrollHeight+50)+'px');
	            $('#noConnectionForm').css('height',(scrollHeight+50)+'px');
	            parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+50), '*');
			}
            this.$el.find('#errorMessages').html((new pcorelib.AlertBoxView({model:{message:message}})).render().$el);
            $('#local_alert').alert();
        },

        showSubOrgasIfNeeded: function (response) {
            var self = this;
            console.log(response.responseJSON);
            if(response.responseJSON.errorCode==15) {
                self.model.hasSubOrgas = true;
                self.model.lastLoginUser = response.responseJSON.candidates[0].nrInOrg;
                self.model.lastLoginPass = $('#inputPassword').val();
                self.model.lastLoginOrg = $('#orgasSelect').val();
                self.model.candidates = response.responseJSON.candidates;
                self.render();
                self.$el.find('#orgaSelectionDiv').hide();
                self.$el.find('#usernameDiv').hide();
                self.$el.find('#passwordDiv').hide();
                self.noSave = false;
            } else {
                self.noSave = true;
                self.model.hasSubOrgas = false;
                self.model.lastLoginUser = null;
                self.model.lastLoginPass = null;
                self.model.lastLoginOrg = null;
                self.showWarning($.t('000-login:err.usrnameorpasswrong'));
            }
        },

        buttonClicked: function(event,buttonId) {
            var self = this;
            this.buttonAlreadyClicked = true;
            if(buttonId=='loginBtn') {

                app.data.userParameters.orgNr = $('#orgasSelect').val();
                app.data.userParameters.password = $('#inputPassword').val();
                app.data.userParameters.user = $('#inputUsername').val();

                var tmpUser = $('#inputUsername').val();
                if($('#suborgasSelect').val()!=null&&$('#suborgasSelect').val()!=undefined&&self.model.lastLoginUser>0) {
                    app.data.userParameters.user = self.model.lastLoginUser + "-" + $('#suborgasSelect').val();
                    app.data.userParameters.orgNr = self.model.lastLoginOrg;
                    app.data.userParameters.password = self.model.lastLoginPass;
                }

                if(!this.isLoginDataValid()) {
                    $('#appContent').css('visibility','visible');
                    if(app.embeddedMode) {
                        this.showWarning('Bitte geben Sie Ihren Stadtmobil Benutzernamen und das zugehörige Passwort ein!');
                    } else {
                        this.showWarning('Bitte geben Sie Ihren Benutzernamen und das Passwort ein!');
                    }
                    return;
                }

                this.listenToOnce(app.requestFactory,'list_organisations:success',function (data){
                    app.userOrgas = data.result;
                    app.userToken = data.token;
                    app.requestFactory.performRequest('get_org_settings');
                });
                this.listenToOnce(app.requestFactory,'list_organisations:error',function(response){
					console.log('WRONG_LOGIN_DATA');
                    this.buttonAlreadyClicked = false;
                    self.showSubOrgasIfNeeded(response);
				});

                this.listenToOnce(app.requestFactory,'get_org_settings:success',function(data) {
                    app.orgSettings = data.result;

                    if(app.orgSettings.settings.defaultSearchRange.coordinate.latitude != null) {
                        app.accessLocationService().set('defaultLatitude',app.orgSettings.settings.defaultSearchRange.coordinate.latitude);
                    }
                    if(app.orgSettings.settings.defaultSearchRange.coordinate.longitude != null) {
                        app.accessLocationService().set('defaultLongitude',app.orgSettings.settings.defaultSearchRange.coordinate.longitude);
                    }

                    app.requestFactory.performRequest('get_member');
                });

				self.noSave = false;
                this.listenToOnce(app.requestFactory,'get_org_settings:error',function(response){
					console.log('WRONG_LOGIN_DATA');
                    this.buttonAlreadyClicked = false;
                    self.showSubOrgasIfNeeded(response);
				});
                this.listenToOnce(app.requestFactory,'get_member:error', function(response){
                    console.log('WRONG_LOGIN_DATA');
                    this.buttonAlreadyClicked = false;
                    self.showSubOrgasIfNeeded(response);
                });
                this.listenToOnce(app.requestFactory,'get_member:success',function(data) {
					if(!self.noSave&&$("#saveDataSelect")[0].selectedIndex==1) {
                        app.configuration.saveLoginData = true;
						app.configuration.loginName = $("#inputUsername").val();
						app.configuration.loginPassword = $("#inputPassword").val();
						app.configuration.loginOrga = $("#orgasSelect").val();
                        if(self.model.hasSubOrgas) {
                            app.configuration.loginName = self.model.lastLoginUser;
                            app.configuration.loginPassword = self.model.lastLoginPass;
                            app.configuration.loginOrga = self.model.lastLoginOrg;
                        }
						app.saveConfiguration();
					} else {
						app.configuration.saveLoginData = false;
					}
                    app.userData = data.result;

                    parent.postMessage('CANTAMEN_LOGIN_SUCCESS', '*');

					if(app.startupPlace) {
						if(app.startupType == 'STADTMOBIL') {
							var tmpPlace = {
                                geoRange: {
                                    coordinate: {
                                        latitude: undefined,
                                        longitude: undefined
                                    }
                                }
                            };
							tmpPlace.name = app.startupPlaceName;
							tmpPlace.geoRange.coordinate.longitude = app.startupX;
							tmpPlace.geoRange.coordinate.latitude = app.startupY;
							var place = JSON.parse(JSON.stringify(tmpPlace));
							app.userNewFavPlaces.push(place);
							app.service.booking.bookingData.selectedFilter = { 'NAME' : 'Alle Fahrzeuge' };
							app.service.booking.bookingData.selectedPlace = place;
							app.service.booking.bookingData.selectedRange = 0.5;
							pcorelib.EventChannel.trigger('app:router:navigate',{route:'newBooking'});
						} else {
                           	var tmpPlace = {
                                geoRange: {
                                    coordinate: {
                                        latitude: undefined,
                                        longitude: undefined
                                    }
                                }
                            };
							tmpPlace.name = app.startupPlaceName;
							tmpPlace.geoRange.coordinate.longitude = app.startupX;
							tmpPlace.geoRange.coordinate.latitude = app.startupY;
							var place = JSON.parse(JSON.stringify(tmpPlace));
							app.userNewFavPlaces.push(place);
							app.service.booking.bookingData.selectedFilter = { 'NAME' : 'stadtflitzer_intern' };
							app.service.booking.bookingData.selectedPlace = place;
							app.service.booking.bookingData.selectedRange = 0.1;
                            app.service.booking.showPlaceWithStartupData([746]);
                            isHomeView = true;
						}

					} else {
						//app.applicationRouter.navigate("newBooking", true);
                        pcorelib.EventChannel.trigger('app:router:navigate',{route:'newBooking'});
					}
                });

				var userParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
				userParams.parameters = {"usage":"BOOKING"};
				app.requestFactory.performRequest('list_organisations',userParams,new Backbone.Collection());

            }
            if(buttonId=='langDeBtn' || buttonId == "langEnBtn") {
                $("#"+buttonId).addClass('active').siblings().removeClass('active');
                this.model.loginOrga = $("#orgasSelect").val();
                this.model.loginName = $("#inputUsername").val();
                this.model.loginPassword = $("#inputPassword").val();
                this.model.saveLogin = ($("#saveDataSelect")[0].selectedIndex==1);
                app.setLanguage(event.target.value);
                this.model.langGerman = (app.data.langParameters.selectedLang=="de");
            }
        }



    });
});
