/*global define*/

define([
	'jquery',
	'underscore',
	'backbone',
	'templates',
	'pcorelib'
], function ($, _, Backbone, JST, pcorelib) {
	'use strict';
	return pcorelib.GoogleMapView.extend({

		hbsTemplate: 'DeterminePlaceView',
		alternateTemplatePath: './scripts/templates/search/',
		addClasses: 'app_content_nopadding',
		addClassesSelector: '#appContent',
		autoReverseGeocoding: true,

		currentSelectedAddress: null,

		events: {
			'click #searchByGeocoder' : 'clickEvent',
			'click #saveByGeocoder': 'clickEvent'
		},

		tmpPlace: {
			"geoRange": {
				"coordinate": {
					"latitude": 52.3882319,
					"longitude": 9.7183679
				},
				"range": 1,
				"rangeUnit": "KILOMETERS"
			},
			"name": "Projektionisten",
			"defaultPlace": false
		},

        initialize: function(options) {
            this.model.options = options;
            if(this.model.options.page && this.model.options.page!=""){
                this.model.isNewbooking = (this.model.options.page.toLowerCase() == "newbooking");
            }
            _.bindAll(this, 'render');
        },

		build: function() {
            this.mapAttributes = {
                zoom: 15,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                center: new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude)
            };
			this.trigger('view:initialized');
		},

        render:function(){
			//alert(app.getProvidedHeight());
			var self = 	this;
            this.height = app.applicationView.getContentHeight()-65+'px';
			if(app.embeddedMode){
				this.height = app.getProvidedHeight()-65+'px';
			}
            pcorelib.GoogleMapView.prototype.render.call(this);
			if(app.embeddedMode){
	            setTimeout(function() {
		            var scrollHeight = app.getProvidedHeight();
					$('#map-canvas').css('height',(scrollHeight-65)+'px');
		            parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight), '*');
		            pcorelib.EventChannel.trigger('app:busy:false');
					self.updateMap();
	            }, 500);
			}
        },

		loadData: function() {
			var self = this;
			pcorelib.EventChannel.trigger('app:busy:true');
			app.accessLocationService().updateLocation(function(location){
				self.centerMapTo(new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude));
				self.updateMap();
				pcorelib.EventChannel.trigger('app:busy:false');
			});
		},

		updateMapOverlay: function() {
			this.addCenterMarker('http://www.daftlogic.com/images/cross-hairs.gif',{coords:[0,0,0,0],type:'rect'});
		},

		reverseGeocodingFinished: function(address) {
			this.updateAddressBox(address);
		},

		updateAddressBox: function(address) {
			this.currentSelectedAddress = address;
			this.$el.find('#addressInput').val(address.getAddressString());
		},

		buttonClicked: function(event,buttonId) {
			var self = this;
			if(buttonId=='searchByGeocoder') {
				var searchStr = $('#addressInput').val();
				pcorelib.EventChannel.trigger('app:busy:true');
				self.performGeocoding(searchStr,function(address,status){
					if (status == google.maps.GeocoderStatus.OK) {
						self.updateAddressBox(address);
						self.map.setCenter((self.currentGeocodingAddress.get('result')).geometry.location);
					}
					pcorelib.EventChannel.trigger('app:busy:false');
				});
			}
			if(buttonId=='saveByGeocoder') {
				$("#savePlaceModal").modal("show");
                $('#modalSave').off('click');
                $('#modalSave').on('click', function(){
					if (self.currentSelectedAddress) {
						self.tmpPlace.name = $('#nameInput').val();
						if (!self.tmpPlace.name) {
							self.tmpPlace.name = self.currentSelectedAddress.getAddressComponent('route');
						}
						self.tmpPlace.geoRange.coordinate.latitude = self.currentSelectedAddress.get('result').geometry.location.lat();
						self.tmpPlace.geoRange.coordinate.longitude = self.currentSelectedAddress.get('result').geometry.location.lng();
						var place = JSON.parse(JSON.stringify(self.tmpPlace));
						app.service.booking.bookingData.selectedPlace = place;
						var newPlacesArray = JSON.parse(JSON.stringify(app.userFavPlaces.places));
						// iterating over places to filter out the temporary ones that aren't supposed to be sent to the server
						for (var i = newPlacesArray.length - 1; i >= 0; i--) {
							if (newPlacesArray[i].tempPlace && newPlacesArray[i].tempPlace == true) {
								newPlacesArray.splice(i, 1);
							}
						};
						newPlacesArray.push(place);
						app.requestFactory.performRequest('set_favorite_places_of_member', {
							"places": newPlacesArray
						});

						self.listenToOnce(app.requestFactory, 'set_favorite_places_of_member:success', function (response) {
							if (self.model.options && self.model.options.page) {
								app.userFavPlaces.places = newPlacesArray;
								pcorelib.EventChannel.trigger('app:router:navigate', {route: self.model.options.page});
							} else {
								pcorelib.EventChannel.trigger('app:router:navigate', {route: "newbooking"});
							}
						});
						self.listenToOnce(app.requestFactory, 'set_favorite_places_of_member:error', function (response) {
							pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
							pcorelib.EventChannel.trigger('app:router:navigate', {route: 'newBooking'});
						});
					}
				});
				$('#modalNoSave').on('click', function () {
					if (self.currentSelectedAddress) {
						self.tmpPlace.name = $('#nameInput').val();
						if (!self.tmpPlace.name) {
							self.tmpPlace.name = self.currentSelectedAddress.getAddressComponent('route');
						}
						self.tmpPlace.geoRange.coordinate.latitude = self.currentSelectedAddress.get('result').geometry.location.lat();
						self.tmpPlace.geoRange.coordinate.longitude = self.currentSelectedAddress.get('result').geometry.location.lng();
						self.tmpPlace.tempPlace = true;
						var place = JSON.parse(JSON.stringify(self.tmpPlace));
						app.userNewFavPlaces.push(place);
						app.service.booking.bookingData.selectedPlace = place;
						pcorelib.EventChannel.trigger('app:router:navigate', {route: 'newBooking'});
					}
				});
			}
		}
	});
});
