/*global EWAAppNeu, Backbone, JST*/

define([
		'jquery',
		'underscore',
		'backbone',
		'templates',
		'pcorelib',
		'bootstrapdtp',
		'clockpicker',
		'moment',
		'../components/SelectionView',
		'helper'
	],

	function ($, _, Backbone, JST, pcorelib, pdt, cp, Moment, SelectionView, Helper) {
		'use strict';
		return pcorelib.BaseView.extend({

			hbsTemplate: 'NewBookingView',
			alternateTemplatePath: './scripts/templates/search/',

			clock1: null,
			clock2: null,
			dtp1: null,
			dtp2: null,
			initial: true,
			selectOrgasField: false,

			events: {
				'click' : 'clickEvent',
				"dp.change #inputBookingStartContainer": "onDateChanged",
				"dp.change #inputBookingEndContainer": "onDateChanged"
			},


			build: function() {
				var self = this;
				var i = 6;

				/* ___ listeners ___ */
				// get favorite places success
				this.listenToOnce(app.requestFactory,'get_favorite_places_of_member:success',function(response){
					app.userFavPlaces = response.result;
					--i;
					self.prepareView(i);
				});
				// get favorite places error
                this.listenToOnce(app.requestFactory,'get_favorite_places_of_member:error',function(){
                    console.log("error requesting places");
                    // Adding the current location
                    app.userFavPlaces = {"places":[]};
                    --i;
                    self.prepareView(i);
                });
                // get settings success
				this.listenToOnce(app.requestFactory,'get_member_settings:success',function(response){
					app.userSettings = response.result;
					--i;
					self.prepareView(i);
				});
				// get settings error
				this.listenToOnce(app.requestFactory,'get_member_settings:error',function(response){
					--i;
					self.prepareView(i);
				});
				// get default booking model success
				this.listenToOnce(app.requestFactory,'get_default_booking_model_for_orga:success',function(response){
					app.bookingModelByOrga = response.result.bookingModel;
					--i;
					self.prepareView(i);
				});
				// get default booking model error
				this.listenToOnce(app.requestFactory,'get_default_booking_model_for_orga:error',function(response){
					--i;
					self.prepareView(i);
				});
				// get bookee types success
				this.listenToOnce(app.requestFactory,'list_bookee_types:success',function(response){
	                app.vehicleBookeeTypes = response.result;
	                --i;
	                self.prepareView(i);
	            });
	            // get bookee types error
	            this.listenToOnce(app.requestFactory,'list_bookee_types:error',function(response){
	                --i;
	                self.prepareView(i);
	            });
	            // get bookee attributes success
	            this.listenToOnce(app.requestFactory,'list_attributes:success',function(response){
	                app.vehicleAttributes = response.result;
	                --i;
	                self.prepareView(i);
	            });
	            // get bookee attributes error
	            this.listenToOnce(app.requestFactory,'list_attributes:error',function(response){
	                --i;
	                self.prepareView(i);
	            });
	            // get organisation data success
	            this.listenToOnce(app.requestFactory, 'get_org_data:success', function(response) {
	            	app.orgData = response.result;
					app.applicationView.rebuildBottomView();
	            	--i;
	                self.prepareView(i);
	            });
	            // get organisation data error
	            this.listenToOnce(app.requestFactory, 'get_org_data:error', function(response) {
	            	--i;
	                self.prepareView(i);
	            });
					// get organisation settings
					this.listenToOnce(app.requestFactory,'get_org_settings:success',function(data) {
						app.selectedOrga.settings = data.result.settings;
					});

				/* ___ requests ___ */
				// get favorite places
				var fullAuth = jQuery.extend(true, {}, app.data.userParameters, app.data.adminParameters);
				app.requestFactory.performRequest('get_favorite_places_of_member', null, new Backbone.Collection());

				// get settings
				var userParamsSettings = JSON.parse(JSON.stringify(app.auth.fullAuthentication));

				var bkfilter = [];
				// userOrgas-Filter
				if (app.userOrgas && app.userOrgas.orgas) {
					_.each(app.userOrgas.orgas, function(orga) {
						bkfilter.push('bkfilter-' + orga.nr)
					});
				} else {
					bkfilter.push('bkfilter-' + (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr))
				}

				userParamsSettings.parameters = {"keys":bkfilter};
				app.requestFactory.performRequest('get_member_settings', userParamsSettings, new Backbone.Collection());

				// get default booking model
				var bookingModelParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
				bookingModelParams.parameters = {"bookeeOrgNr": (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr)};
				app.requestFactory.performRequest('get_default_booking_model_for_orga', bookingModelParams, new Backbone.Collection());

				// get bookee types
				var userParamsTypes = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
	            userParamsTypes.parameters = {"bookeeOrgNr": (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr)};
	            app.requestFactory.performRequest('list_bookee_types', userParamsTypes, new Backbone.Collection());

	            // get bookee attributes
	            app.requestFactory.performRequest('list_attributes', userParamsTypes, new Backbone.Collection());

	            // get organisation data
	            app.requestFactory.performRequest('get_org_data', null, new Backbone.Collection());
			},

			prepareView:function(i){
				if(i==0){
					/*
					 * Prepare initial vehicle-filter
					 */
					var rangeList = app.data.rangeParameters;

					var favPlaces = app.userFavPlaces?app.userFavPlaces.places:[];
					_.each(app.userNewFavPlaces,function(place){
						favPlaces.push(place);
					});
                    if(app.userFavPlaces.places) {
                        // adding standard address if no places could be fetched
                        if(app.userFavPlaces.places.length==0){
                            var standardPlaceLatitude =
                                (app.orgSettings.settings.defaultSearchRange.coordinate.latitude != null)
                                ?app.orgSettings.settings.defaultSearchRange.coordinate.latitude
                                :app.accessLocationService().get('defaultLatitude');
                            var standardPlaceLongitude =
                                (app.orgSettings.settings.defaultSearchRange.coordinate.longitude != null)
                                    ?app.orgSettings.settings.defaultSearchRange.coordinate.longitude
                                    :app.accessLocationService().get('defaultLongitude');
                            app.userFavPlaces.places.push(
                                    {
                                        "geoRange": {
                                            "coordinate": {
                                                "latitude": standardPlaceLatitude,
                                                "longitude": standardPlaceLongitude
                                            },
                                            "range": app.orgSettings.settings.defaultSearchRange.range,
                                            "rangeUnit": app.constants.distanceUnit[app.orgSettings.settings.defaultSearchRange.rangeUnit]
                                        },
                                        "name": "Standard Adresse",
                                        "defaultPlace": false,
                                        // Flag for filtering
                                        tempPlace: true
                                    }
                            );
                        };
                        // Adding the current location
                        app.userFavPlaces.places.push(
                            {
                                "geoRange": {
                                    "coordinate": {
                                        "latitude": app.accessLocationService().get('location').latitude,
                                        "longitude": app.accessLocationService().get('location').longitude
                                    },
                                    "range": 1,
                                    "rangeUnit": app.constants.distanceUnit[app.orgSettings?app.orgSettings.settings.distanceUnit:'km']
                                },
                                "name": "Meine GPS-Position",
                                "defaultPlace": false,
                                // Flag for filtering
                                tempPlace: true
                            }
                        );
                    }
					var filterArray = app.userSettings.settings['bkfilter-'+ (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr)];
					_.each(app.newFilterObj,function(filter){
						filterArray.push(filter);
					});
                    if(!filterArray || filterArray==undefined || filterArray.length==0){
                        filterArray = [];
                        Helper._buildSetMemberSettingsRequest('bkfilter-'+ (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr), filterArray);
                    }
					/*
					 * Build Model
					 */

					app.service.booking.startDate = app.service.booking.bookingData.startDate?(new Date(app.service.booking.bookingData.startDate)):(new Date());
					app.service.booking.endDate = app.service.booking.bookingData.endDate?(new Date(app.service.booking.bookingData.endDate)):(new Date());

					this.model = {
						// Settings
						langGerman: (app.data.langParameters.selectedLang=="de"),
						distanceUnit: app.constants.distanceUnit[app.orgSettings?app.orgSettings.settings.distanceUnit:'km'],
						// Options
						vehicleFilter: this.getVehicleFilterIncludingAllCars(filterArray),
						favPlaces: favPlaces,
						ranges: rangeList,

						// Orgs to search in
						userOrgas: app.userOrgas,
						selectedOrga: "",

						bookingStart: Helper._formatDateString(app.service.booking.startDate,"min"),
						bookingEnd: Helper._formatDateString(app.service.booking.endDate,"min")
					};

					this.trigger('view:initialized');
				}
			},

            getVehicleFilterIncludingAllCars: function(filterArray) {
                var resultFilterArray = new Array();
                _.each(filterArray,function(filter){
				    resultFilterArray.push(filter);
				});
                return this.addAllCarsFilter(resultFilterArray);
            },

            addAllCarsFilter: function(filterArray) {
                filterArray.unshift({NAME:$.t("200-newbooking:txt.allcars"),WTYP:[],WAT:[], UWAT:[], allFilters:true});
                return filterArray;
            },

			render:function(){
				var self = this;
				pcorelib.BaseView.prototype.render.call(this);
				isHomeView = true;
				if(app.embeddedMode) {
					$('#appContent').css('visibility','visible');
				}
				self.renderClocks();
				var filterView = self.renderSelectionView($.t('200-newbooking:lbl.selfilt'),this.model.vehicleFilter);
				self.renderPlaceView($.t('200-newbooking:lbl.selpos'),this.model.favPlaces);
				self.renderRangeView($.t('200-newbooking:lbl.selrange'),this.model.ranges);
				if(self.selectOrgasField) {
					self.renderOrgSelectionView($.t('200-newbooking:lbl.selorg'),this.model.userOrgas.orgas);
				}
				self.validateClocks();
				this.initial = false;
                if(app.embeddedMode){
                    var scrollHeight = document.getElementById('NewBookingView').scrollHeight;
                    $('#NewBookingView').css('height',(scrollHeight+125+50)+'px');
                    parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+125+50), '*');
                }

				 self.listenTo(self, 'orgaChange', function(orga) {
					 // get new orga settings
					 if (orga.nr !== app.userData.member.orgnr) {
	 					app.requestFactory.performRequest('get_org_settings', { orgNr: app.selectedOrga.nr });
	 				}


					// update filter
					var filterArray = app.userSettings.settings['bkfilter-' + orga.nr];
					_.each(app.newFilterObj,function(filter){
						filterArray.push(filter);
					});

					var vehicleFilter = this.getVehicleFilterIncludingAllCars(filterArray);
					self.filterView.updateValues(vehicleFilter);


					var userParamsTypes = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
					userParamsTypes.parameters = {"bookeeOrgNr": orga.nr};

					// update bookee types
					app.requestFactory.performRequest('list_bookee_types', userParamsTypes, new Backbone.Collection(), true);
					this.listenToOnce(app.requestFactory,'list_bookee_types:success',function(response){
							app.vehicleBookeeTypes = response.result;
					});
					this.listenToOnce(app.requestFactory,'list_bookee_types:error',function(response){
							app.vehicleBookeeTypes = null;
					});

					// update bookee attributes
					app.requestFactory.performRequest('list_attributes', userParamsTypes, new Backbone.Collection(), true);
					this.listenToOnce(app.requestFactory,'list_attributes:success',function(response){
							app.vehicleAttributes = response.result;
					});
					this.listenToOnce(app.requestFactory,'list_attributes:error',function(response){
							app.vehicleAttributes = null;
					});


				});


			},

			renderPlaceView: function(title,values) {
				var self = this;
				var sv = new (SelectionView.extend({
					_templateFactory: app.templateFactory,
					model: {
						title: title,
						values: values,
						addButton: true
					},
					mapValue: function(value) {
						var isSelected = false;
						if(	app.service.booking.bookingData.selectedPlace&&
							app.service.booking.bookingData.selectedPlace.name==value.name) {
							isSelected = true;
						} else {
							isSelected = app.service.booking.bookingData.selectedPlace?false:value.defaultPlace;
						}
						return {
							value: value,
							name: value.name,
							selected: isSelected
						};
					},
					addButton: function() {
						pcorelib.EventChannel.trigger('app:router:navigate',{route:'newPlace/newBooking'});
					},
					valueSelected: function(value) {
						self.model.selectedPlace = value;
						/*
						selectValue = value.geoRange.range;
						if(self.rangeViewPointer!=null) {
							self.rangeViewPointer.render();
						}
						*/
						//$("div.id_100 select").val("val2");
						//app.service.booking.bookingData.selectedRange = value.geoRange.range;

						if(self.rangeViewPointer!=null) {
							self.rangeViewPointer.selectValue(value.geoRange.range);
							//self.rangeViewPointer.render();
						}


					}
				}))();
				this.$el.find('.form-horizontal').append(sv.render().$el);
			},

			rangeViewPointer: null,

			renderRangeView: function(title,values) {
				var self = this;
				var sv = new (SelectionView.extend({
					_templateFactory: app.templateFactory,
					model: {
						id: 'radSelect',
						title: title,
						values: values
					},
					mapValue: function(value) {
						var returnValue;
						var defaultValue=1;
						for (var i=0;i<self.model.favPlaces.length;i++){
							var place = self.model.favPlaces[i];
							if(place.defaultPlace && place.defaultPlace==true){
								returnValue = (place.geoRange && place.geoRange.range) ? place.geoRange.range : defaultValue;
							}
						}
						if(returnValue == null || returnValue == undefined){
							returnValue = defaultValue;
						}
						if(app.service.booking.bookingData.selectedRange) {
							returnValue = app.service.booking.bookingData.selectedRange;
						}
						return {
							value: value,
							name: value + ' ' + app.constants.distanceUnit[app.orgSettings?app.orgSettings.settings.distanceUnit:'km'],
							selected: value==returnValue
						};
					},
					valueSelected: function(value) {
						self.model.selectedRange = value;
					}
				}))();
				this.rangeViewPointer = sv;
				this.$el.find('.form-horizontal').append(sv.render().$el);
			},

			renderOrgSelectionView: function(title,values) {
				var self = this;
				if(values.length > 1){// more than one entry in orgas


					var sv = new (SelectionView.extend({
						_templateFactory: app.templateFactory,
						model: {
							title: title,
							values: values
						},
						mapValue: function(value) {
							var selected = false;
							if(app.selectedOrga){

								if(value.nr == app.selectedOrga.nr){

									selected = true;

								}

							}else{

								if(value.nr == app.userData.member.orgnr) selected = true;

							}
							return {
								value: value,
								name: value.name,
								selected: selected
							};
						},
						valueSelected: function(value) {
							app.selectedOrga = value;
							self.trigger('orgaChange', value);
						}
					}))();
					this.$el.find('.form-horizontal').append(sv.render().$el);

				}
			},

			renderSelectionView: function(title,values) {
				var self = this;
				self.filterView = new (SelectionView.extend({
					_templateFactory: app.templateFactory,
					model: {
						title: title,
						values: values,
						addButton: true
					},
					mapValue: function(value) {
						var isSelected = false;
						if(	app.service.booking.bookingData.selectedFilter&&
							app.service.booking.bookingData.selectedFilter.NAME==value.NAME) {
							isSelected = true;
						} else {
							isSelected = app.service.booking.bookingData.selectedFilter?false:value.DEFAULT==true;
						}
						if(app.startupPlace && app.startupType != 'STADTMOBIL') {
							if(value.NAME=='stadtflitzer_intern' || value.NAME=='stadflitzer_intern' || value.NAME=='Stadtflitzer') {
								app.service.booking.bookingData.selectedFilter = { 'NAME' : 'Stadtflitzer' };
								isSelected = true;
							} else {
								isSelected = false;
							}

						}
						return {
							value: value,
							name: value.NAME,
							selected: isSelected
						};
					},
					addButton: function() {
						pcorelib.EventChannel.trigger('app:router:navigate',{route:'newFilter'});
					},
					valueSelected: function(value) {
						self.model.selectedFilter = value;
					}
				}))();
				this.$el.find('.form-horizontal').append(self.filterView.render().$el);
			},

			renderClocks: function() {
				var self = this;
				this.clock1 = $('#inputBookingStartContainerT').clockpicker({
					donetext: 'Done',
					autoclose: true,
					placement:'bottom',
					afterDone: function() {
						self.validateClocks(true,true,false);
					},
					beforeShow: function() {
						self.validateClocks(null,true,false);
					}
				});
				this.clock2 = $('#inputBookingEndContainerT').clockpicker({
					donetext: 'Done',
					autoclose: true,
					placement:'bottom',
					afterDone: function() {
						self.validateClocks(true,false,true);
					},
					beforeShow: function() {
						self.validateClocks(null,false,true);
					}
				});
				if(this.model.bookingStart) {
					var currentTime = this.model.bookingStart.substring(this.model.bookingStart.length-5);
					$('#inputBookingStartT').val(currentTime);
					var currentTime2 = this.model.bookingEnd.substring(this.model.bookingEnd.length-5);
					$('#inputBookingEndT').val(currentTime2);
				}
				this.dtp1 = $('#inputBookingStartContainer').datetimepicker({
					locale: 'de',
					//minDate: Moment(self.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate(),
                    // bugfix: Date has to be set to beginning of the current Day
                    // otherwise the plugin thinks, the clickable current day is "before" the minDate
                    // because it checks in seconds, not days
                    minDate: new Date().setHours(0,0,0,0),
					maxDate: app.service.booking.getBookingModel().max.inDifferenceFromNow(),
					format: 'L',
					ignoreReadonly: true,
					allowInputToggle: true,
					widgetPositioning:{horizontal: 'left'}
				});
				this.dtp2 = $('#inputBookingEndContainer').datetimepicker({
					locale: 'de',
					//minDate: Moment(self.model.bookingEnd, "DD.MM.YYYY HH:mm").utc().toDate(),
                    minDate: new Date().setHours(0,0,0,0),
					maxDate: app.service.booking.getBookingModel().max.inDifferenceFromNow(),
					ignoreReadonly: true,
					allowInputToggle: true,
					format: 'L', widgetPositioning:{horizontal: 'left'}
				});

				$('#inputBookingStartContainer').data("DateTimePicker").date(Moment(this.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate());
				$('#inputBookingEndContainer').data("DateTimePicker").date(Moment(this.model.bookingEnd, "DD.MM.YYYY HH:mm").utc().toDate());

			},

			buttonClicked: function(event,buttonId) {
				if(buttonId=='searchBtn') {
					app.service.booking.updateBookingData(
						Moment(this.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate(),
						Moment(this.model.bookingEnd, "DD.MM.YYYY HH:mm").utc().toDate(),
						this.model.selectedFilter,
						this.model.selectedPlace,
						this.model.selectedRange,
						null,
						app.selectedOrga
					);
					pcorelib.EventChannel.trigger('app:router:navigate',{route:'bookies',params:{noCache:true}});

				}
				if(buttonId=='addNewPlace') {
					pcorelib.EventChannel.trigger('app:router:navigate',{route:'newPlace'});
				}
				if(buttonId=='showMyBookingsBtn') {
					this.$el.find('#newBookingBtn').removeClass('active');
					pcorelib.EventChannel.trigger('app:router:navigate',{route:'myBookingsView'});
				}
			},

			/*
			 * Change-Listener: Dates
			 */
			onDateChanged: function(ev) {
				if(this.initial) {
					return;
				}
				var self = this;
				var d = new Date()
				var n = d.getTimezoneOffset();
				var bookingModel = app.service.booking.getBookingModel();
				if(ev.currentTarget.id=='inputBookingStartContainer') {
					if($('#inputBookingEndContainer').data("DateTimePicker"))
						$('#inputBookingEndContainer').data("DateTimePicker").minDate(ev.date);
					var timeSTr = Helper._formatDateString(new Date(ev.date.toDate()),"min");
					var newTime = timeSTr.substring(0,timeSTr.length-5) + $('#inputBookingStartT').val();
					this.model.bookingStart = newTime;
				}
				if(ev.currentTarget.id=='inputBookingEndContainer') {
					var timeSTr = Helper._formatDateString(new Date(ev.date.toDate()),"min");
					var newTime = timeSTr.substring(0,timeSTr.length-5) + $('#inputBookingEndT').val();
					this.model.bookingEnd = newTime;

				}
				if(this.initial==false) {
					self.validateClocks();
				}
			},

			/*
			 * Helper
			 */

			isSameDay: function(date1,date2) {
				return (date1.getDate()==date2.getDate())&&
					(date1.getMonth()==date2.getMonth())&&
					(date1.getYear()==date2.getYear());
			},

			utcStartTime: function() {
				var self = this;
				var newStartTime = self.model.bookingStart.substring(0,self.model.bookingStart.length-5) + $('#inputBookingStartT').val();
				var newUtcTimeStart = Moment(newStartTime, "DD.MM.YYYY HH:mm").utc().toDate();
				return newUtcTimeStart;
			},

			utcEndTime: function() {
				var self = this;
				var newStartTime = self.model.bookingEnd.substring(0,self.model.bookingEnd.length-5) + $('#inputBookingEndT').val();
				var newUtcTimeStart = Moment(newStartTime, "DD.MM.YYYY HH:mm").utc().toDate();
				return newUtcTimeStart;
			},

			validateClocks: function(isFloor,starClicked,endClicked) {
				var self = this;
				if (typeof isFloor !== "boolean") {
					isFloor = app.orgSettings.settings.bookingFormStartInterval === "CURRENT" ? true : false;
				}
				// Build new Current-Value
				var utcCurrent = Moment().utc().toDate()
				// Build new Start-Value
				var newUtcTimeStart = self.utcStartTime();
				var newUtcTimeEnd = self.utcEndTime();
				var isToday = self.isSameDay(utcCurrent,newUtcTimeStart);
				var standAndEndSameDay = self.isSameDay(newUtcTimeEnd,newUtcTimeStart);
				// Start-Time in past
				if((isToday||standAndEndSameDay)&&newUtcTimeStart.getTime()<utcCurrent.getTime()) {
					// Update to min Start-Time
					newUtcTimeStart = utcCurrent;
				}
				// End-Time smaller than Start-Time
				if((isToday||standAndEndSameDay)&&newUtcTimeEnd.getTime()<=newUtcTimeStart.getTime()) {
					// Update to min Start-Time
					newUtcTimeEnd = newUtcTimeStart;
				}
				if(!endClicked) {
					self.model.bookingStart = app.service.booking.getDate('start',true,'min',newUtcTimeStart,isFloor);
				}
				self.model.bookingEnd = app.service.booking.getDate('end',true,'min',newUtcTimeEnd,isFloor);
				$('#inputBookingStartT').val(self.model.bookingStart.substring(self.model.bookingStart.length-5));
				$('#inputBookingEndT').val(self.model.bookingEnd.substring(self.model.bookingEnd.length-5));
				self.updateAllowedHoursAndMinutes();

				var aa = self.model.bookingStart.substring(0,self.model.bookingStart.length-5);
				var bb = self.model.bookingEnd.substring(0,self.model.bookingEnd.length-5);
				$('#inputBookingStart').val(aa);
				$('#inputBookingEnd').val(bb);

			},

			updateAllowedHoursAndMinutes: function() {
				var self = this;
				var interval = app.service.booking.getBookingModel().interval.inMinutes();
				var avMinutes = [];
				var startValue = 0;
				for (var i = 0; i < (60/interval); i++) {
					avMinutes.push(startValue+(i*interval));
				}
				var utcTimeStart = Moment(self.model.bookingStart, "DD.MM.YYYY HH:mm").utc().toDate();
				var utcTimeEnd = Moment(self.model.bookingEnd, "DD.MM.YYYY HH:mm").utc().toDate();

				var minDiff = app.service.booking.getBookingModel().min.inMinutes();
				var diff = utcTimeEnd.getTime() - utcTimeStart.getTime();
				var msec = diff;
				var mm = Math.floor(msec / 1000 / 60);

				var utcCurrent = Moment().utc().toDate()
				var utcCurrentHours = utcCurrent.getHours();
				var startIsToday = self.isSameDay(utcCurrent,utcTimeStart);
				var endIsToday = self.isSameDay(utcCurrent,utcTimeEnd);
				var startEndSameDay = self.isSameDay(utcTimeStart,utcTimeEnd);
				var avHoursStart = null;
				if(startIsToday) {
					avHoursStart = []
					for (var i = 0; i < 24; i++) {
						if(i>=utcCurrentHours) {
							avHoursStart.push(i);
						}
					}
				}
				var avHoursEnd = null;
				if(startEndSameDay) {
					avHoursEnd = []
					for (var i = 0; i < 24; i++) {
						if(i>=utcTimeStart.getHours()) {
							avHoursEnd.push(i);
						}
					}
				}
				if(this.clock1!=null&&startIsToday) {
					this.clock1.clockpicker('setAllowedHoursAndMinutes',avHoursStart,avMinutes);
				} else {
					this.clock1.clockpicker('setAllowedHoursAndMinutes',null,avMinutes);
				}
				if(this.clock2!=null) {
					if(endIsToday||startEndSameDay) {
						this.clock2.clockpicker('setAllowedHoursAndMinutes',avHoursEnd,avMinutes);
					} else {
						this.clock2.clockpicker('setAllowedHoursAndMinutes',null,avMinutes);
					}
				}
			}

		});
	});
