/*global EWAAppNeu, Backbone, JST*/

define([
        'jquery',
        'underscore',
        'backbone',
        'templates',
        'pcorelib',
        'helper'
    ],

    function ($, _, Backbone, JST, pcorelib, Helper) {
        'use strict';
        return pcorelib.BaseView.extend({

            hbsTemplate: 'NewFilterView',
            alternateTemplatePath: './scripts/templates/search/',

            events: {
                "click #vehTypesSelect":                "vehicleTypeChanged",
                "click #vehAttrGroup":                  "changeAttribute",
                "click #selectAllVehicles":             "allVehiclesSelected",
                "click #selectAllVehiclesLabel":        "allVehiclesSelected",
                "click #inputStandart":                 "inputStandardSelected",
                "click #inputStandartLabel":            "inputStandardSelected",
                "click #proceedBtn, #proceedSaveBtn":   "buttonClicked"
            },

            build: function() {
                var self = this;
                var i = 2;
                this.listenToOnce(app.requestFactory,'list_bookee_types:success',function(response){
                    app.vehicleBookeeTypes = response.result;
                    --i;
                    self.prepareView(i);

                });
                this.listenToOnce(app.requestFactory,'list_attributes:success',function(response){
                    app.vehicleAttributes = response.result;
                    --i;
                    self.prepareView(i);

                });
                //var fullAuth = jQuery.extend(true, {}, app.data.userParameters, app.data.adminParameters);
                var userParams = JSON.parse(JSON.stringify(app.auth.fullAuthentication));
                userParams.parameters = {"bookeeOrgNr": (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr)};
                app.requestFactory.performRequest('list_bookee_types', userParams, new Backbone.Collection());

                app.requestFactory.performRequest('list_attributes', userParams, new Backbone.Collection());
            },

            prepareView: function(i){
                if(i == 0){

                    var vehicleClassesList = app.vehicleBookeeTypes.types;
                    var vehicleAttributes= app.vehicleAttributes.attributes;


                    this.model = {
                        langGerman: (app.data.langParameters.selectedLang == "de"),
                        vehicleTypes:vehicleClassesList,
                        vehicleAttributes: vehicleAttributes,
                        allVehiclesChecked:true,
                        asStandard:false
                    };

                    this.trigger('view:initialized');
                }
            },

	        render: function () {
	            pcorelib.BaseView.prototype.render.call(this);
	
	            if(app.embeddedMode){
	                var scrollHeight = document.getElementById('formNewFilterView').scrollHeight;
	                $('#formNewFilterView').css('height',(scrollHeight+50)+'px');
	                parent.postMessage('CANTAMEN-FRAME-LOADED:'+(scrollHeight+125+50), '*');
		            pcorelib.EventChannel.trigger('app:busy:false');
		        }
	
	            return this;
	        },

            buttonClicked: function(event) {
                var buttonId = event.target.id;

                if (buttonId == 'proceedBtn') {
                    var temporary = true;
                    var newFilter = this.getParametersForSave(temporary);
                    if (newFilter.NAME == "") {
                        newFilter.NAME = $.t('546-settings-filter-edit:txt.defaultName');
                    }
                    app.newFilterObj.push(newFilter);
                    app.service.booking.bookingData.selectedFilter = newFilter;
                    pcorelib.EventChannel.trigger('app:router:navigate', {route:'newBooking'});

                } else if (buttonId == 'proceedSaveBtn') {
                    var temporary = false;
                    var filters = this.getFiltersForSave(temporary);
                    for (var i = filters.length -1; i >= 0; i--) {
                        if(filters[i].tempFilter && filters[i].tempFilter == true){
                            filters.splice(i,1);
                        };
                    }
                    this.listenToOnce(app.requestFactory, 'set_member_settings:success', function (response){
                        pcorelib.EventChannel.trigger('app:router:navigate', {route:'newBooking'});
                    });
                    this.listenToOnce(app.requestFactory,'set_member_settings:error', app.applicationRouter.errorHandler);
                    Helper._buildSetMemberSettingsRequest('bkfilter-'+ (app.selectedOrga ? app.selectedOrga.nr : app.data.userParameters.orgNr), filters);
                }
            },

            vehicleTypeChanged:function(event){
                if(this.model.allVehiclesChecked){
                    return false;
                }
            },

            allVehiclesSelected:function(event){
                this.model.allVehiclesChecked = !this.model.allVehiclesChecked;
                this.setVehicleTypesSelected(this.model.allVehiclesChecked);
            },

            inputStandardSelected:function(event){
                this.model.asStandard = !this.model.asStandard;
                $("#inputStandart").prop("checked",this.model.asStandard);
            },

            changeAttribute:function(event){
                var buttonId = event.target.id;
                $("#"+buttonId).addClass('active').siblings().removeClass('active');
            },

            setVehicleTypesSelected: function(selected){
                for(var i = 0; i < this.model.vehicleTypes.length;i++){
                    var typeObj = this.model.vehicleTypes[i];
                    $("#selectAllVehicles").prop("checked",selected);
                    if(selected){
                        $("#vehTypesSelect li").addClass("disabled");
                        $("#vehTypesSelect #type-"+typeObj.nr).prop("checked", true);
                    } else {
                        $("#vehTypesSelect li").removeClass("disabled");
                        $("#vehTypesSelect #type-"+typeObj.nr).prop("checked", false);
                    }
                }
            },

            getParametersForSave: function(temp){
                var newFilter = {};
                var name = $("#inputSaveAs").val();

                newFilter.NAME = name;
                if(this.model.asStandard){
                    newFilter.DEFAULT = this.model.asStandard;
                }
                if(this.getSelectedWTypes()!=[]){
                    newFilter.WTYP = this.getSelectedWTypes();
                }
                if(this.getSelectedWATs(true)!=[]){
                    newFilter.WAT = this.getSelectedWATs(true);
                }
                if(this.getSelectedWATs(false)!=[]){
                    newFilter.UWAT = this.getSelectedWATs(false);
                }
                if (temp) {
                    newFilter.tempFilter = true;
                }
                return newFilter;
            },

            getFiltersForSave: function (temp) {
                var filterArray = app.userSettings.settings['bkfilter-'+ (app.selectedOrga ? app.selectedOrga.nr : app.data.userParameters.orgNr)] || [];
                var newFilter = this.getParametersForSave(temp);
                if (newFilter.NAME == "") {
                    newFilter.NAME = $.t('546-settings-filter-edit:txt.defaultName');
                }
                filterArray.push(newFilter);
                return filterArray;
            },

            getSelectedWTypes: function(){
                var selectedTypes=[];
                if(!this.model.allVehiclesChecked){
                    for(var i = 0; i< this.model.vehicleTypes.length; i++){
                        var typeObj = this.model.vehicleTypes[i];
                        if($("#vehTypesSelect #type-"+typeObj.nr).prop("checked")){
                            selectedTypes.push(typeObj.nr);
                        }
                    }
                }
                return selectedTypes;
            },

            getSelectedWATs: function(selected){
                var selectedAttrs=[];
                for(var i = 0; i < this.model.vehicleAttributes.length; i++){
                    var attrObj = this.model.vehicleAttributes[i];
                    if(selected){
                        var attrYes=$("#vehAttrGroup #attribute-"+attrObj.nr+"-yes");
                        if(attrYes.hasClass("active")){
                            selectedAttrs.push(attrObj.nr);
                        }
                    }
                    else{
                        var attrNo=$("#vehAttrGroup #attribute-"+attrObj.nr+"-no");
                        if(attrNo.hasClass("active")){
                            selectedAttrs.push(attrObj.nr);
                        }
                    }
                }
                return selectedAttrs;
            }
        });
    });
