/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'moment'
], function ($, _, Backbone, JST, pcorelib, Moment) {
    'use strict';
    return pcorelib.GoogleMapView.extend({

		addClasses: 'app_content_nopadding',
		addClassesSelector: '#appContent',
        currentLocation: {
            "latitude": 0,
            "longitude": 0
        },

        placesCollection: null,

        range: {
            "coordinate": {
                "latitude": 0,
                "longitude": 0
            },
            "range": 1,
            "rangeUnit":"KILOMETERS"
        },

        build: function() {
            this.mapAttributes = {
                zoom: 15,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                center: new google.maps.LatLng(app.accessLocationService().get('location').latitude, app.accessLocationService().get('location').longitude)
            };
            this.trigger('view:initialized');
        },

        render:function(){
            this.height = app.applicationView.getContentHeight()+'px';
            pcorelib.GoogleMapView.prototype.render.call(this);
        },

        loadData: function() {
            var self = this;
            pcorelib.EventChannel.trigger('app:busy:true');
				
				self.loadDataMapCB = function(location,collection){
                if(collection) {
                    self.placesCollection = collection;
                }

                self.currentLocation.latitude = location.latitude;
                self.currentLocation.longitude = location.longitude;
                self.map.setCenter(new google.maps.LatLng(self.currentLocation.latitude, self.currentLocation.longitude));
				
                self.updateMap();
					 pcorelib.EventChannel.trigger('app:busy:false');
            }
				
				// Wenn nicht in eigener Orga, die Karte initial auf die Default-Koordianten der anderen Orga setzen.
				if (app.selectedOrga &&
						app.selectedOrga.nr &&
						app.selectedOrga.nr !== app.userData.member.orgnr && 
						app.selectedOrga.settings &&
						app.selectedOrga.settings.defaultSearchRange
					) {
						app.requestFactory.requestPlacesForPosition({
							 "coordinate": {
								  "latitude": app.selectedOrga.settings.defaultSearchRange.coordinate.latitude,
								  "longitude": app.selectedOrga.settings.defaultSearchRange.coordinate.longitude
							 },
							 "range": 1,
							 "rangeUnit":"KILOMETERS"
						},self.loadDataMapCB);
				} else {
					// Eigene Orga: Aktuelle Position oder Default-Koordianten.
					app.accessLocationService().updateLocation(function(location){
						 app.requestFactory.requestPlacesForPosition({
							  "coordinate": {
									"latitude": location.latitude,
									"longitude": location.longitude
							  },
							  "range": 1,
							  "rangeUnit":"KILOMETERS"
						 },self.loadDataMapCB);
					});
				}
				

        },

        updateMapOverlay: function(mapView) {
            mapView.addCenterMarker('http://www.daftlogic.com/images/cross-hairs.gif',{coords:[0,0,0,0],type:'rect'});
            mapView.addPositionMarker(this.currentLocation.latitude, this.currentLocation.longitude);
            if(mapView.placesCollection) {
                var stationImage;
                var floaterImage;
                if(app.orgSettings.settings && app.orgSettings.settings.geoMarkers && app.orgSettings.settings.geoMarkers.length > 0){
                    _.each(app.orgSettings.settings.geoMarkers, function(marker){
                        _.each(marker.usage,function(usage){
                          if (usage=="FIXED_STATION"){
                              stationImage = {
                                  url: marker.imageUrl,
                                  //scaledSize: new google.maps.Size(21, 34),
                                  //size: new google.maps.Size(44, 74),
                                  anchor: new google.maps.Point(marker.anchorRight,marker.anchorBottom)
                              };
                          } else if(usage=="FREE_FLOAT"){
                              floaterImage = {
                                  url: marker.imageUrl,
                                  //scaledSize: new google.maps.Size(21, 34),
                                  //size: new google.maps.Size(44, 74),
                                  anchor: new google.maps.Point(marker.anchorRight,marker.anchorBottom)
                              };
                          }
                       });
                    });
                } else {
                    stationImage = {
                        url: "images/icon_place.png",
                        scaledSize: new google.maps.Size(21, 34),
                        size: new google.maps.Size(44, 74),
                        anchor: new google.maps.Point(10,33)
                    };
                    floaterImage = {
                        url: "images/icon_place_stadtflitzer.png",
                        scaledSize: new google.maps.Size(21, 34),
                        size: new google.maps.Size(44, 74),
                        anchor: new google.maps.Point(10,33)
                    };
                }
                mapView.placesCollection.each(function(place){
                    if(place.attributes.isPosition) {
                        var pinImage = floaterImage;
                    } else {
                        var pinImage = stationImage;
                    }
                    mapView.markerMap['place_'+place.get('nr')] = new google.maps.Marker({
                        position: new google.maps.LatLng(place.get('latitude'),place.get('longitude')),
                        icon: pinImage
                    });
                    mapView.markerMap['place_'+place.get('nr')].place = place;
                });
            }
        },

        buttonClickListener: function(mapView,marker,ev) {
            /* Old code from previous behaviour (directing to an availability check for the selected place)

            var nr = marker.name.substring(6);
			var currentPlace = mapView.placesCollection.find(function(item){
				return item.get('nr') == nr;
			});
            var self = this;
			var baseDate = new Date();
			app.service.booking.updateBookingData(
				Moment(baseDate, "DD.MM.YYYY HH:mm").utc().toDate(),
				Moment(baseDate, "DD.MM.YYYY HH:mm").utc().toDate(),
				null,
				null,
				null,
				{
					placeNr: currentPlace.get('nr'),
					latitude: currentPlace.get('latitude'),
					longitude: currentPlace.get('longitude')
				}
			);
			app.requestFactory.overrideParams['list_available_bookees'] = null;*/

            var nr = marker.name.substring(6);
            var currentPlace = mapView.placesCollection.find(function(item) {
                return item.get('nr') == nr;
            });
            var selectedPlace = {
                "geoRange": {
                    "coordinate": {
                        "latitude": currentPlace.get('latitude'),
                        "longitude": currentPlace.get('longitude')
                    },
                    "range": 1,
                    "rangeUnit": app.constants.distanceUnit[app.orgSettings.settings.distanceUnit]
                },
                "name": currentPlace.get('name'),
                "defaultPlace": false,
                // Flag for filtering
                tempPlace: true
            };
            app.userNewFavPlaces.push(selectedPlace);
            app.service.booking.bookingData.selectedPlace = selectedPlace;
				// set orga for selected Place
				app.selectedOrga.nr = currentPlace.get('orgnr');
            pcorelib.EventChannel.trigger('app:router:navigate',{route:'newBooking'});
        },

        markerClickListener: function(mapView,marker) {
            mapView.showInfoWindow(marker,new (pcorelib.BaseView.extend({
                //hbsStringTemplate: '<div>'+marker.place.get('name')+'<button class="showStreetViewButton" type="button"></button>'+'</div>'
                hbsStringTemplate: '<div>'+marker.place.get('name')+'<br><button id="avButton" class="avButton highlightBtn btn btn-default" style="margin-top: 8px;" type="button">Verfügbarkeit prüfen</button></div>'
            }))());
        },

        dragListener: function(mapView) {
            this.updatePosition(mapView);
        },

        zoomListener: function(mapView){
            var coords = this.getSelectedPosition();
            var bounds = this.map.getBounds();
            var distance = google.maps.geometry.spherical.computeDistanceBetween(coords,bounds.getNorthEast());

            this.range.range = distance / 1000; // Meter noch nicht implementiert auf dem server
            this.updatePosition(mapView);
        },

        updatePosition: function(mapView){
            var coords = this.getSelectedPosition();
            this.range.coordinate.latitude = coords.lat();
            this.range.coordinate.longitude = coords.lng();
            pcorelib.EventChannel.trigger('app:busy:true');
            app.requestFactory.updatePositionAndRequestPlaces(function(location,collection){
                pcorelib.EventChannel.trigger('app:busy:false');
                if(collection) {
                    mapView.placesCollection = collection;
                }
                mapView.updateMap();
            },this.range);
        }

    });
});
