/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/InfoPanel',
    'moment'
], function ($, _, Backbone, JST, pcorelib, InfoPanel, moment) {
    'use strict';
    return pcorelib.BaseView.extend({

        hbsTemplate: 'FilterEditView',
        alternateTemplatePath: './scripts/templates/settings/',
        id: 'FilterEditView',
        events: {
        	"click #confirmChangeBtn":  "confirmChange",
            "click #cancelChangeBtn":  "cancelChange",
            "click .vehAttributesGroupClick": "changeAttribute",
            "click #selectAllVehicles": "selectAllAttributes"
        },

        changeAttribute: function(ev) {
        	var filter = $('.vehAttrGroup')[this.model.index];
            var target = ev.target.id;
            $($(filter).find("#"+target)).addClass('active').siblings().removeClass('active');
        },

        selectAllAttributes: function(event){
			var selected = null;
			var attributes = $($('.allVehiclesSelected')[this.model.index]);
			var allBoxes = $($('.vehTypesSelect')[this.model.index]).find('.vehTypeOption');

        	if(attributes.prop('checked')) {
        		attributes.prop('checked', false);
        		selected = false;
        	} else {
        		attributes.prop('checked', true);
        		selected = true;
        	}

            _.each(allBoxes, function (box) {
            	$(box).prop('checked',selected);
            });
        },

        confirmChange: function(event) {
            var self = this;
            event.preventDefault();
            var index = self.model.index;
            var filters = app.userSettings.settings['bkfilter-'+ (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr)];
            
           	this.setFilterAttributes(filters, index);

            pcorelib.EventChannel.trigger("filter:changed:" + index, index, filters);
        },

        // updates filter attributes depending on checkboxes
        setFilterAttributes: function(filters, index) {
        	var self = this;
			var typesHTML = $('.bookee_list_item_content')[index];
            var types = [];
            var yesAttr = [];
            var noAttr = [];

            // types
            _.each($(typesHTML).find('.vehTypeOption'), function (box) {
            	if($(box).prop('checked')) {
            		types.push($(box).prop('value'));
           		}
           		filters[index].WTYP = types;
            });

            // attributes
            _.each($(typesHTML).find('.yes'), function (box) {
            	if($(box).hasClass('active')) {
            		yesAttr.push($(box).prop('value'));
           		}
           		filters[index].WAT = yesAttr;
            });
            _.each($(typesHTML).find('.no'), function (box) {
            	if($(box).hasClass('active')) {
            		noAttr.push($(box).prop('value'));
           		}
           		filters[index].UWAT = noAttr;
            });
            
            // name
            if($("#inputFiltername-" + self.model.index).val() != "") {
                filters[index].NAME = $("#inputFiltername-" + index).val();
            } else {
                filters[index].NAME = $.t('546-settings-filter-edit:txt.defaultName');
            };

            // default
            _.each($('.defaultCb'), function (cb, index){
                if($(cb).is(":checked")) {
                    filters[index].DEFAULT = true;
                } else {
                    filters[index].DEFAULT = false;
                };
        	});

        	if(filters[index].WTYP != [] || filters[index].WAT != [] || filters[index].UWAT != []) {
        		filters[index].allFilters = undefined;
        	}
        },

        cancelChange: function(ev) {
            var myRegExp = /\d/g;
            var classes = $(ev.currentTarget).attr('class');
            var index = classes.match(myRegExp).join('');
            
            if(this.model.toggleMode) {
                $('.panel-body').eq(index).toggle();
            }
        }
    });
});
