/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/InfoPanel',
    './FilterEditView',
    'moment',
    'helper'
], function ($, _, Backbone, JST, pcorelib, InfoPanel, FilterEditView, moment, Helper) {
    'use strict';
    return pcorelib.BaseView.extend({

        hbsTemplate: 'SettingsFilterView',
        alternateTemplatePath: './scripts/templates/settings/',
        id: 'SettingsFilterView',
        events:{
            "click #addFilterBtn":  "addFilter"
        },

        build: function() {
            var self = this;
            this.trigger('view:initialized');
        },

        render: function () {
            if(app.embeddedMode) {
                $('#appContent').css('visibility','inherit');
            }
            pcorelib.BaseView.prototype.render.call(this);

            var self = this;
            var index = 0;

            var filters = app.userSettings.settings['bkfilter-'+ (app.selectedOrga && app.selectedOrga.nr ? app.selectedOrga.nr : app.data.userParameters.orgNr)];
            for (var i = filters.length -1; i >= 0; i--) {
                if(filters[i].tempFilter && filters[i].tempFilter == true){
                    filters.splice(i,1);
                };
            }
            _.each(filters, function (filter) {
                if(filter){
                    self.renderFilter(index,filter);
                    index++;
                }
            });

            if(index == 0){
                $("#noFilterText").removeClass("hidden");
            }
            $('#showMyPlacesBtn').removeClass('active');
            $('#showMyFiltersBtn').addClass('active');
            return this;
        },

        renderFilter: function(index, filter) {
            var self = this;
            var name = "";
            if(filter && filter.NAME) {
                name = filter.NAME;
            }
            var panel = new InfoPanel({
                model: {
                    index: index,
                    hasAction:true,
                    panelTitle: name,
                    iconColor: ewaIconColor,
                    iconClasses: 'glyphicon glyphicon-trash',
                    filter: filter,
                    toggleMode: true,
                    toggleClosed: true,
                    vehicleTypes: app.vehicleBookeeTypes.types,
                    vehicleAttributes: app.vehicleAttributes.attributes
                },
                itemClicked: function (ev, value) {
                    //Set only one default place in the list
                    if(ev.target.id == "defaultCb-" + index) {
                        $('.defaultCb').each(function (index,item) {
                            if(this.id != ev.target.id){
                                if($(this).is(":checked")){
                                    $(this).prop('checked', false);
                                }
                            }
                        });
                    }
                },
                iconClicked:function (){
                    $('#confirmDeleteFilterModal').modal('show');
                    $('#modalDeleteFilterConfirm').off('click');
                    $('#modalDeleteFilterConfirm').on('click', function(){
                        self.setSettings(app.userSettings.settings['bkfilter-'+ (app.selectedOrga ? app.selectedOrga.nr : app.data.userParameters.orgNr)], true, index);
                    });
                }
            });

            this.checkAttributes(panel.model.index, panel.model.filter, panel.model.filter.WAT, panel.model.filter.UWAT);

            panel.contentView = new FilterEditView({model: panel.model});
            panel.renderTo(self.$el.find('#item_list'));

            pcorelib.EventChannel.on("filter:changed:" + index, function (index, filters) {
                self.setSettings(filters);
            });
        },

        // checks filters' attributes to display their current values
        checkAttributes: function(indexParam, filterParam, yesAttrParam, noAttrParam) {
            var self = this;
            setTimeout(function() {
                var index = indexParam;
                var yesAttr = yesAttrParam;
                var noAttr = noAttrParam;
                var types = filterParam.WTYP;
                var typesHTML = self.$el.find('.bookee_list_item_content')[index];
                var allFilters = filterParam.allFilters;

                if(allFilters === true) {
                	// check all attributes
                	$(typesHTML).find('#vehAttrGroup').find('.yes').addClass('active').siblings().removeClass('active');
                	// check all types
                	$(typesHTML).find('#vehTypesSelect').find('input').prop('checked', true);
                } else {
                	// check attributes with yes
                	_.each(yesAttr, function(yesAttr) {
                		$(typesHTML).find('#attribute-'+yesAttr).find('.yes').addClass('active').siblings().removeClass('active');
                	});
                	// check attributes with no
                	_.each(noAttr, function(noAttr) {
                		$(typesHTML).find('#attribute-'+noAttr).find('.no').addClass('active').siblings().removeClass('active');
                	});

	                // check types
	                _.each(types, function (type) {
	              		$(typesHTML).find('.type-'+type).prop('checked', true);
	                });
                }
            }, 200);
        },

        setSettings: function(filters, del, index){
            var self = this;
            var filtersArray = filters;
            if(del) {
            	filtersArray.splice(index, 1);
            }
            app.requestFactory.listenToOnce(app.requestFactory, 'set_member_settings:success', function(response) {
                self.render();
            });
            app.requestFactory.listenToOnce(app.requestFactory, 'set_member_settings:error', function(response) {
                pcorelib.EventChannel.trigger('error', $.t('999-err-msg:code.'+response.responseJSON.errorCode));
            });

            Helper._buildSetMemberSettingsRequest('bkfilter-'+ (app.selectedOrga ? app.selectedOrga.nr : app.data.userParameters.orgNr), filtersArray);
        },

        addFilter:function(){
            pcorelib.EventChannel.trigger('app:router:navigate', {route:'newFilter'});
        }

    });
});
