/*global define*/

define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'pcorelib',
    'components/InfoPanel',
    './PlaceEditView',
    'moment'
], function ($, _, Backbone, JST, pcorelib, InfoPanel, PlaceEditView, Moment) {
    'use strict';
    return pcorelib.GoogleMapView.extend({

        hbsTemplate: 'SettingsPlacesView',
        alternateTemplatePath: './scripts/templates/settings/',
        id: 'SettingsPlacesView',
        events:{
            "click #addPlaceBtn":  "addPlace"
        },

        build: function() {
            var self = this;
            this.trigger('view:initialized');
        },

        renderPlace: function(index, place) {
            var self = this;
            var name = "";
            var address = "";
            var latlng = {
                lat: place.geoRange.coordinate.latitude,
                lng: place.geoRange.coordinate.longitude
            };
            
            if(place && place.name) {
                name = place.name;
            }
            var range = 1;
            if(place && place.geoRange && place.geoRange.range){
                range = place.geoRange.range;
            }
            if(self.geocoder == null) {
                self.geocoder = new google.maps.Geocoder();
            }
            self.geocoder.geocode({'location': latlng}, function(results, status) {              
                if (status == google.maps.GeocoderStatus.OK) {
                    self.parseReverseGeocodingResponse(results);
                    panel.model.addressname = results[0].formatted_address;
                }
                pcorelib.EventChannel.trigger('geocoding:address:'+index);
            });

            var lines = [];
            var lineIndex = 0;
            var panel = new InfoPanel({
                model: {
                    index: index,
                    hasAction:true,
                    panelTitle: name,
                    lines: lines,
                    iconColor: ewaIconColor,
                    iconClasses: 'glyphicon glyphicon-trash',
                    toggleMode: true,
                    toggleClosed: true,
                    place:place,
                    selectedRange:range,
                    distanceUnit: app.constants.distanceUnit[app.orgSettings.settings.distanceUnit]
                },
                itemClicked: function (ev, value) {
                    //Set only one default place in the list
                    if(ev.target.id == "defaultCb-" + index) {
                        $('.defaultCb').each(function (index, item) {
                            if(this.id != ev.target.id){
                                if($(this).is(":checked")){
                                    $(this).prop('checked', false);
                                }
                            }
                        });
                    }
                },
                iconClicked:function (){
                    var localSelf = this;
                    $('#confirmDeletePlaceModal').modal('show');
                    $('#modalDeletePlaceConfirm').off('click');
                    $('#modalDeletePlaceConfirm').on('click',function(){
                        self.setPlaces(app.userFavPlaces.places, true, index);
                    });
                }
            });

            panel.model.ranges = app.data.rangeParameters;
            panel.model.name = name;

            panel.contentView = new PlaceEditView({model: panel.model});
            panel.renderTo(self.$el.find('#item_list'));

            pcorelib.EventChannel.on('geocoding:address:'+index, function() {
                if(panel.model.addressname == "") {
                    $($('.addressname')[index]).empty();
                } else {
                    $($('.addressname')[index]).empty();
                    $($('.addressname')[index]).html(panel.model.addressname); 
                }
            });

            //Place data is changed
            pcorelib.EventChannel.on("place:changed:" + index, function(index, places){
                self.setPlaces(places, false, index);
            });
        },

        addPlace: function(){
            pcorelib.EventChannel.trigger('app:router:navigate', {route: 'newPlace/settings'});
        },

        setPlaces:function(places, del, index){
            var self = this;
            var placesArray = places;
            if(del) {
                placesArray.splice(index, 1);
            }
            // iterating over places to filter out the temporary ones that aren't supposed to be sent to the server
            for (var i = placesArray.length -1; i >= 0; i--) {
                if(placesArray[i].tempPlace && placesArray[i].tempPlace == true){
                    placesArray.splice(i,1);
                }
            }
            this.listenToOnce(app.requestFactory,'set_favorite_places_of_member:success', function(response){
                self.render();
            });
            this.listenToOnce(app.requestFactory,'set_favorite_places_of_member:error', function(response){
                pcorelib.EventChannel.trigger('error', response.responseJSON.errorCode);
            });

            app.requestFactory.performRequest('set_favorite_places_of_member', {
                "places": placesArray
            });
        },

        render: function () {
			if(app.embeddedMode) {
				$('#appContent').css('visibility','inherit');
			}
            pcorelib.BaseView.prototype.render.call(this);
            var self = this;

            var index = 0;
            _.each(app.userFavPlaces.places, function (place) {
                if(place && (place.tempPlace == undefined || !place.tempPlace) ){
                    self.renderPlace(index,place);
                    index++;
                }
            });
            if(index == 0){
                $("#noPlacesText").removeClass("hidden");
            }
            $('#showMyPlacesBtn').addClass('active');
            $('#showMyFiltersBtn').removeClass('active');
            return this;
        }
    });
});
